@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dfile.c,v 1.15 96/05/13 08:03:59 abe Exp $";
+static char *rcsid = "$Id: dfile.c,v 1.1 97/02/24 07:30:46 abe Exp $";
 #endif
 
 
@@ -58,7 +58,7 @@ is_file_named(p, ty, ch, ic)
  */
 	for (f = 0, s = Sfile; s; s = s->next) {
 
-#if	_AIXV>=4140
+#if	AIXV>=4140
 	/*
 	 * Check for a clone match.
 	 */
@@ -70,7 +70,7 @@ is_file_named(p, ty, ch, ic)
 				f = 3;
 				break;
 		}
-#endif	/* _AIXV>=4140 */
+#endif	/* AIXV>=4140 */
 
 	/*
 	 * Search by path name, as requested.
@@ -126,14 +126,14 @@ is_file_named(p, ty, ch, ic)
 		if (f == 2) {
 			(void) strcpy(Namech, p);
 
-#if	_AIXV>=4140
+#if	AIXV>=4140
 		} else if (f == 3 && ClonePtc >= 0
 		       &&  major(Lf->dev) == ClonePtc)
 		{
 			(void) strcpy(Namech, s->name);
 			(void) sprintf(endnm(), "/%d", minor(Lf->dev));
 
-#endif	/* _AIXV>=4140 */
+#endif	/* AIXV>=4140 */
 
 		} else {
 			(void) strcpy(Namech, s->name);
@@ -160,17 +160,33 @@ print_file()
 	int mjd, mnd;
 
 	if (Hdr == 0) {
-	    (void) printf("COMMAND     PID%s     USER   FD   TYPE     DEVICE ",
-		Fpgrp ? "   PGRP" : "");
+	    (void) printf(
+		"COMMAND     PID%s%s     USER   FD   TYPE     DEVICE ",
+		Fpgrp ? "   PGRP" : "",
+
+#if	defined(HASPPID)
+		Fppid ? "   PPID" : ""
+#else	/* !defined(HSPPID) */
+		""
+#endif	/* defined(HASPPID) */
+
+		);
 	    (void) printf("%10s", Foffset ? "OFFSET"
 					  : Fsize ? "SIZE"
 						  : "SIZE/OFF");
 	    (void) puts("      INODE NAME");
 	    Hdr++;
 	}
-	(void) printf("%-9.9s%6d", Lp->cmd, Lp->pid);
+	(void) printf("%-*.*s%6d", CMDL, CMDL,
+	    Lp->cmd ? Lp->cmd : "(unknown)", Lp->pid);
 	if (Fpgrp)
 	    printf(" %6d", Lp->pgrp);
+
+#if	defined(HASPPID)
+	if (Fppid)
+	    printf(" %6d", Lp->ppid);
+#endif	/* defined(HASPPID) */
+
 	(void) printf(" %8.8s %4s%c%c %4.4s ",
 			printuid((UID_ARG)Lp->uid, NULL),
 			Lf->fd,
@@ -182,11 +198,11 @@ print_file()
 		mjd = major(Lf->dev);
 		mnd = minor(Lf->dev);
 
-#if	_AIXV>=3200
+#if	AIXV>=3200
 		if (Lf->dev & SDEV_REMOTE)
 		    (void) fputs("  NFS ", stdout);
 		else
-#endif	/* _AIXV>=3200 */
+#endif	/* AIXV>=3200 */
 		{ 
 		    if (mjd < 10000)
 			(void) printf(" %4d,", mjd);
@@ -229,12 +245,3 @@ print_file()
 	}
 	printname(1);
 }
-
-
-/*
- * The ck_file_arg() function is obtained from ../common/ckfa.frag.
- * The process_file() function is obtained from ../common/prfp.frag.
- */
-
-/* CKFA_EXPDEV is not defined. */
-#define	CKFA_MPXCHAN	1
