@@ -30,6 +30,11 @@
  */
 
 
+/*
+ * $Id: dlsof.h,v 1.4 97/08/15 08:16:36 abe Exp $
+ */
+
+
 #if	!defined(AIX_LSOF_H)
 #define	AIX_LSOF_H	1
 
@@ -38,16 +43,17 @@
 #include <dirent.h>
 #include <setjmp.h>
 #include <unistd.h>
-#include <sys/domain.h>
 
 # if	!defined(_KERNEL)
 #define _KERNEL	1
 # endif
 
+#include <sys/vnode.h>
 #include <sys/file.h>
 #include <procinfo.h>
 #define p_pid	pi_pid
 #define	p_pgrp	pi_pgrp
+#define	p_ppid	pi_ppid
 #define p_stat	pi_stat
 #define p_uid	pi_uid
 #undef	sleep
@@ -65,18 +71,24 @@
 #include <net/route.h>
 #include <net/raw_cb.h>
 
-# if	_AIXV>=4100
+# if	AIXV>=4100
 #include <netinet/ip.h>
-# endif	/* _AIXV>=4100 */
+# endif	/* AIXV>=4100 */
 
 #include <netinet/in_pcb.h>
+#include <netinet/ip_var.h>
+#include <netinet/tcp.h>
+#include <netinet/tcpip.h>
+#include <netinet/tcp_fsm.h>
+#include <netinet/tcp_timer.h>
+#include <netinet/tcp_var.h>
 #include <sys/vattr.h>
+#include <sys/statfs.h>
 #include <sys/vfs.h>
 #include <sys/vmount.h>
-#include <sys/vnode.h>
 
-# if	_AIXV>=4100
-#  if	_AIXV>=4110
+# if	AIXV>=4100
+#  if	AIXV>=4110
 #undef	u
 #undef	u_comm
 #undef	u_cdir
@@ -84,7 +96,7 @@
 #undef	u_maxofile
 #undef	u_ufd
 #undef	u_loader
-#  endif	/* _AIXV>=4110 */
+#  endif	/* AIXV>=4110 */
 
 #define	u_comm		U_comm
 #define	u_cdir		U_cdir
@@ -92,13 +104,15 @@
 #define	u_maxofile	U_maxofile
 #define	u_ufd		U_ufd
 #define	u_loader	U_loader
-# endif	/* _AIXV>=4100 */
+# endif	/* AIXV>=4100 */
 
+# if	defined(HAS_NFS)
 #include <nfs/rnode.h>
+# endif	/* defined(HAS_NFS) */
 
-#  if	_AIXV>=3200
+#  if	AIXV>=3200
 #include <sys/specnode.h>
-#  endif	/* _AIXV>=3200 */
+#  endif	/* AIXV>=3200 */
 
 /*
  * AIX doesn't supply cdrnode.h.
@@ -128,23 +142,61 @@ struct le {
 	uint dummy3;
 	struct file *fp;		/* file table entry pointer */
 
-# if	_AIXV>=4110
+# if	AIXV>=4110
 	int ft;				/* file type indicator */
 	unsigned dummy4;
 	char *dummy5;
 	unsigned dummy6;
 	char *dummy7[3];
 	char *nm;			/* name */
-# endif	/* _AIXV>=4110 */
+# endif	/* AIXV>=4110 */
 
 };
 
+# if	defined(HAS_AFS)
+#include <afs/stds.h>
+#include <afs/param.h>
+#include <afs/afsint.h>
+#include <afs/vldbint.h>
+
+
+/*
+ * This is an emulation of the afs_rwlock_t definition that appears in
+ * the AFS sources in afs/lock.h.
+ */
+
+struct afs_lock {
+
+#  if	HAS_AFS<304
+    unsigned long d1[4];
+#  else	/* HAS_AFS>=304 */
+    unsigned long d1[6];
+#  endif	/* HAS_AFS<304 */
+
+};
+typedef struct afs_lock afs_lock_t;
+typedef struct afs_lock afs_rwlock_t;
+
+#define	KERNEL
+#include <afs/afs.h>
+#undef	KERNEL
+# endif	/* defined(HAS_AFS) */
+
 
 /*
  * Miscellaneous definitions.
  */
 
-#define	COMP_P		void
+# if	defined(HAS_AFS)
+#define	AFSAPATHDEF	"/usr/vice/etc/dkload/???"
+#define	AFSDEV		1	/* AFS "fake" device number */
+
+#  if	!defined(MNT_AFS)
+#define	MNT_AFS		AFS_MOUNT_AFS
+#  endif	/* !defined(MNT_AFS) */
+# endif	/* defined(HAS_AFS) */
+
+#define	COMP_P		const void
 #define DEVINCR		1024	/* device table malloc() increment */
 #define	KA_T		off_t
 #define KMEM		"/dev/kmem"
@@ -153,13 +205,13 @@ struct le {
 #define MALLOC_S	unsigned
 #define	N_UNIX		"/unix"
 #define PROCSIZE	sizeof(struct procinfo)
-#define QSORT_P		char
+#define QSORT_P		void
 #define	READLEN_T	unsigned
 #define STRNCPY_L	size_t
 #define U_SIZE		sizeof(struct user)
 
 
-# if	_AIXV>=4110
+# if	AIXV>=4110
 /*
  * readx() definitions for AIX 4.1.1 and above.
  */
@@ -170,7 +222,60 @@ struct le {
 #define	LIBNMLN		40			/* maximum library table name
 						 * length */
 #define	RDXMASK		0x0fffffff		/* kreadx() address mask */
-# endif	/* _AIXV>=4110 */
+# endif	/* AIXV>=4110 */
+
+
+/*
+ * Name list (Nl[]) indexes
+ */
+
+#define	X_UADDR		0
+#define	X_NL_NUM	1
+
+
+/*
+ * Define for ckfa.c
+ */
+
+#define	CKFA_MPXCHAN	1
+
+
+/*
+ * Definitions for dvch.c
+ */
+
+#if	AIXV>=4140
+#define	DCACHE_CLONE	rw_clone_sect	/* clone function for read_dcache */
+#define	DCACHE_CLR	clr_sect	/* function to clear clone and
+					 * pseudo caches when reading the
+					 * device cache file fails */
+#endif	/* AIXV>=4140 */
+
+
+# if	defined(HAS_AFS)
+/*
+ * AFS name list (AFSnl[]) indexes
+ */
+
+#define	X_AFS_FID	0
+#define	X_AFS_VOL	1
+#define	X_AFSNL_NUM	2
+# endif	/* defined(HAS_AFS) */
+
+
+# if	AIXV>=4140
+/*
+ * Local clone information
+ */
+
+struct clone {
+	struct l_dev cd;		/* device, inode, name, verify status */
+	struct clone *next;		/* next entry */
+};
+extern struct clone *Clone;
+extern int CloneMaj;
+extern int ClonePtc;
+#endif	/* AIXV>=4140 */
 
 
 /*
@@ -229,7 +334,18 @@ struct sfile {
  * Miscellaneous external definitions
  */
 
-extern int Kmem;
-extern int Mem;
+# if	defined(HAS_AFS)
+extern struct nlist AFSnl[];	/* AFS kernel symbol name list table */
+
+#  if	defined(HASAOPT)
+extern char *AFSApath;		/* alternate AFS name list path (from -a) */
+#  endif	/* defined(HASAOPT) */
+
+extern struct vfs *AFSVfsp;	/* AFS struct vfs kernel pointer */
+# endif	/* defined(HAS_AFS) */
+
+extern int Kd;
+extern int Km;
+extern struct nlist Nl[];
 
 #endif	/* AIX_LSOF_H */
