@@ -32,13 +32,23 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dsock.c,v 1.13 95/06/02 07:44:14 abe Exp Locker: abe $";
+static char *rcsid = "$Id: dsock.c,v 1.3 97/09/23 08:20:25 abe Exp $";
 #endif
 
 
 #include "lsof.h"
 
 
+/*
+ * We include <sys/domain.h> here instead of "dlsof.h" for gcc's benefit.
+ * It's loader can't handle the multiple CONST u_char arrays declared in
+ * <net/net_globals.h> -- e.g., etherbroadcastaddr[].  (<sys/domain.h>
+ * #include's <net/net_globals.h>.)
+ */
+
+#include <sys/domain.h>
+
+
 /*
  * process_socket() - process socket file
  */
@@ -49,16 +59,20 @@ process_socket(sa)
 {
 	struct domain d;
 	char dev_ch[32];
+	struct in_addr *fa = (struct in_addr *)NULL;
 	int fam;
+	int fp, lp;
 	struct gnode g;
 	struct inode i;
 	struct inpcb inp;
 	int is = 0;
+	struct in_addr *la = (struct in_addr *)NULL;
 	struct mbuf mb;
 	struct protosw p;
 	struct rawcb raw;
 	struct socket s;
 	struct sockaddr sad;
+	struct tcpcb t;
 	struct unpcb uc, unp;
 	struct sockaddr_un *ua = NULL;
 	struct sockaddr_un un;
@@ -92,18 +106,25 @@ process_socket(sa)
                 return;
         }
 /*
- * Set size, based on access type.
+ * Save size information.
  */
 	if (Fsize) {
 		if (Lf->access == 'r')
-			Lf->sz = s.so_rcv.sb_cc;
+			Lf->sz = (unsigned long)s.so_rcv.sb_cc;
 		else if (Lf->access == 'w')
-			Lf->sz = s.so_snd.sb_cc;
+			Lf->sz = (unsigned long)s.so_snd.sb_cc;
 		else
-			Lf->sz = s.so_rcv.sb_cc + s.so_snd.sb_cc;
+			Lf->sz = (unsigned long)s.so_rcv.sb_cc + s.so_snd.sb_cc;
 		Lf->sz_def = 1;
 	} else
 		Lf->off_def = 1;
+
+#if	defined(HASTCPTPIQ)
+	Lf->lts.rq = s.so_rcv.sb_cc;
+	Lf->lts.sq = s.so_snd.sb_cc;
+	Lf->lts.rqs = Lf->lts.sqs = 1;
+#endif	/* defined(HASTCPTPIQ) */
+
 /*
  * Process socket by the associated domain family.
  */
@@ -148,24 +169,25 @@ process_socket(sa)
 		    if (raw.rcb_laddr
 		    && !kread((KA_T)raw.rcb_laddr, (char *)&sad, sizeof(sad))) {
 			if (sad.sa_family == AF_INET)
-			    printinaddr((struct in_addr *)&sad.sa_data[2], -1);
+			    la = (struct in_addr *)&sad.sa_data[2];
 			else if (sad.sa_family)
 			    printrawaddr(&sad);
 		    }
 		    if (raw.rcb_laddr
 		    && !kread((KA_T)raw.rcb_faddr, (char *)&sad, sizeof(sad))) {
-			if (sad.sa_family == AF_INET) {
-			    (void) strcat(endnm(), "->");
-			    printinaddr((struct in_addr *)&sad.sa_data[2], -1);
-			} else if (sad.sa_family) {
+			if (sad.sa_family == AF_INET)
+			    fa = (struct in_addr *)&sad.sa_data[2];
+			else if (sad.sa_family) {
 			    (void) strcat(endnm(), "->");
 			    printrawaddr(&sad);
 			}
 		    }
+		    if (fa || la)
+			(void) ent_inaddr(la, -1, fa, -1);
 		} else {
 
 		/*
-		 * Print Internet socket information.
+		 * Save Internet socket information.
 		 */
 		    if (kread((KA_T) s.so_pcb, (char *) &inp, sizeof(inp))
 		    ||  (struct socket *)sa != inp.inp_socket) {
@@ -177,12 +199,19 @@ process_socket(sa)
 		    (void) sprintf(dev_ch, "0x%08x",
 			(inp.inp_ppcb == NULL) ? s.so_pcb : inp.inp_ppcb);
 		    enter_dev_ch(dev_ch);
-		    printinaddr(&inp.inp_laddr, (int)ntohs(inp.inp_lport));
+		    la = &inp.inp_laddr;
+		    lp = (int)ntohs(inp.inp_lport);
 		    if (inp.inp_faddr.s_addr != INADDR_ANY || inp.inp_fport
 		    != 0) {
-			(void) strcat(endnm(), "->");
-			    printinaddr(&inp.inp_faddr,
-				(int)ntohs(inp.inp_fport));
+			fa =  &inp.inp_faddr;
+			fp = (int)ntohs(inp.inp_fport);
+		    }
+		    if (fa || la)
+			(void) ent_inaddr(la, lp, fa, fp);
+		    if (p.pr_protocol == IPPROTO_TCP && inp.inp_ppcb
+		    &&  kread((KA_T)inp.inp_ppcb, (char *)&t, sizeof(t)) == 0) {
+			Lf->lts.type = 0;
+			Lf->lts.state.i = (int)t.t_state;
 		    }
 		}
 		break;
@@ -216,12 +245,12 @@ process_socket(sa)
 			break;
 		    }
 
-#if	_AIXV>=3200
+#if	AIXV>=3200
 		    ua = (struct sockaddr_un *) ((char *) &mb +
 			(mb.m_hdr.mh_data - (caddr_t) unp.unp_addr));
-#else	/* _AIXV<3200 */
+#else	/* AIXV<3200 */
 		    ua = (struct sockaddr_un *)(((char *)&mb) + mb.m_off);
-#endif	/* _AIXV>=3200 */
+#endif	/* AIXV>=3200 */
 
 		}
 		if (ua == NULL) {
@@ -258,11 +287,14 @@ process_socket(sa)
 	 * Read any associated vnode and then read its gnode and inode.
 	 */
 		g.gn_type = VSOCK;
-		if (unp.unp_vnode && readvnode(unp.unp_vnode, &v) == 0) {
-			if (v.v_gnode && readgnode(v.v_gnode, &g) == 0) {
+		if (unp.unp_vnode
+		&&  readvnode((caddr_t)unp.unp_vnode, &v) == 0) {
+			if (v.v_gnode
+			&&  readgnode((caddr_t)v.v_gnode, &g) == 0) {
 				Lf->lock = isglocked(&g);
 				if (g.gn_type == VSOCK && g.gn_data
-				&& readinode(g.gn_data, &i) == 0)
+				&& readinode((struct inode *)g.gn_data, &i)
+				== 0)
 					is = 1;
 			}
 		}
@@ -283,7 +315,7 @@ process_socket(sa)
 			if (mb.m_len >= sizeof(struct sockaddr_un))
 				mb.m_len = sizeof(struct sockaddr_un) - 1;
 			*((char *)ua + mb.m_len) = '\0';
-			if (Sfile && is_file_named(ua->sun_path, VSOCK, 0))
+			if (Sfile && is_file_named(ua->sun_path, VSOCK, 0, 0))
 				Lf->sf |= SELNM;
 			else
 				(void) strcpy(Namech, ua->sun_path);
