@@ -30,6 +30,64 @@
  */
 
 
+/*
+ * $Id: machine.h,v 1.8 97/09/23 08:29:29 abe Exp $
+ */
+
+
+#if	!defined(MACHINE_H)
+#define	MACHINE_H	1
+
+
+# if	AIXV>=4200
+/*
+ * AIX 4.2 requires that <sys/mstsave.h> be #include'd early.  It needs
+ * <sys/types.h> and _KERNEL.
+ *
+ * For gcc's sake, some redefinitions after including <sys/types.h> insure
+ * the off64_t and offset_t types are aligned on an 8 byte boundary.
+ */
+ 
+#include <sys/types.h>
+
+# if	defined(__GNUC__)
+typedef	long long	aligned_off64_t __attribute__ ((aligned (8)));
+typedef	long long	aligned_offset_t  __attribute__ ((aligned (8)));
+#define	off64_t		aligned_off64_t
+#define	offset_t	aligned_offset_t
+# endif	/* defined(__GNUC__) */
+
+#define	_KERNEL	1
+#include <sys/mstsave.h>
+#undef	_KERNEL
+# endif	/* AIXV>=4200 */
+
+
+/*
+ * GET_MAX_FD is defined for those dialects that provide a function other than
+ * getdtablesize() to obtain the maximum file descriptor number plus one.
+ */
+
+/* #define	GET_MAX_FD	?	*/
+
+
+/*
+ * HASAOPT is defined for those dialects that have AFS support; it specifies
+ * that the default path to an alternate AFS kernel name list file may be
+ * supplied with the -A <path> option.
+ */
+
+/* #define	HASAOPT		1 */
+
+
+/*
+ * HASBLKDEV is defined for those dialects that want block device information
+ * recorded in BDevtp[].
+ */
+
+#define	HASBLKDEV	1
+
+
 /*
  * HASDCACHE is defined for those dialects that support a device cache
  * file.
@@ -50,11 +108,14 @@
  *
  * HASSYSDC defines a public device cache file path.  When it's defined, it's
  * used as the path from which to read the device cache.
+ *
+ * Consult the 00DCACHE and 00FAQ files of the lsof distribution for more
+ * information on device cache file path construction.
  */
 
 #define	HASDCACHE	1
 #define	HASENVDC	"LSOFDEVCACHE"
-#define	HASPERSDC	".lsof_%L"
+#define	HASPERSDC	"%h/%p.lsof_%L"
 #define	HASPERSDCPATH	"LSOFPERSDCPATH"
 /* #define	HASSYSDC	"/var/adm/lsof.dc" */
 
@@ -70,9 +131,17 @@
  * HASFIFONODE is defined for those dialects that have FIFO nodes.
  */
 
-#if	_AIXV>=3200
+# if	AIXV>=3200
 #define	HASFIFONODE	1
-#endif	/* _AIXV>=3200 */
+# endif	/* AIXV>=3200 */
+
+
+/*
+ * HASFSINO is defined for those dialects that have the file system
+ * inode element, fs_ino, in the lfile structure definition in lsof.h.
+ */
+
+/* #define	HASFSINO	1 */
 
 
 /*
@@ -90,7 +159,8 @@
 
 
 /*
- * HASINODE is defined for those dialects that have inodes.
+ * HASINODE is defined for those dialects that have inodes and wish to
+ * use readinode() from node.c.
  */
 
 #define	HASINODE	1
@@ -118,7 +188,7 @@
  * them.  If any of the additional elements need to be preset in the
  * alloc_lfile() function of proc.c, the SETLFILEADD macro may be defined
  * to do that.  The HASXOPT definition may be used to select the conditions
- * under which provate lfile elements are used.
+ * under which private lfile elements are used.
  *
  * Field identifiers 1 through 9 are reserved for dialect-specific field
  * output.  Define the string HASFIELDAP<x> as the help (-F?) name of the
@@ -146,10 +216,21 @@
 
 /*
  * HASNLIST is defined for those dialects that use nlist() to acccess
- * kernel symbols.
+ * kernel symbols.  (AIX lsof doesn't use nlist, it uses knlist.)
  */
 
-#define	HASNLIST	1
+/* #define	HASNLIST	1 */
+
+
+/*
+ * HASPIPEFN is defined for those dialects that have a special function to
+ * process DTYPE_PIPE file structure entries.  Its value is the name of the
+ * function.
+ *
+ * NOTE: don't forget to define a prototype for this function in dproto.h.
+ */
+
+/* #define	HASPIPEFN	process_pipe */
 
 
 /*
@@ -159,6 +240,30 @@
 /* #define	HASPIPENODE	1 */
 
 
+/*
+ * HASPPID is defined for those dialects that support identification of
+ * the parent process IDentifier (PPID) of a process.
+ */
+
+#define	HASPPID		1
+
+
+/*
+ * HASPRIVFILETYPE and PRIVFILETYPE are defined for dialects that have a
+ * file structure type that isn't defined by a DTYPE_* symbol.  They are
+ * used in lib/prfp.c to select the type's processing.
+ *
+ * PRIVFILETYPE is the definition of the f_type value in the file struct.
+ *
+ * HASPRIVFILETYPE is the name of the processing function.
+ */
+
+# if	AIXV>=4140
+#define	HASPRIVFILETYPE	process_shmt
+#define	PRIVFILETYPE	15
+# endif	/* AIXV>=4140 */
+
+
 /*
  * HASPROCFS is defined for those dialects that have a proc file system --
  * usually /proc and usually in SYSV4 derivatives.
@@ -182,21 +287,13 @@
 /* #define		HASPINFO	"/pinfo" */
 
 
-/*
- * HASPWSTAYOPEN is defined for those dialects that have a method to hold
- * the password data base open between searches.  Setting this definition
- * implies the availability of a dialect-specific pw_stay_open() function
- * to implement the method.
- */
-
-/* #define	HASPWSTAYOPEN	1 */
-
-
 /*
  * HASRNODE is defined for those dialects that have rnodes.
  */
 
+#if	defined(HAS_NFS)
 #define	HASRNODE	1
+#endif	/* defined(HAS_NFS) */
 
 
 /*
@@ -223,6 +320,22 @@
 /* #define	HASSTREAMS	1 */
 
 
+/*
+ * HASTCPTPIQ is defined for dialects where it is possible to report the
+ * TCP/TPI Recv-Q and Send-Q values produced by netstat.
+ */
+
+#define	HASTCPTPIQ	1
+
+
+/*
+ * HASTCPTPIW is defined for dialects where it is possible to report the
+ * TCP/TPI send and receive window sizes produced by netstat.
+ */
+
+/* #define	HASTCPTPIW	1 */
+
+
 /*
  * HASTMPNODE is defined for those dialects that have tmpnodes.
  */
@@ -250,9 +363,13 @@
  * If you want to disable the use of readx() permanently, leave HASXOPT
  * undefined and set HASXOPT_VALUE to 0.  To enable readx() permanently,
  * leave HASXOPT undefined and set HASXOPT_VALUE to 1.
+ *
+ * Define HASXOPT_ROOT if you want to restrict the use of the X option to
+ * processes whose real UID is root.
  */
 
 #define	HASXOPT		"use readx() -- *RISKY*"
+/* #define	HASXOPT_ROOT	1 */
 #define	HASXOPT_VALUE	0
 
 
@@ -264,6 +381,32 @@
 #define	UID_ARG	uid_t
 
 
+/*
+ * Each USE_LIB_<function_name> is defined for dialects that use the
+ * <function_name> in the lsof library.
+ *
+ * Note: other definitions and operations may be required to condition the
+ * library function source code.  They may be found in the dialect dlsof.h
+ * header files.
+ */
+
+#define	USE_LIB_CK_FILE_ARG			1	/* ckfa.c */
+/* #define	USE_LIB_COMPLETEVFS		1	   cvfs.c */
+#define	USE_LIB_FIND_CH_INO			1	/* fino.c */
+/* #define	USE_LIB_IS_FILE_NAMED		1	   isfn.c */
+#define	USE_LIB_LKUPDEV				1	/* lkud.c */
+/* #define	USE_LIB_PRINTCHDEVNAME		1	   pdvn.c */
+#define	USE_LIB_PROCESS_FILE			1	/* prfp.c */
+/* #define	USE_LIB_PRINT_FILE		1	   prtf.c */
+#define	USE_LIB_PRINT_TCPTPI			1	/* ptti.c */
+/* #define	USE_LIB_READDEV			1	   rdev.c */
+/* #define	USE_LIB_READMNT			1	   rmnt.c */
+/* #define	USE_LIB_READVFS			1	   rvfs.c */
+/* #define	USE_LIB_RNAM			1	   rnam.c */
+/* #define	USE_LIB_RNCH			1	   rnch.c */
+/* #define	USE_LIB_RNMH			1	   rnmh.c */
+
+
 /*
  * WARNDEVACCESS is defined for those dialects that should issue a warning
  * when lsof can't access /dev (or /device) or one of its sub-directories.
@@ -273,6 +416,14 @@
 #define	WARNDEVACCESS	1
 
 
+/*
+ * WARNINGSTATE is defined for those dialects that want to suppress all lsof
+ * warning messages.
+ */
+
+/* #define	WARNINGSTATE	1	warnings are enabled by default */
+
+
 /*
  * WILLDROPGID is defined for those dialects whose lsof executable runs
  * setgid(not_real_GID) and whose setgid power can be relinquished after
@@ -287,3 +438,5 @@
  */
 
 #define	zeromem(a, l)		bzero(a, l)
+
+#endif	/* !defined(MACHINE_H) */
