@@ -30,6 +30,11 @@
  */
 
 
+/*
+ * $Id: dlsof.h,v 1.4 97/08/15 08:19:18 abe Exp $
+ */
+
+
 #if	!defined(FREEBSD_LSOF_H)
 #define	FREEBSD_LSOF_H	1
 
@@ -51,34 +56,50 @@
 #include <sys/socketvar.h>
 #include <sys/un.h>
 #include <sys/unpcb.h>
+
+# if	FREEBSDV>=300
+#undef	INADDR_LOOPBACK
+# endif	/* FREEBSDV>=300 */
+
 #include <netinet/in.h>
 #include <netinet/in_systm.h>
 #include <netinet/ip.h>
 #include <net/route.h>
 #include <netinet/in_pcb.h>
+#include <netinet/ip_var.h>
+#include <netinet/tcp.h>
+#include <netinet/tcpip.h>
+#include <netinet/tcp_fsm.h>
+#include <netinet/tcp_timer.h>
+#include <netinet/tcp_var.h>
 #include <sys/ucred.h>
 #include <sys/uio.h>
 #include <sys/vnode.h>
 #include <net/raw_cb.h>
 #include <sys/domain.h>
 
-# if	_FREEBSDV<200
-#include <ufs/lockf.h>
+# if	FREEBSDV<200
 #include <ufs/quota.h>
 #include <ufs/inode.h>
 #include <ufs/ufsmount.h>
 #include <ufs/mfsnode.h>
-# else	/* _FREEBSDV>=200 */
+# else	/* FREEBSDV>=200 */
 #include <paths.h>
-#include <sys/fcntl.h>
-struct vop_advlock_args { int dummy; };	/* to pacify lf_advlock() prototype */
-#include <sys/lockf.h>
 #include <ufs/ufs/quota.h>
 #include <ufs/ufs/inode.h>
+
+#  if   FREEBSDV<220
 #include <ufs/mfs/mfsnode.h>
-# endif	/* _FREEBSDV<200 */
+#  endif        /* FREEBSDV<220 */
+
+# endif	/* FREEBSDV<200 */
 
 #include <nfs/nfsv2.h>
+
+# if	defined(HASRPCV2H)
+#include <nfs/rpcv2.h>
+# endif	/* defined(HASRPCV2H) */
+
 #include <nfs/nfs.h>
 #include <nfs/nfsnode.h>
 #include <sys/proc.h>
@@ -86,58 +107,103 @@ struct vop_advlock_args { int dummy; };	/* to pacify lf_advlock() prototype */
 #undef	TRUE
 #undef	FALSE
 
-# if	_FREEBSDV<200
+# if	FREEBSDV<200
 #include <sys/kinfo.h>
-# else	/* _FREEBSDV>=200 */
+# else	/* FREEBSDV>=200 */
 #include <sys/sysctl.h>
-# endif	/* _FREEBSDV<200 */
+# endif	/* FREEBSDV<200 */
+
+# if	defined(HASFDESCFS)
+#define	_KERNEL
+#define	KERNEL
+#include <miscfs/fdesc/fdesc.h>
+#undef	_KERNEL
+#undef	KERNEL
+# endif	/* defined(HASFDESCFS) */
 
 # if	defined(HASPROCFS)
-#  if	_FREEBSDV<200
+#  if	FREEBSDV<200
 #include <procfs/pfsnode.h>
-# else	/* _FREEBSDV>=200 */
-#include <procfs/procfs.h>
+# else	/* FREEBSDV>=200 */
+#include <miscfs/procfs/procfs.h>
 #include <machine/reg.h>
-# endif	/* _FREEBSDV<200 */
+# endif	/* FREEBSDV<200 */
 
 #define	PNSIZ		5
 # endif	/* defined(HASPROCFS) */
 
-# if	_FREEBSDV<200
+# if	FREEBSDV<200
 #define	P_COMM		p_comm
 #define	P_FD		p_fd
 #define	P_PID		p_pid
 #define	P_PGRP		p_pgrp
+#define	P_STAT		p_stat
 #define	P_VMSPACE	p_vmspace
-# else	/* _FREEBSDV>=200 */
+# else	/* FREEBSDV>=200 */
+#define	P_ADDR		kp_eproc.e_paddr
 #define	P_COMM		kp_proc.p_comm
 #define	P_FD		kp_proc.p_fd
 #define	P_PID		kp_proc.p_pid
 #define	P_PGRP		kp_eproc.e_pgid
+#define	P_PPID		kp_eproc.e_ppid
+#define	P_STAT		kp_proc.p_stat
 #define	P_VMSPACE	kp_proc.p_vmspace
-# endif	/* _FREEBSDV<200 */
+# endif	/* FREEBSDV<200 */
 
 #define	KERNEL
+#include <sys/fcntl.h>
 #include <sys/file.h>
 #undef	KERNEL
+
+# if	FREEBSDV<200
+#include <ufs/lockf.h>
+# else	/* FREEBSDV>=200 */
+struct vop_advlock_args { int dummy; };	/* to pacify lf_advlock() prototype */
+#include <sys/lockf.h>
+# endif	/* FREEBSDV<200 */
+
 #include <vm/vm.h>
+
+#  if   FREEBSDV>=220
+#include <sys/pipe.h>
+#   if	defined(HASVMLOCKH)
+#include <vm/lock.h>
+#   endif	/* defined(HASVMLOCKH) */
+#include <vm/pmap.h>
+#  endif        /* FREEBSDV>=220 */
+
 #include <vm/vm_map.h>
 #include <vm/vm_object.h>
 #include <vm/vm_pager.h>
 
+#  if   FREEBSDV>=220
+#undef	B_NEEDCOMMIT
+#include <sys/buf.h>
+#include <sys/user.h>
+#include <ufs/mfs/mfsnode.h>
+#  endif        /* FREEBSDV>=220 */
+
+
 #define	COMP_P		const void
 #define DEVINCR		1024	/* device table malloc() increment */
 
-# if	_FREEBSDV<200
+# if	FREEBSDV<200
 #define	KA_T		off_t
-# else	/* _FREEBSDV>=200 */
+# else	/* FREEBSDV>=200 */
 #define	KA_T		u_long
-# endif	/* _FREEBSDV<200 */
+# endif	/* FREEBSDV<200 */
 
 #define	KMEM		"/dev/kmem"
 #define MALLOC_P	void
 #define FREE_P		MALLOC_P
 #define MALLOC_S	size_t
+
+# if	defined(N_UNIXV)
+#define	N_UNIX_TMP(x)	#x
+#define	N_UNIX_STR(x)	N_UNIX_TMP(x)
+#define	N_UNIX		N_UNIX_STR(N_UNIXV)
+# endif	/* defined(N_UNIXV) */
+
 #define QSORT_P		void
 #define	READLEN_T	int
 #define STRNCPY_L	size_t
@@ -148,9 +214,15 @@ struct vop_advlock_args { int dummy; };	/* to pacify lf_advlock() prototype */
  * Global storage definitions (including their structure definitions)
  */
 
-# if	_FREEBSDV>=200
+struct file * Cfp;
+
+# if	FREEBSDV>=200
 extern kvm_t *Kd;
-# endif	/* _FREEBSDV>=200 */
+# endif	/* FREEBSDV>=200 */
+
+# if	defined(P_ADDR)
+extern KA_T Kpa;
+# endif	/* defined(P_ADDR) */
 
 struct l_vfs {
 	struct mount *addr;		/* kernel address */
@@ -172,14 +244,15 @@ struct mounts {
 	dev_t rdev;			/* st_rdev */
 };
 
-#define	X_NCACHE	0		/* Nl[] index */
-#define	X_NCSIZE	1		/* Nl[] index */
+#define	X_NCACHE	"ncache"
+#define	X_NCSIZE	"ncsize"
+#define	NL_NAME		n_name
 
 extern int Np;				/* number of kernel processes */
 
-# if	_FREEBSDV>=200
+# if	FREEBSDV>=200
 extern struct kinfo_proc *P;		/* local process table copy */
-# endif	/* _FREEBSDV>=200 */
+# endif	/* FREEBSDV>=200 */
 
 struct sfile {
 	char *aname;			/* argument file name */
@@ -195,4 +268,44 @@ struct sfile {
 
 };
 
+
+/*
+ * Definitions for rdev.c
+ */
+
+#define	DIRTYPE	dirent
+#define HASDNAMLEN	1	/* struct DIRTYPE has d_namlen element */
+
+
+/*
+ * Definitions for rnam.c
+ */
+
+# if     defined(HASNCACHE)
+#include <sys/uio.h>
+#include <sys/namei.h>
+#define	NCACHE		namecache	/* kernel's structure name */
+#define	NCACHE_NM	nc_name		/* name in NCACHE */
+#define	NCACHE_NMLEN	nc_nlen		/* name length in NCACHE */
+
+#  if	FREEBSDV<205
+#define	NCACHE_NXT	nc_nxt		/* link in NCACHE */
+#  else	/* FREEBSDV>=205 */
+#   if	FREEBSDV<210
+#define	NCACHE_NXT	nc_lru.tqe_next	/* link in NCACHE */
+#   else	/* FREEBSDV>=210 */
+#include <stddef.h>
+#define	NCACHE_NXT	nc_hash.le_next	/* link in NCACHE */
+#   endif	/* FREEBSDV<210 */
+#  endif	/* FREEBSDV<205 */
+
+#define	NCACHE_NODEADDR	nc_vp		/* node address in NCACHE */
+#define	NCACHE_PARADDR	nc_dvp		/* parent node address in NCACHE */
+
+#  if	defined(HASNCVPID)
+#define	NCACHE_NODEID	nc_vpid		/* node ID in NCACHE */
+#define	NCACHE_PARID	nc_dvpid	/* parent node ID in NCACHE */
+#  endif	/* DEFINED(HASNCVPID) */
+# endif  /* defined(HASNCACHE) */
+
 #endif	/* FREEBSD_LSOF_H */
