@@ -32,14 +32,24 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dnode.c,v 1.13 95/06/26 09:32:02 abe Exp $";
+static char *rcsid = "$Id: dnode.c,v 1.3 97/04/29 10:02:10 abe Exp $";
 #endif
 
 
 #include "lsof.h"
 
+#if	defined(HASPROCFS)
+#define PROCFSNAME_TMP(x)	#x
+#define	PROCFSNAME_STR(x)	PROCFSNAME_TMP(x)
+#define	PROCFSNAME		PROCFSNAME_STR(HASPROCFS)
+#endif	/* defined(HASPROCFS) */
 
-#if	_FREEBSDV>=200
+#if	defined(HASFDESCFS)
+_PROTOTYPE(static int lkup_dev_tty,(dev_t *dr, unsigned long *ir));
+#endif	/* defined(HASFDESCFS) */
+
+
+#if	FREEBSDV>=200
 # if	defined(HASPROCFS)
 _PROTOTYPE(static void getmemsz,(pid_t pid));
 
@@ -61,15 +71,61 @@ getmemsz(pid)
 			if (p->P_VMSPACE == NULL
 			||  kread((KA_T)p->P_VMSPACE, (char *)&vm, sizeof(vm)))
 				return;
-			Lf->sz = (int)ctob(vm.vm_tsize + vm.vm_dsize +
-					   vm.vm_ssize);
+			Lf->sz = (unsigned long)ctob(vm.vm_tsize + vm.vm_dsize
+						     + vm.vm_ssize);
 			Lf->sz_def = 1;
 			return;
 		}
 	}
 }
 # endif	/* defined(HASPROCFS) */
-#endif	/* _FREEBSDV>=200 */
+#endif	/* FREEBSDV>=200 */
+
+
+#if	defined(HASFDESCFS)
+/*
+ * lkup_dev_tty() - look up /dev/tty
+ */
+
+static int
+lkup_dev_tty(dr, ir)
+	dev_t *dr;			/* place to return device number */
+	unsigned long *ir;		/* place to return inode number */
+{
+	int i;
+
+	readdev(0);
+
+# if	defined(HASDCACHE)
+
+lkup_dev_tty_again:
+
+# endif	/* defined(HASDCACHE) */
+
+	for (i = 0; i < Ndev; i++) {
+	    if (strcmp(Devtp[i].name, "/dev/tty") == 0) {
+
+# if	defined(HASDCACHE)
+		if (DCunsafe && !Devtp[i].v && !vfy_dev(&Devtp[i]))
+		    goto lkup_dev_tty_again;
+# endif	/* defined(HASDCACHE) */
+
+		*dr = Devtp[i].rdev;
+		*ir = (unsigned long)Devtp[i].inode;
+		return(1);
+	    }
+	}
+
+# if	defined(HASDCACHE)
+	if (DCunsafe) {
+	    (void) rereaddev();
+	    goto lkup_dev_tty_again;
+	}
+# endif	/* defined(HASDCACHE) */
+
+	return(-1);
+}
+#endif	/* defined(HASFDESCFS) */
 
 
 /*
@@ -82,32 +138,49 @@ process_node(va)
 {
 
 	dev_t dev;
+	int devs = 0;
 	char dev_ch[32];
-	struct inode *i = NULL;
-	struct lockf lf;
+	struct inode *i = (struct inode *)NULL;
+	struct lockf lf, *lff, *lfp;
 	int lt;
-	struct mfsnode *m = NULL;
-	struct nfsnode *n = NULL;
+	struct mfsnode *m = (struct mfsnode *)NULL;
+	struct nfsnode *n = (struct nfsnode *)NULL;
 	char *ty;
 	enum vtype type;
 	struct vnode *v, vb;
 	struct l_vfs *vfs;
 
-#if	_FREEBSDV>=200
+#if	FREEBSDV>=200
 	struct inode ib;
 	struct mfsnode mb;
 	struct nfsnode nb;
-#endif	/* _FREEBSDV>=200 */
+#endif	/* FREEBSDV>=200 */
+
+#if	defined(HAS9660FS)
+	dev_t iso_dev;
+	unsigned long iso_ino, iso_sz;
+	int iso_stat = 0;
+#endif	/* defined(HAS9660FS) */
+
+#if	defined(HASFDESCFS)
+	struct fdescnode *f = (struct fdescnode *)NULL;
+	static dev_t f_tty_dev;
+	static unsigned long f_tty_ino;
+	static int f_tty_s = 0;
+# if	FREEBSDV>=200
+	struct fdescnode fb;
+# endif	/* FREEBSDV>=200 */
+#endif	/* defined(HASFDESCFS) */
 
 #if	defined(HASPROCFS)
-	struct pfsnode *p = NULL;
+	struct pfsnode *p = (struct pfsnode *)NULL;
 	struct procfsid *pfi;
 	static int pgsz = -1;
 	struct vmspace vm;
 
-# if	_FREEBSDV>=200
+# if	FREEBSDV>=200
 	struct pfsnode pb;
-# endif	/* _FREEBSDV>=200 */
+# endif	/* FREEBSDV>=200 */
 #endif	/* defined(HASPROCFS) */
 
 /*
@@ -119,9 +192,9 @@ process_node(va)
 	}
 	v = &vb;
 	if (readvnode((caddr_t)va, v)) {
-                enter_nm(Namech);
-                return;
-        }
+		enter_nm(Namech);
+		return;
+	}
 
 # if	defined(HASNCACHE)
 	Lf->id = v->v_id;
@@ -160,11 +233,23 @@ process_node(va)
  * Define the specific node pointer.
  */
 	switch (v->v_tag) {
+
+#if	defined(HAS9660FS)
+	case VT_ISOFS:
+		if (read_iso_node(v, &iso_dev, &iso_ino, &iso_sz)) {
+			(void) sprintf(Namech, "no iso node: %#x", v->v_data);
+			enter_nm(Namech);
+			return;
+		}
+		iso_stat = 1;
+		break;
+#endif	/* defined(HAS9660FS) */
+
 	case VT_MFS:
 
-#if	_FREEBSDV<200
+#if	FREEBSDV<200
 		m = (struct mfsnode *)v->v_data;
-#else	/* _FREEBSDV>=200 */
+#else	/* FREEBSDV>=200 */
 		if (!v->v_data
 		||  kread((KA_T)v->v_data, (char *)&mb, sizeof(mb))) {
 			(void) sprintf(Namech, "no mfs node: %#x", v->v_data);
@@ -172,14 +257,14 @@ process_node(va)
 			return;
 		}
 		m = &mb;
-#endif	/* _FREEBSDV<200 */
+#endif	/* FREEBSDV<200 */
 
 		break;
 	case VT_NFS:
 
-#if	_FREEBSDV<200
+#if	FREEBSDV<200
 		n = (struct nfsnode *)v->v_data;
-#else	/* _FREEBSDV>=200 */
+#else	/* FREEBSDV>=200 */
 		if (!v->v_data
 		||  kread((KA_T)v->v_data, (char *)&nb, sizeof(nb))) {
 			(void) sprintf(Namech, "no nfs node: %#x", v->v_data);
@@ -187,16 +272,33 @@ process_node(va)
 			return;
 		}
 		n = &nb;
-#endif	/* _FREEBSDV<200 */
+#endif	/* FREEBSDV<200 */
 
 		break;
 
+#if	defined(HASFDESCFS)
+	case VT_FDESC:
+
+# if	FREEBSDV<200
+		f = (struct fdescnode *)v->v_data;
+# else	/* FREEBSDV>=200 */
+		if (kread((KA_T)v->v_data, (char *)&fb, sizeof(fb)) != 0) {
+			(void) sprintf(Namech, "can't read fdescnode at: %x",
+				v->v_data);
+			enter_nm(Namech);
+			return;
+		}
+		f = &fb;
+		break;
+# endif	/* FREEBSDV<200 */
+#endif	/* defined(HASFDESCFS) */
+
 #if	defined(HASPROCFS)
 	case VT_PROCFS:
 
-# if	_FREEBSDV<200
+# if	FREEBSDV<200
 		p = (struct pfsnode *)v->v_data;
-# else	/* _FREEBSDV>=200 */
+# else	/* FREEBSDV>=200 */
 		if (!v->v_data
 		||  kread((KA_T)v->v_data, (char *)&pb, sizeof(pb))) {
 			(void) sprintf(Namech, "no pfs node: %#x", v->v_data);
@@ -204,70 +306,139 @@ process_node(va)
 			return;
 		}
 		p = &pb;
-# endif	/* _FREEBSDV<200 */
+# endif	/* FREEBSDV<200 */
 
 		break;
 #endif	/* defined(HASPROCFS) */
 
 	case VT_UFS:
 
-#if	_FREEBSDV<200
+
+#if	FREEBSDV<200
 		i = (struct inode *)v->v_data;
-#else	/* _FREEBSDV>=200 */
+#else	/* FREEBSDV>=200 */
 		if (!v->v_data
-		||  kread((KA_T)v->v_data, (char *)&ib, sizeof(ib))) {
+		    ||  kread((KA_T)v->v_data, (char *)&ib, sizeof(ib))) {
 			(void) sprintf(Namech, "no ifs node: %#x", v->v_data);
 			enter_nm(Namech);
 			return;
 		}
 		i = &ib;
-#endif	/* _FREEBSDV<200 */
+#endif	/* FREEBSDV<200 */
 
-		if (i->i_lockf != NULL
-		&&  kread((KA_T)i->i_lockf, (char *)&lf, sizeof(lf)) == 0) {
+		if ((lff = i->i_lockf)) {
 
 		/*
 		 * Determine the lock state.
 		 */
-			if (lf.lf_start == (off_t)0 && lf.lf_end == (off_t)-1)
+		    lfp = lff;
+		    do {
+			if (kread((KA_T)lfp, (char *)&lf, sizeof(lf)))
+			    break;
+			lt = 0;
+			switch (lf.lf_flags & (F_FLOCK|F_POSIX)) {
+			case F_FLOCK:
+			    if (Cfp && (struct file *)lf.lf_id == Cfp)
 				lt = 1;
+			    break;
+			case F_POSIX:
+
+#if	defined(P_ADDR)
+			    if ((KA_T)lf.lf_id == Kpa)
+				lt = 1;
+#endif	/* defined(P_ADDR) */
+
+			    break;
+			}
+			if (!lt)
+			    continue;
+			if (lf.lf_start == (off_t)0
+			&&  lf.lf_end == 0xffffffffffffffffLL)
+			    lt = 1;
 			else
-				lt = 0;
+			    lt = 0;
 			if (lf.lf_type == F_RDLCK)
-				Lf->lock = lt ? 'R' : 'r';
+			    Lf->lock = lt ? 'R' : 'r';
 			else if (lf.lf_type == F_WRLCK)
-				Lf->lock = lt ? 'W' : 'w';
+			    Lf->lock = lt ? 'W' : 'w';
 			else if (lf.lf_type == (F_RDLCK | F_WRLCK))
-				Lf->lock = 'u';
+			    Lf->lock = 'u';
+			break;
+		    } while ((lfp = lf.lf_next) && lfp != lff);
 		}
 		break;
+	default:
+		if (v->v_type == VBAD || v->v_type == VNON)
+			break;
+		(void) sprintf(Namech, "unknown file system type: %d",
+			v->v_tag);
+		enter_nm(Namech);
+		return;
 	}
 /*
  * Get device and type for printing.
  */
 	type = v->v_type;
-	if (n != NULL)
+	if (n != NULL) {
 		dev = n->n_vattr.va_fsid;
-	else if (i != NULL)
-		dev = (type == VCHR) ? i->i_rdev : i->i_dev;
+		devs = 1;
+	} else if (i != NULL) {
+		dev = (type == VCHR || type == VBLK) ? i->i_rdev : i->i_dev;
+		devs = 1;
+	}
+
+#if	defined(HASFDESCFS)
+	else if (f != NULL) {
+		if (f->fd_link
+		&&  kread((KA_T)f->fd_link, Namech, sizeof(Namech) - 1) == 0)
+		    Namech[sizeof(Namech) - 1] = '\0';
+		else if (f->fd_type == Fctty) {
+		    if (f_tty_s == 0)
+			f_tty_s = lkup_dev_tty(&f_tty_dev, &f_tty_ino);
+		    if (f_tty_s == 1) {
+			dev = f_tty_dev;
+			Lf->inode = f_tty_ino;
+			devs = Lf->inp_ty = 1;
+		    }
+		}
+	}
+#endif	/* defined(HASFDESCFS) */
+
+#if	defined(HAS9660FS)
+	else if (iso_stat) {
+		dev = iso_dev;
+		devs = Lf->inp_ty = 1;
+	}
+#endif	/* defined(HAS9660FS) */
+
+
 /*
  * Obtain the inode number.
  */
 	if (i != NULL) {
-		Lf->inode = (unsigned long)i->i_number;
-		Lf->inp_ty = 1;
+		if (type != VBLK) {
+			Lf->inode = (unsigned long)i->i_number;
+			Lf->inp_ty = 1;
+		}
 	} else if (n != NULL) {
 		Lf->inode = (unsigned long)n->n_vattr.va_fileid;
 		Lf->inp_ty = 1;
 	}
 
+#if	defined(HAS9660FS)
+	else if (iso_stat) {
+		Lf->inode = iso_ino;
+		Lf->inp_ty = 1;
+	}
+#endif	/* defined(HAS9660FS) */
+
 #if	defined(HASPROCFS)
-# if	_FREEBSDV>=200
+# if	FREEBSDV>=200
 	else if (p != NULL) {
 		Lf->inode = (unsigned long)p->pfs_fileno;
 		Lf->inp_ty = 1;
 	}
-# endif	/* _FREEBSDV>=200 */
+# endif	/* FREEBSDV>=200 */
 #endif	/* defined(HASPROCFS) */
 
 /*
@@ -283,7 +454,7 @@ process_node(va)
 			break;
 		case N_NFS:
 			if (n != NULL) {
-				Lf->sz = (off_t)n->n_vattr.va_size;
+				Lf->sz = (unsigned long)n->n_vattr.va_size;
 				Lf->sz_def = 1;
 			}
 			break;
@@ -291,52 +462,62 @@ process_node(va)
 #if	defined(HASPROCFS)
 		case N_PROC:
 
-# if	_FREEBSDV<200
+# if	FREEBSDV<200
 			if (type == VDIR || p == NULL || p->pfs_vs == NULL
 			||  kread((KA_T)p->pfs_vs, (char *)&vm, sizeof(vm)))
 				break;
 			if (pgsz < 0)
 				pgsz = getpagesize();
-			Lf->sz = (off_t)((pgsz * vm.vm_tsize)
+			Lf->sz = (unsigned long)((pgsz * vm.vm_tsize)
 			       +         (pgsz * vm.vm_dsize)
 			       +         (pgsz * vm.vm_ssize));
 			Lf->sz_def = 1;
 			break;
-# else	/* _FREEBSD>=200 */
+# else	/* FREEBSDV>=200 */
 			if (p) {
 				switch(p->pfs_type) {
 				case Proot:
 				case Pproc:
-					Lf->sz = DEV_BSIZE;
+					Lf->sz = (unsigned long)DEV_BSIZE;
 					Lf->sz_def = 1;
 					break;
 				case Pmem:
 					(void) getmemsz(p->pfs_pid);
 					break;
 				case Pregs:
-					Lf->sz = sizeof(struct reg);
+					Lf->sz = (unsigned long)
+						 sizeof(struct reg);
 					Lf->sz_def = 1;
 					break;
 				case Pfpregs:
-					Lf->sz = sizeof(struct fpreg);
+					Lf->sz = (unsigned long)
+						 sizeof(struct fpreg);
 					Lf->sz_def = 1;
 					break;
 				}
 			}
-# endif	/* _FREEBSDV<200 */
+# endif	/* FREEBSDV<200 */
 #endif	/* defined(HASPROCFS) */
 
 		case N_REGLR:
 			if (type == VREG || type == VDIR) {
 				if (i != NULL) {
-					Lf->sz = i->i_size;
+					Lf->sz = (unsigned long)i->i_size;
 					Lf->sz_def = 1;
 				} else if (m != NULL) {
-					Lf->sz = m->mfs_size;
+					Lf->sz = (unsigned long)m->mfs_size;
 					Lf->sz_def = 1;
 				}
+
+#if	defined(HAS9660FS)
+				else if (iso_stat) {
+					Lf->sz = iso_sz;
+					Lf->sz_def = 1;
+				}
+#endif	/* defined(HAS9660FS) */
+
 			}
-			else if (type == VCHR && !Fsize)
+			else if ((type == VCHR || type == VBLK) && !Fsize)
 				Lf->off_def = 1;
 			break;
 		}
@@ -364,16 +545,26 @@ process_node(va)
 	case VREG:
 	case VDIR:
 		ty = (type == VREG) ? "VREG" : "VDIR";
-		Lf->dev_def = 1;
-		Lf->dev = dev;
+		if (devs) {
+			Lf->dev_def = 1;
+			Lf->dev = dev;
+		}
 		break;
 	case VBLK:
 		ty = "VBLK";
+		if (devs) {
+			Lf->dev_def = 1;
+			Lf->dev = dev;
+		}
+		Ntype = N_BLK;
 		break;
 	case VCHR:
-		Lf->dev = dev;
-		Lf->dev_def = Lf->is_chr_dev = 1;
 		ty = "VCHR";
+		if (devs) {
+			Lf->dev = dev;
+			Lf->dev_def = 1;
+		}
+		Ntype = N_CHR;
 		break;
 	case VLNK:
 		ty = "VLNK";
@@ -389,8 +580,10 @@ process_node(va)
 		ty = "VBAD";
 		break;
 	case VFIFO:
-		Lf->dev = dev;
-		Lf->dev_def = 1;
+		if (devs) {
+			Lf->dev = dev;
+			Lf->dev_def = 1;
+		}
 		ty = "FIFO";
 		break;
 	default:
@@ -403,6 +596,7 @@ process_node(va)
 	}
 	if (ty)
 		(void) strcpy(Lf->type, ty);
+	Lf->ntype = Ntype;
 /*
  * Handle some special cases:
  *
@@ -424,16 +618,16 @@ process_node(va)
 	else if (p != NULL) {
 		Lf->dev_def = 0;
 
-# if	_FREEBSDV<200
+# if	FREEBSDV<200
 		if (type == VDIR)
-			(void) sprintf(Namech, "/%s", HASPROCFS);
+			(void) sprintf(Namech, "/%s", PROCFSNAME);
 		else
-			(void) sprintf(Namech, "/%s/%0*d", HASPROCFS,
+			(void) sprintf(Namech, "/%s/%0*d", PROCFSNAME,
 				PNSIZ, p->pfs_pid);
 		enter_nm(Namech);
-# else	/* _FREEBSDV>=200 */
+# else	/* FREEBSDV>=200 */
 		ty = NULL;
-		(void) sprintf(Namech, "/%s", HASPROCFS);
+		(void) sprintf(Namech, "/%s", PROCFSNAME);
 		switch (p->pfs_type) {
 		case Proot:
 			ty = "PDIR";
@@ -479,10 +673,19 @@ process_node(va)
 			(void) strcpy(Lf->type, ty);
 		enter_nm(Namech);
 
-# endif	/* _FREEBSD<200 */
+# endif	/* FREEBSDV<200 */
 	}
 #endif	/* defined(HASPROCFS) */
 
+#if	defined(HASBLKDEV)
+/*
+ * If this is a VBLK file and it's missing an inode number, try to
+ * supply one.
+ */
+	if (Lf->inp_ty == 0 && type == VBLK && Lf->dev_def)
+		find_bl_ino();
+#endif	/* defined(HASBLKDEV) */
+
 /*
  * If this is a VCHR file and it's missing an inode number, try to
  * supply one.
@@ -525,3 +728,49 @@ process_node(va)
 	if (Namech[0])
 		enter_nm(Namech);
 }
+
+
+#if	FREEBSDV>=220
+/*
+ * process_pipe() - process a file structure whose type is DTYPE_PIPE
+ */
+
+void
+process_pipe(pa)
+	caddr_t pa;			/* pipe structure address */
+{
+	char dev_ch[32];
+	struct pipe p;
+
+	if (!pa || kread((KA_T)pa, (char *)&p, sizeof(p))) {
+		(void) sprintf(Namech,
+			"can't read DTYPE_PIPE pipe struct: %#x", pa);
+		enter_nm(Namech);
+		return;
+	}
+	(void) strcpy(Lf->type, "PIPE");
+	(void) sprintf(dev_ch, "%#x", pa);
+	enter_dev_ch(dev_ch);
+	if (Foffset)
+		Lf->off_def = 1;
+	else {
+		Lf->sz = p.pipe_buffer.size;
+		Lf->sz_def = 1;
+	}
+	if (p.pipe_peer)
+		(void) sprintf(Namech, "->%#x", p.pipe_peer);
+	else
+		Namech[0] = '\0';
+	if (p.pipe_buffer.cnt)
+		(void) sprintf(endnm(), "%, cnt=%d", p.pipe_buffer.cnt);
+	if (p.pipe_buffer.in)
+		(void) sprintf(endnm(), "%, in=%d", p.pipe_buffer.in);
+	if (p.pipe_buffer.out)
+		(void) sprintf(endnm(), "%, out=%d", p.pipe_buffer.out);
+/*
+ * Enter name characters.
+ */
+	if (Namech[0])
+		enter_nm(Namech);
+}
+#endif	/* FREEBSDV>=220 */
