@@ -32,24 +32,14 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dproc.c,v 1.12 95/09/19 10:14:46 abe Exp $";
+static char *rcsid = "$Id: dproc.c,v 1.2 97/04/15 10:38:46 abe Exp $";
 #endif
 
 #include "lsof.h"
 
-#if     defined(HASNCACHE)
-#include <sys/uio.h>
-#include <sys/namei.h>
-#endif  /* defined(HASNCACHE) */
-
 
 _PROTOTYPE(static void enter_vn_text,(KA_T va, int *n));
 _PROTOTYPE(static void get_kernel_access,(void));
-
-#if     defined(HASNCACHE)
-_PROTOTYPE(static void ncache_load,(void));
-#endif  /* defined(HASNCACHE) */
-
 _PROTOTYPE(static void process_text,(KA_T vm));
 
 
@@ -82,6 +72,7 @@ enter_vn_text(va, n)
  * Save the text file information.
  */
 	alloc_lfile(" txt", -1);
+	Cfp = (struct file *)NULL;
 	process_node((caddr_t)va);
 	if (Lf->sf)
 		link_lfile();
@@ -97,7 +88,7 @@ enter_vn_text(va, n)
 				(void) fprintf(stderr,
 					"%s: no txt ptr space, PID %d\n",
 					Pn, Lp->pid);
-				exit(1);
+				Exit(1);
 			}
 			Nv = 10;
 		} else {
@@ -108,7 +99,7 @@ enter_vn_text(va, n)
 				(void) fprintf(stderr,
 					"%s: no more txt ptr space, PID %d\n",
 					Pn, Lp->pid);
-				exit(1);
+				Exit(1);
 			}
 		}
 	}
@@ -132,45 +123,42 @@ gather_proc_info()
 	static int nofb = 0;
 	static struct file **ofb = NULL;
 	int pgrp, pid;
+	int ppid = 0;
 	short pss, sf;
 	int px;
 	uid_t uid;
 
-#if	_FREEBSDV<200
+#if	FREEBSDV<200
 	struct proc *p;
 	struct pcred pc;
 	struct pgrp pg;
-#else	/* _FREEBSDV>=200 */
+#else	/* FREEBSDV>=200 */
 	struct kinfo_proc *p;
-#endif	/* _FREEBSDV<200 */
+#endif	/* FREEBSDV<200 */
 
 /*
  * Read the process table.
  */
 
-#if	_FREEBSDV<200
+#if	FREEBSDV<200
 	if ((Np = kvm_getprocs(KINFO_PROC_ALL, 0)) < 0)
-#else	/* _FREEBSDV>=200 */
+#else	/* FREEBSDV>=200 */
 	if ((P = kvm_getprocs(Kd, KERN_PROC_ALL, 0, &Np)) == NULL)
-#endif	/* _FREEBSDV<200 */
+#endif	/* FREEBSDV<200 */
 
 	{
 		(void) fprintf(stderr, "%s: can't read process table: %s\n",
 			Pn,
 
-#if	_FREEBSDV<200
+#if	FREEBSDV<200
 			kvm_geterr()
-#else	/* _FREEBSDV>=200 */
+#else	/* FREEBSDV>=200 */
 			kvm_geterr(Kd)
-#endif	/* _FREEBSDV<200 */
+#endif	/* FREEBSDV<200 */
 
 		);
-		exit(1);
+		Exit(1);
 	}
-/*
- * Initialize the UID cache.
- */
-	(void) iuidcache(Np);
 
 #if	defined(HASNCACHE)
 /*
@@ -183,21 +171,23 @@ gather_proc_info()
  * Examine proc structures and their associated information.
  */
 
-#if	_FREEBSDV<200
+#if	FREEBSDV<200
 	for (px = 0; px < Np; px++)
-#else	/* _FREEBSDV>=200 */
+#else	/* FREEBSDV>=200 */
 	for (p = P, px = 0; px < Np; p++, px++)
-#endif	/* _FREEBSDV<200 */
+#endif	/* FREEBSDV<200 */
 
 	{
 
-#if	_FREEBSDV<200
+#if	FREEBSDV<200
 	/*
 	 * Read process information, process group structure (if
-	 * necessary),  and User ID (if necessary).
+	 * necessary), and User ID (if necessary).
 	 */
 		if ((p = kvm_nextproc()) == NULL)
 			continue;
+		if (p->P_STAT == 0 || p->P_STAT == SZOMB)
+			continue;
 		pg.pg_id = 0;
 		if (Fpgrp && p->p_pgrp) {
 			if (kread((KA_T)p->p_pgrp, (char *)&pg, sizeof(pg)))
@@ -208,10 +198,16 @@ gather_proc_info()
 		||  kread((KA_T)p->p_cred, (char *)&pc, sizeof(pc)))
 			continue;
 		uid = pc.p_ruid;
-#else	/* _FREEBSDV>=200 */
+#else	/* FREEBSDV>=200 */
+		if (p->P_STAT == 0 || p->P_STAT == SZOMB)
+			continue;
 		pgrp = p->P_PGRP;
 		uid = p->kp_eproc.e_ucred.cr_uid;
-#endif	/* _FREEBSDV<200 */
+#endif	/* FREEBSDV<200 */
+
+#if	defined(HASPPID)
+		ppid = p->P_PPID;
+#endif	/* defined(HASPPID) */
 
 	/*
 	 * See if process is excluded.
@@ -223,32 +219,34 @@ gather_proc_info()
 		if (p->P_FD == NULL
 		||  kread((KA_T)p->P_FD, (char *)&fd, sizeof(fd)))
 			continue;
+		if (!fd.fd_refcnt || fd.fd_lastfile > fd.fd_nfiles)
+			continue;
 	/*
 	 * Allocate a local process structure.
 	 */
 		if (is_cmd_excl(p->P_COMM, &pss, &sf))
 			continue;
-		alloc_lproc(p->P_PID, pgrp, (UID_ARG)uid, p->P_COMM,
+		alloc_lproc(p->P_PID, pgrp, ppid, (UID_ARG)uid, p->P_COMM,
 			(int)pss, (int)sf);
 		Plf = NULL;
+
+#if	defined(P_ADDR)
+	/*
+	 * Save the kernel proc struct address, if P_ADDR is defined.
+	 */
+		Kpa = (KA_T)p->P_ADDR;
+#endif	/* defined(P_ADDR) */
+
 	/*
 	 * Save current working directory information.
 	 */
 		if (fd.fd_cdir) {
 			alloc_lfile(CWD, -1);
+			Cfp = (struct file *)NULL;
 			process_node((caddr_t)fd.fd_cdir);
 			if (Lf->sf)
 				link_lfile();
 		}
-	/*
-	 * Save root directory information.
-	 */
-		if (fd.fd_rdir != NULL) {
-			alloc_lfile(RTD, -1);
-			process_node((caddr_t)fd.fd_rdir);
-			if (Lf->sf)
-				link_lfile();
-		}
 	/*
 	 * Save information on the text file.
 	 */
@@ -267,17 +265,17 @@ gather_proc_info()
 				(void) fprintf(stderr,
 					"%s: PID %d, no file * space\n",
 					Pn, p->P_PID);
-				exit(1);
+				Exit(1);
 			}
 		} else if (nf > nofb) {
-			nofb = fd.fd_nfiles;
+			nofb = nf;
 			if ((ofb = (struct file **)realloc((MALLOC_P *)ofb,
 				    (nofb * sizeof(struct file *))))
 			== NULL) {
 				(void) fprintf(stderr,
 					"%s: PID %d, no realloc file * space\n",
 					Pn, p->P_PID);
-					exit(1);
+				Exit(1);
 			}
 		}
 		if (kread((KA_T)fd.fd_ofiles, (char *)ofb,
@@ -289,7 +287,7 @@ gather_proc_info()
 		for (i = 0; i < nf; i++) {
 			if (ofb[i]) {
 				alloc_lfile(NULL, i);
-				process_file(ofb[i]);
+				process_file((Cfp = ofb[i]));
 				if (Lf->sf)
 					link_lfile();
 			}
@@ -310,28 +308,23 @@ gather_proc_info()
 static void
 get_kernel_access()
 {
-	char *nunix = N_UNIX;
-	int nunix_stat = 0;
 
-#if	_FREEBSDV>=200
 /*
- * Set name list file path to booted kernel.
+ * Set name list file path.
  */
-	if (!Nmlst) {
-		const char *bf;
-		char *bfc;
-		MALLOC_S len;
+	if (!Nmlst)
 
-		if ((bf = getbootfile()) != NULL) {
-			len = (MALLOC_S)strlen(bf);
-			if ((bfc = (char *)malloc(len + 1)) != NULL) {
-				(void) strcpy(bfc, bf);
-				nunix = bfc;
-				nunix_stat = 1;
-			}
+#if	defined(N_UNIX)
+		Nmlst = N_UNIX;
+#else	/* !defined(N_UNIX) */
+	{
+		if (!(Nmlst = get_nlist_path(1))) {
+			(void) fprintf(stderr,
+				"%s: can't get kernel name list path\n", Pn);
+			Exit(1);
 		}
 	}
-#endif	/* _FREEBSDV>=200 */
+#endif	/* defined(N_UNIX) */
 
 #if	defined(WILLDROPGID)
 /*
@@ -346,39 +339,40 @@ get_kernel_access()
  */
 	if ((Memory && !is_readable(Memory, 1))
 	||  (Nmlst && !is_readable(Nmlst, 1)))
-		exit(1);
+		Exit(1);
 #endif	/* defined(WILLDROPGID) */
 
 /*
  * Open kernel memory access.
  */
 
-#if	_FREEBSDV<200
+#if	FREEBSDV<200
 	if (kvm_openfiles(Nmlst, Memory, NULL) == -1)
-#else	/* _FREEBSDV>=200 */
+#else	/* FREEBSDV>=200 */
 	if ((Kd = kvm_open(Nmlst, Memory, NULL, O_RDONLY, NULL)) == NULL)
-#endif	/* _FREEBSDV<200 */
+#endif	/* FREEBSDV<200 */
 
 	{
 		(void) fprintf(stderr,
 			"%s: kvm_open%s (namelist=%s, core = %s): %s\n",
 			Pn,
 
-#if	_FREEBSDV<200
+#if	FREEBSDV<200
 			"files",
-#else	/* _FREEBSDV>=200 */
+#else	/* FREEBSDV>=200 */
 			"",
-#endif	/* _FREEBSDV<200 */
+#endif	/* FREEBSDV<200 */
 
 			Nmlst ? Nmlst : "default",
 			Memory  ? Memory  : "default",
 			strerror(errno));
-		exit(1);
+		Exit(1);
 	}
-	if (nlist(Nmlst ? Nmlst : nunix, Nl) < 0) {
+	(void) build_Nl(Drive_Nl);
+	if (nlist(Nmlst, Nl) < 0) {
 		(void) fprintf(stderr, "%s: can't read namelist from %s\n",
-			Pn, Nmlst ? Nmlst : nunix);
-		exit(1);
+			Pn, Nmlst);
+		Exit(1);
 	}
 
 #if	defined(WILLDROPGID)
@@ -389,14 +383,45 @@ get_kernel_access()
 		(void) dropgid();
 #endif	/* defined(WILLDROPGID) */
 
-/*
- * If space was allocated for the name list file path, free it.
- */
-	if (nunix_stat)
-		(void) free((FREE_P *)nunix);
 }
 
 
+#if	!defined(N_UNIX)
+/*
+ * get_nlist_path() - get kernel name list path
+ */
+
+char *
+get_nlist_path(ap)
+	int ap;				/* on success, return an allocated path
+					 * string pointer if 1; return a
+					 * constant character pointer if 0;
+					 * return NULL if failure */
+{
+	const char *bf;
+	static char *bfc;
+	MALLOC_S bfl;
+/*
+ * Get bootfile name.
+ */
+	if ((bf = getbootfile())) {
+	    if (!ap)
+		return("");
+	    bfl = (MALLOC_S)(strlen(bf) + 1);
+	    if (!(bfc = (char *)malloc(bfl))) {
+		(void) fprintf(stderr,
+		    "%s: can't allocate %d bytes for boot file path: %s\n",
+		    Pn, bfl, bf);
+		Exit(1);
+	    }
+	    (void) strcpy(bfc, bf);
+	    return(bfc);
+	}
+	return((char *)NULL);
+}
+#endif	/* !defined(N_UNIX) */
+
+
 /*
  * initialize() - perform all initialization
  */
@@ -420,11 +445,11 @@ kread(addr, buf, len)
 {
 	int br;
 
-#if	_FREEBSDV<200
+#if	FREEBSDV<200
 	br = kvm_read((void *)addr, (void *)buf, len);
-#else	/* _FREEBSDV>=200 */
+#else	/* FREEBSDV>=200 */
 	br = kvm_read(Kd, (u_long)addr, buf, len);
-#endif	/* _FREEBSDV<200 */
+#endif	/* FREEBSDV<200 */
 
 	return((br == len) ? 0 : 1);
 }
@@ -440,10 +465,14 @@ process_text(vm)
 	int i, j;
 	KA_T ka;
 	int n = 0;
-	struct pager_struct pg;
 	struct vm_map_entry vmme, *e;
 	struct vm_object vmo;
 	struct vmspace vmsp;
+
+#if	FREEBSDV<220
+	struct pager_struct pg;
+#endif	/* FREEBSDV<220 */
+
 /*
  * Read the vmspace structure for the process.
  */
@@ -468,47 +497,47 @@ process_text(vm)
 			if (kread(ka, (char *)e, sizeof(vmme)))
 				return;
 		}
+
+#if	defined(MAP_ENTRY_IS_A_MAP)
+		if (e->eflags & (MAP_ENTRY_IS_A_MAP|MAP_ENTRY_IS_SUB_MAP))
+#else	/* !defined(MAP_ENTRY_IS_A_MAP) */
 		if (e->is_a_map || e->is_sub_map)
+#endif	/* defined(MAP_ENTRY_IS_A_MAP) */
+
 			continue;
 	/*
 	 * Read the map entry's object and the object's shadow.
-	 * Look for a PG_VNODE pager handle.
+	 * Look for: a PG_VNODE pager handle (FreeBSD < 2.2);
+	 * an OBJT_VNODE object type (FreeBSD >= 2.2).
 	 */
 		for (j = 0, ka = (KA_T)e->object.vm_object;
 		     j < 2 && ka;
-		     j++, ka = (KA_T)vmo.shadow)
+		     j++,
+
+#if	FREEBSDV<220
+		     ka = (KA_T)vmo.shadow
+#else	/* FREEBSDV>=220 */
+		     ka = (KA_T)vmo.backing_object
+#endif	/* FREEBSDV<220 */
+		     )
 		{
 			if (kread(ka, (char *)&vmo, sizeof(vmo)))
 				break;
+
+#if	FREEBSDV<220
 			if ((ka = (KA_T)vmo.pager) == NULL
 			||   kread(ka, (char *)&pg, sizeof(pg)))
 				continue;
 			if (pg.pg_handle == NULL || pg.pg_type != PG_VNODE)
 				continue;
 			(void) (enter_vn_text((KA_T)pg.pg_handle, &n));
+#else	/* FREEBSDV>=220 */
+			if (vmo.type != OBJT_VNODE
+			||  vmo.handle == (void *)NULL)
+				continue;
+			(void) (enter_vn_text((KA_T)vmo.handle, &n));
+#endif	/* FREEBSDV<220 */
+
 		}
 	}
 }
-
-
-/*
- * The ncache_addr(), ncache_load(), and ncache_lookup() functions are
- * obtained from ../common/rnam.frag.
- */
-
-#if	defined(HASNCACHE)
-#define	NCACHE		namecache	/* kernel's structure name */
-#define	NCACHE_NM	nc_name		/* name in NCACHE */
-#define	NCACHE_NMLEN	nc_nlen		/* name length in NCACHE */
-# if	_FREEBSDV<205
-#define	NCACHE_NXT	nc_nxt		/* link in NCACHE */
-# else	/* _FREEBSDV>=205 */
-#define	NCACHE_NXT	nc_lru.tqe_next	/* link in NCACHE */
-# endif	/* _FREEBSDV<205 */
-#define	NCACHE_NODEADDR	nc_vp		/* node address in NCACHE */
-#define	NCACHE_NODEID	nc_vpid		/* node ID in NCACHE */
-#define	NCACHE_PARADDR	nc_dvp		/* parent node address in NCACHE */
-#define	NCACHE_PARID	nc_dvpid	/* parent node ID in NCACHE */
-#endif	/* defined(HASNCACHE) */
-
-
