@@ -30,10 +30,44 @@
  */
 
 
+/*
+ * $Id: machine.h,v 1.8 97/09/23 09:47:44 abe Exp $
+ */
+
+
+#if	!defined(MACHINE_H)
+#define	MACHINE_H	1
+
+
 #include <sys/types.h>
 #include <sys/param.h>
 
 
+/*
+ * GET_MAX_FD is defined for those dialects that provide a function other than
+ * getdtablesize() to obtain the maximum file descriptor number plus one.
+ */
+
+/* #define	GET_MAX_FD	?	*/
+
+
+/*
+ * HASAOPT is defined for those dialects that have AFS support; it specifies
+ * that the default path to an alternate AFS kernel name list file may be
+ * supplied with the -A <path> option.
+ */
+
+/* #define	HASAOPT		1 */
+
+
+/*
+ * HASBLKDEV is defined for those dialects that want block device information
+ * recorded in BDevtp[].
+ */
+
+#define	HASBLKDEV	1
+
+
 /*
  * HASDCACHE is defined for those dialects that support a device cache
  * file.
@@ -54,11 +88,14 @@
  *
  * HASSYSDC defines a public device cache file path.  When it's defined, it's
  * used as the path from which to read the device cache.
+ *
+ * Consult the 00DCACHE and 00FAQ files of the lsof distribution for more
+ * information on device cache file path construction.
  */
 
 #define	HASDCACHE	1
 #define	HASENVDC	"LSOFDEVCACHE"
-#define	HASPERSDC	".lsof_%L"
+#define	HASPERSDC	"%h/%p.lsof_%L"
 #define	HASPERSDCPATH	"LSOFPERSDCPATH"
 /* #define	HASSYSDC	"/var/adm/lsof.dc" */
 
@@ -77,6 +114,14 @@
 /* #define	HASFIFONODE	1 */
 
 
+/*
+ * HASFSINO is defined for those dialects that have the file system
+ * inode element, fs_ino, in the lfile structure definition in lsof.h.
+ */
+
+/* #define	HASFSINO	1 */
+
+
 /*
  * HASGNODE is defined for those dialects that have gnodes.
  */
@@ -92,7 +137,8 @@
 
 
 /*
- * HASINODE is defined for those dialects that have inodes.
+ * HASINODE is defined for those dialects that have inodes and wish to
+ * use readinode() from node.c.
  */
 
 #define	HASINODE	1
@@ -120,7 +166,7 @@
  * them.  If any of the additional elements need to be preset in the
  * alloc_lfile() function of proc.c, the SETLFILEADD macro may be defined
  * to do that.  The HASXOPT definition may be used to select the conditions
- * under which provate lfile elements are used.
+ * under which private lfile elements are used.
  *
  * Field identifiers 1 through 9 are reserved for dialect-specific field
  * output.  Define the string HASFIELDAP<x> as the help (-F?) name of the
@@ -166,6 +212,19 @@
 #define	HASNLIST	1
 
 
+/*
+ * HASPIPEFN is defined for those dialects that have a special function to
+ * process DTYPE_PIPE file structure entries.  Its value is the name of the
+ * function.
+ *
+ * NOTE: don't forget to define a prototype for this function in dproto.h.
+ */
+
+# if	FREEBSDV>=220
+#define	HASPIPEFN	process_pipe
+# endif	/* FREEBSDV>=220 */
+
+
 /*
  * HASPIPENODE is defined for those dialects that have pipe nodes.
  */
@@ -173,9 +232,36 @@
 /* #define	HASPIPENODE	1 */
 
 
+/*
+ * HASPPID is defined for those dialects that support identification of
+ * the parent process IDentifier (PPID) of a process.
+ */
+
+# if	FREEBSDV>=200
+#define	HASPPID		1
+# endif	/* FREEBSDV>=200 */
+
+
+/*
+ * HASPRIVFILETYPE and PRIVFILETYPE are defined for dialects that have a
+ * file structure type that isn't defined by a DTYPE_* symbol.  They are
+ * used in lib/prfp.c to select the type's processing.
+ *
+ * PRIVFILETYPE is the definition of the f_type value in the file struct.
+ *
+ * HASPRIVFILETYPE is the name of the processing function.
+ */
+
+/* #define	HASPRIVFILETYPE	process_shmf	*/
+/* #define	PRIVFILETYPE	??	*/
+
+
 /*
  * HASPROCFS is defined for those dialects that have a proc file system --
- * usually /proc and usually in SYSV4 derivatives.
+ * usually /proc and usually in SYSV4 derivatives.  For FreeBSD 2.0 and
+ * above the Configure script defines HASPROCFS when it can find
+ * /usr/src/sys/miscfs/procfs/procfs.h; below 2.0, Configure always defines
+ * HASPROCFS.
  *
  * HASFSTYPE is defined for those systems that have a file system type
  * string, st_fstype, in the stat() buffer.
@@ -195,23 +281,13 @@
  * by inode number.
  */
 
-#define		HASPROCFS	"proc"
+/* #define		HASPROCFS	"proc" */
 /* #define		HASFSTYPE	1 */
 /* #define		HASPINFO	"/pinfo" */
 
-#if	_FREEBSDV>=200
+# if	FREEBSDV>=200
 #define	HASPINODEN
-#endif	/* _FREEBSDV>=200 */
-
-
-/*
- * HASPWSTAYOPEN is defined for those dialects that have a method to hold
- * the password data base open between searches.  Setting this definition
- * implies the availability of a dialect-specific pw_stay_open() function
- * to implement the method.
- */
-
-/* #define	HASPWSTAYOPEN	1 */
+# endif	/* FREEBSDV>=200 */
 
 
 /*
@@ -245,6 +321,22 @@
 /* #define	HASSTREAMS	1 */
 
 
+/*
+ * HASTCPTPIQ is defined for dialects where it is possible to report the
+ * TCP/TPI Recv-Q and Send-Q values produced by netstat.
+ */
+
+#define	HASTCPTPIQ	1
+
+
+/*
+ * HASTCPTPIW is defined for dialects where it is possible to report the
+ * TCP/TPI send and receive window sizes produced by netstat.
+ */
+
+/* #define	HASTCPTPIW	1 */
+
+
 /*
  * HASTMPNODE is defined for those dialects that have tmpnodes.
  */
@@ -280,6 +372,37 @@
 #define	UID_ARG	int
 
 
+/*
+ * Each USE_LIB_<function_name> is defined for dialects that use the
+ * <function_name> in the lsof library.
+ *
+ * Note: other definitions and operations may be required to condition the
+ * library function source code.  They may be found in the dialect dlsof.h
+ * header files.
+ */
+
+/* #define	USE_LIB_CK_FILE_ARG		1	   ckfa.c */
+/* #define	USE_LIB_COMPLETEVFS		1	   cvfs.c */
+#define	USE_LIB_FIND_CH_INO			1	/* fino.c */
+#define	USE_LIB_IS_FILE_NAMED			1	/* isfn.c */
+#define	USE_LIB_LKUPDEV				1	/* lkud.c */
+#define	USE_LIB_PRINTCHDEVNAME			1	/* pdvn.c */
+#define	USE_LIB_PROCESS_FILE			1	/* prfp.c */
+/* #define	USE_LIB_PRINT_FILE		1	   prtf.c */
+#define	USE_LIB_PRINT_TCPTPI			1	/* ptti.c */
+#define	USE_LIB_READDEV				1	/* rdev.c */
+/* #define	USE_LIB_READMNT			1	   rmnt.c */
+/* #define	USE_LIB_READVFS			1	   rvfs.c */
+
+#if	FREEBSDV<210
+#define	USE_LIB_RNAM				1	/* rnam.c */
+#else	/* FREEBSDV>=210 */
+#define	USE_LIB_RNMH				1	/* rnmh.c */
+#endif	/* FREEBSDV<210 */
+
+/* #define	USE_LIB_RNCH			1	   rnch.c */
+
+
 /*
  * WARNDEVACCESS is defined for those dialects that should issue a warning
  * when lsof can't access /dev (or /device) or one of its sub-directories.
@@ -289,6 +412,14 @@
 #define	WARNDEVACCESS	1
 
 
+/*
+ * WARNINGSTATE is defined for those dialects that want to suppress all lsof
+ * warning messages.
+ */
+
+/* #define	WARNINGSTATE	1	warnings are enabled by default */
+
+
 /*
  * WILLDROPGID is defined for those dialects whose lsof executable runs
  * setgid(not_real_GID) and whose setgid power can be relinquished after
@@ -302,4 +433,6 @@
  * zeromem is a macro that uses bzero or memset.
  */
 
-#define	zeromem(a, l)	bzero(a, l)
+#define	zeromem(a, l)	memset(a, 0, l)
+
+#endif	/* !defined(MACHINE_H) */
