@@ -30,6 +30,11 @@
  */
 
 
+/*
+ * $Id: dlsof.h,v 1.2 97/08/15 08:19:49 abe Exp $
+ */
+
+
 #if	!defined(HPUX_LSOF_H)
 #define	HPUX_LSOF_H	1
 
@@ -40,17 +45,36 @@
 #include <string.h>
 #include <nlist.h>
 #include <unistd.h>
+
+# if	HPUXV<1020
 #include <sys/vnode.h>
+# endif	/* HPUXV<1020 */
+
 #include <sys/domain.h>
+
+# if	HPUXV>=1020
+#define	_INCLUDE_STRUCT_FILE	1
+# endif	/* HPUXV>=1020 */
+
 #include <sys/file.h>
+
+# if	HPUXV>=1020
+#undef	_INCLUDE_STRUCT_FILE
+# endif	/* HPUXV>=1020 */
+
 #include <sys/mbuf.h>
 
-# if	_HPUXV>=800
+# if	HPUXV>=800
+#undef	_PROTOTYPES
 #include <sys/pstat.h>
-# endif	/* _HPUXV>=800 */
+# endif	/* HPUXV>=800 */
 
 #include <sys/resource.h>
+
+# if	HPUXV<1010
 #include <sys/proc.h>
+# endif	/* HPUXV<1010 */
+
 #include <sys/protosw.h>
 #include <sys/socket.h>
 #include <sys/socketvar.h>
@@ -58,26 +82,33 @@
 #include <net/route.h>
 #include <net/raw_cb.h>
 #include <netinet/in_pcb.h>
+#include <netinet/ip_var.h>
+#include <netinet/tcp.h>
+#include <netinet/tcpip.h>
+#include <netinet/tcp_fsm.h>
+#include <netinet/tcp_timer.h>
+#include <netinet/tcp_var.h>
 #include <rpc/types.h>
 
-# if	_HPUXV>=1000
+# if	HPUXV>=1000
+#include <rpc/xdr.h>
 #include <rpc/auth.h>
 #include <rpc/clnt.h>
 #include <sys/cdfsdir.h>
 #include <sys/cdfs.h>
 #include <sys/cdnode.h>
-# endif	/* _HPUXV>=1000 */
+# endif	/* HPUXV>=1000 */
 
 #include <nfs/nfs.h>
 #include <nfs/nfs_clnt.h>
 #include <nfs/rnode.h>
 #include <nfs/snode.h>
 
-# if	_HPUXV>=1000
+# if	HPUXV>=1000
 #define	_KERNEL
 #include <nfs/fifonode.h>
 #undef	_KERNEL
-# endif	/* _HPUXV>=1000 */
+# endif	/* HPUXV>=1000 */
 
 # if	defined(DTYPE_LLA)
 #define	_KERNEL	1
@@ -106,7 +137,7 @@ struct mount {
 	dev_t	m_dev;		 /* device mounted */
    };
 
-# if	_HPUXV>=800
+# if	HPUXV>=800
 /*
  * These definitions are from <sys/vfs.h>, defined under the _KERNEL symbol.
  * Unfortunately, defining _KERNEL causes <sys/vfs.h> to include other
@@ -115,121 +146,118 @@ struct mount {
 #define MOUNT_UFS 0
 #define MOUNT_NFS 1
 #define MOUNT_CDFS 2
-# endif	/* _HPUXV>=800 */
+# endif	/* HPUXV>=800 */
 
+# if	defined(HAS_CONST)
+#define	COMP_P		const void
+# else	/* !defined(HAS_CONST) */
 #define	COMP_P		void
+# endif	/* defined(HAS_CONST) */
 
-# if	_HPUXV>=800
+# if	HPUXV>=800
 #define	CURDIR	p->p_cdir
-#define	FILEPTR	oft.ofile[j-1]
 #define	ROOTDIR	p->p_rdir
-# else	/* _HPUXV<800 */
+# else	/* HPUXV<800 */
 #define CURDIR	u->u_cdir
-#define	FILEPTR	u->u_ofile.ofile[j-1]
 #define	ROOTDIR	u->u_rdir
-# endif	/* _HPUXV>=800 */
+# endif	/* HPUXV>=800 */
 
 #define DEVINCR		1024	/* device table malloc() increment */
 #define	KA_T		off_t
 #define	KMEM		"/dev/kmem"
-#define MALLOC_P	char
+#define MALLOC_P	void
 #define FREE_P		void
 #define MALLOC_S	unsigned
 #define MOUNTED		MNT_MNTTAB
 
-# if	_HPUXV<1000
+# if	HPUXV<1000
 #define N_UNIX		"/hp-ux"
-# else	/* _HPUXV>=1000 */
+# else	/* HPUXV>=1000 */
 #define N_UNIX		"/stand/vmunix"
-# endif	/* _HPUXV<1000 */
+# endif	/* HPUXV<1000 */
 
 #define QSORT_P		void
 #define	READLEN_T	int
 #define STRNCPY_L	size_t
 #define SWAP		"/dev/swap"
 
-# if	_HPUXV<800
+# if	HPUXV<800
 #define unp_addr	unp_locaddr
 /*
  * HP-UX <8 SWAP must be read in DEV_BSIZE chunks.
  */
 #define U_SIZE		(((DEV_BSIZE+sizeof(struct user))/DEV_BSIZE)*DEV_BSIZE)
-# endif	/* _HPUXV<800 */
+# endif	/* HPUXV<800 */
 
-# if	_HPUXV>=800
+# if	HPUXV>=800
 #define	U_SIZE		sizeof(struct user)
-# endif	/* _HPUXV>=800 */
+# endif	/* HPUXV>=800 */
+
+
+# if	defined(HAS_AFS)
+/*
+ * AFS definitions
+ */
+
+#define	AFSAPATHDEF	"/usr/adm/afs/kload"
+#define	AFSDEV		1		/* AFS "fake" device number */
+
+#  if	defined(HASAOPT)
+extern char *AFSApath;			/* alternate AFS name list path
+					 * (from -A) */
+#  endif	/* defined(HASAOPT) */
+
+extern struct vfs *AFSVfsp;		/* AFS struct vfs kernel pointer */
+# endif	/* defined(HAS_AFS) */
 
 
 /*
  * Global storage definitions (including their structure definitions)
  */
 
-extern int Kmem;
+extern int Kd;
+extern KA_T Kpa;
+
+# if	HPUXV>=1010
+extern caddr_t Ktp;
+#endif	/* HPUXV>=1010 */
 
 struct l_vfs {
 	struct vfs *addr;		/* kernel address */
 	dev_t dev;			/* device */
 	char *dir;			/* mounted directory */
 	char *fsname;			/* file system name */
+
+# if	defined(HASFSINO)
+	ino_t fs_ino;			/* file system inode number */
+# endif	/* defined(HASFSINO) */
+
 	struct l_vfs *next;		/* forward link */
 };
 extern struct l_vfs *Lvfs;
 
-# if	_HPUXV<800
+# if	HPUXV<800
 extern int Mem;
-# endif	/* _HPUXV<800 */
+# endif	/* HPUXV<800 */
 
 struct mounts {
-        dev_t dev;              	/* st_dev */
-        char *dir;              	/* directory */
-        char *fsname;           	/* file system */
+	dev_t dev;              	/* st_dev */
+	char *dir;              	/* directory */
+	char *fsname;           	/* file system */
 	ino_t inode;			/* st_ino */
 	mode_t mode;			/* st_mode */
-        struct mounts *next;    	/* forward link */
+	struct mounts *next;    	/* forward link */
 	dev_t rdev;			/* st_rdev */
 };
 
-# if	defined(hp9000s300)
-#define	X_NCACHE	0
-#define	X_NCSIZE	1
-#define X_PROC          2
-#define X_NFS_OPS	3
-#define X_NPROC         4
-#define X_VN_FOPS	5
+#define	X_NCACHE	"ncache"
+#define	X_NCSIZE	"ncsize"
+#define	NL_NAME		n_name
 
-#  if	_HPUXV<800
-#define X_USRPTMAP	6
-#define X_USRPT		7
-#  endif	/* _HPUXV<800 */
-# endif	/* defined(hp9000s300) */
-
-# if	defined(hp9000s800)
-#define	X_NCACHE	0
-#define	X_NCSIZE	1
-#define X_PROC          2
-#define X_NFS_OPS	3
-#define X_NPROC         4
-#define X_VN_FOPS	5
-
-# if	_HPUXV<800
-#define X_UBASE		6
-#define X_NPIDS		7
-#  else	/* _HPUXV>=800 */
-#   if	_HPUXV>=1000
-#define	X_CDFS_OPS	6
-#define	X_FIFO_OPS	7
-#define	X_PIPE_OPS	8
-#define	X_SPEC_OPS	9
-#define X_VXFS_OPS	10
-#   endif	/* _HPUXV>=1000 */
-#  endif	/* _HPUXV<800 */
-# endif	defined(hp9000s800)
-
-# if	_HPUXV<800 && defined(hp9000s800)
+# if	HPUXV<800 && defined(hp9000s800)
 extern int npids;
 extern struct proc *proc;
-# endif	/* _HPUXV<800 && defined(hp9000s800) */
+# endif	/* HPUXV<800 && defined(hp9000s800) */
 
 struct sfile {
 	char *aname;			/* argument file name */
@@ -244,19 +272,47 @@ struct sfile {
 	struct sfile *next;		/* forward link */
 };
 
-# if	_HPUXV<800
+# if	HPUXV<800
 extern int Swap;
-# endif	/* _HPUXV<800 */
+# endif	/* HPUXV<800 */
 
-# if	_HPUXV<800 && defined(hp9000s800)
+# if	HPUXV<800 && defined(hp9000s800)
 extern struct user *ubase;
-# endif	/* _HPUXV<800 && defined(hp9000s800) */
+# endif	/* HPUXV<800 && defined(hp9000s800) */
 
-# if	_HPUXV<800 && defined(hp9000s300)
+# if	HPUXV<800 && defined(hp9000s300)
 extern struct pte *Usrptmap;
 extern struct pte *usrpt;
-# endif	/* _HPUXV<800 && defined(hp9000s300) */
+# endif	/* HPUXV<800 && defined(hp9000s300) */
 
 extern struct fileops *Vnfops;
 
+
+/*
+ * Definitions for rdev.c
+ */
+
+#define	DIRTYPE	dirent
+#define HASDNAMLEN	1	/* struct DIRTYPE has d_namlen element */
+
+
+/*
+ * Definition for rmnt.c
+ */
+
+#define MNTSKIP \
+	{ if (strcmp(mp->mnt_type, MNTTYPE_IGNORE) == 0) \
+		continue; }
+
+/*
+ * Definitions for rnch.c
+ */
+
+# if     defined(HASNCACHE)
+#include <sys/dnlc.h>
+#  if	HPUXV<1000
+#define	ADDR_NCACHE	1
+#  endif	/* HPUXV<1000 */
+# endif  /* defined(HASNCACHE) */
+
 #endif	/* HPUX_LSOF_H */
