@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dproc.c,v 1.12 95/09/19 10:16:21 abe Exp $";
+static char *rcsid = "$Id: dproc.c,v 1.3 97/07/31 09:54:59 abe Exp $";
 #endif
 
 #include "lsof.h"
@@ -42,6 +42,65 @@ static char *rcsid = "$Id: dproc.c,v 1.12 95/09/19 10:16:21 abe Exp $";
 #endif  /* defined(HASNCACHE) */
 
 
+#if	HPUXV>=1010
+/*
+ * HP doesn't include a definition for the proc structure in HP-UX 10.10
+ * or above in an attempt to force use of pstat(2).  Unfortunately, pstat(2)
+ * doesn't return the information lsof needs.  Hence, this private proc
+ * structure definition.
+ */
+
+#include <sys/vas.h>
+
+#define	SZOMB	3
+
+# if	HPUXV<1020
+struct proc {
+	caddr_t d1[2];			/* dummy to occupy space */
+	caddr_t p_firstthreadp;		/* thread pointer */
+	caddr_t d2[4];			/* dummy to occupy space */
+	int p_stat;			/* process status */
+	caddr_t d3[9];			/* dummy to occupy space */
+	uid_t p_uid;			/* UID */
+	caddr_t d4[2];			/* dummy to occupy space */
+	gid_t p_pgrp;			/* process group ID */
+	pid_t p_pid;			/* PID */
+	pid_t p_ppid;			/* parent PID */
+	caddr_t d5[9];			/* dummy to occupy space */
+	vas_t *p_vas;			/* virtual address space */
+	caddr_t d6[16];			/* dummy to occupy space */
+	int p_maxof;			/* max open files allowed */
+	struct vnode *p_cdir;		/* current directory */
+	struct vnode *p_rdir;		/* root directory */
+	struct ofile_t **p_ofilep;	/* file descriptor chunks */
+	caddr_t d7[43];			/* dummy to occupy space */
+};
+# else	/* HPUXV>=1020 */
+struct proc {
+	caddr_t d1[2];			/* dummy to occupy space */
+	caddr_t p_firstthreadp;		/* thread pointer */
+	caddr_t d2[6];			/* dummy to occupy space */
+	int p_stat;			/* process status */
+	caddr_t d3[14];			/* dummy to occupy space */
+	uid_t p_uid;			/* real UID */
+	uid_t p_suid;			/* effective UID */
+	caddr_t d4;			/* dummy to occupy space */
+	gid_t p_pgrp;			/* process group ID */
+	pid_t p_pid;			/* PID */
+	pid_t p_ppid;			/* parent PID */
+	caddr_t d5[9];			/* dummy to occupy space */
+	vas_t *p_vas;			/* virtual address space */
+	caddr_t d6[16];			/* dummy to occupy space */
+	int p_maxof;			/* max open files allowed */
+	struct vnode *p_cdir;		/* current directory */
+	struct vnode *p_rdir;		/* root directory */
+	struct ofile_t **p_ofilep;	/* file descriptor chunks */
+	caddr_t d7[84];			/* dummy to occupy space */
+};
+# endif	/* HPUXV>=1020 */
+#endif	/* HPUXV<1010 */
+
+
 /*
  * Local static values
  */
@@ -49,21 +108,17 @@ static char *rcsid = "$Id: dproc.c,v 1.12 95/09/19 10:16:21 abe Exp $";
 static KA_T Kp;				/* kernel's process table address */
 static int Np;				/* number of kernel processes */
 
-#if	_HPUXV>=800
+#if	HPUXV>=800
 static MALLOC_S Nv = 0;			/* number of entries in vnode cache */
 static struct vnode **Vp = NULL;	/* vnode cache */
-#endif	/* _HPUXV>=800 */
+#endif	/* HPUXV>=800 */
 
 
 _PROTOTYPE(static void get_kernel_access,(void));
 
-#if     defined(HASNCACHE)
-_PROTOTYPE(static void ncache_load,(void));
-#endif  /* defined(HASNCACHE) */
-
-#if	_HPUXV>=800
+#if	HPUXV>=800
 _PROTOTYPE(static void process_text,(struct vas *vasp));
-#endif	/* _HPUXV>=800 */
+#endif	/* HPUXV>=800 */
 
 
 /*
@@ -73,15 +128,32 @@ _PROTOTYPE(static void process_text,(struct vas *vasp));
 void
 gather_proc_info()
 {
+	struct file *fp;
 	int err, i, j;
 
-#if	_HPUXV>=800
+#if	HPUXV>=1020
+struct ofile_t {
+	struct ofa {
+		struct file *ofile;
+		caddr_t d1[2];
+	} ofa[SFDCHUNK];
+};
+#endif	/* HPUXV>=1020 */
+
+#if	HPUXV>=800
 	char *c, *s;
 	struct ofile_t *ofp;
 	struct ofile_t oft;
 	struct pst_status ps;
+
+# if	HPUXV<1010
 	struct user us;
-#else	/* _HPUXV<800 */
+# else	/* HPUXV>=1010 */
+	struct user {
+		char u_comm[PST_CLEN];
+	} us;
+# endif	/* HPUXV<1010 */
+#else	/* HPUXV<800 */
 	int k;
 	long sw;
 	char us[U_SIZE];	/* must read HP-UX SWAP in DEV_BSIZE chunks */
@@ -90,10 +162,9 @@ gather_proc_info()
 	struct pte pte1, pte2;
 	KA_T pte_off, pte_addr;
 # endif	/* defined(hp9000s300) */
-#endif	/* _HPUXV>=800 */
+#endif	/* HPUXV>=800 */
 
 	struct proc *p;
-	KA_T pa;
 	struct proc pbuf;
 	short pss, sf;
 	int px;
@@ -107,15 +178,17 @@ gather_proc_info()
  * Examine proc structures and their associated information.
  */
 
-#if	_HPUXV>=800
-	for (p = &pbuf, px = 0, u = &us; px < Np; px++)
-#else	/* _HPUXV<800 */
-	for (p = &pbuf, px = 0, u = (struct user *)us; px < np; px++)
-#endif	/* _HPUXV>=800 */
+#if	HPUXV>=800
+	u = &us;
+	(void) zeromem((char *)u, U_SIZE);
+	for (p = &pbuf, px = 0; px < Np; px++)
+#else	/* HPUXV<800 */
+	for (p = &pbuf, px = 0, u = (struct user *)us; px < Np; px++)
+#endif	/* HPUXV>=800 */
 
 	{
-		pa = Kp + (KA_T)(px * sizeof(struct proc));
-		if (kread(pa, (char *)&pbuf, sizeof(pbuf)))
+		Kpa = Kp + (KA_T)(px * sizeof(struct proc));
+		if (kread(Kpa, (char *)&pbuf, sizeof(pbuf)))
 			continue;
 		if (p->p_stat == 0 || p->p_stat == SZOMB)
 			continue;
@@ -125,19 +198,27 @@ gather_proc_info()
 		if (is_proc_excl(p->p_pid, (int)p->p_pgrp, (UID_ARG)p->p_uid,
 		    &pss, &sf))
 			continue;
+
+#if	HPUXV>=1010
+	/*
+	 * Save the kernel thread pointer.
+	 */
+		Ktp = p->p_firstthreadp;
+#endif	/* HPUXV>=1010 */
+
 	/*
 	 * Read the user area.
 	 */
 
-#if	_HPUXV>=800
+#if	HPUXV>=800
 /*
  * Use the pstat() syscall to read process status.
  */
-	(void) zeromem(u, U_SIZE);
 	if (pstat(PSTAT_PROC, &ps, sizeof(ps), 0, p->p_pid) != 1) {
+	    if (!Fwarn)
 		(void) fprintf(stderr, "%s: can't pstat process %d: %s\n",
 			Pn, p->p_pid, strerror(errno));
-		continue;
+	    continue;
 	}
 /*
  * Scan the pst_cmd command buffer and skip to the last component of the
@@ -160,8 +241,7 @@ gather_proc_info()
 		u->u_comm[i] = *c++;
 	}
 	u->u_comm[i] = '\0';
-#else	/* _HPUXV<800 */
-
+#else	/* HPUXV<800 */
 /*
  * Read the user area from the swap file or memory.
  */
@@ -201,20 +281,20 @@ gather_proc_info()
 # endif	/* defined(hp9000s300) */.
 
 # if	defined(hp9000s800)
-		if (kread((KA_T)uvadd((struct proc *)pa), (char *)u,
+		if (kread((KA_T)uvadd((struct proc *)Kpa), (char *)u,
 		    sizeof(struct user)))
 			continue;
 	}
 # endif	/* defined(hp9000s800) */
-#endif	/* _HPUXV>=800 */
+#endif	/* HPUXV>=800 */
 
 	/*
 	 * Allocate a local process structure.
 	 */
 		if (is_cmd_excl(u->u_comm, &pss, &sf))
 			continue;
-		alloc_lproc(p->p_pid, (int)p->p_pgrp, (UID_ARG)p->p_uid,
-			u->u_comm, (int)pss, (int)sf);
+		alloc_lproc(p->p_pid, (int)p->p_pgrp, (int)p->p_ppid,
+			(UID_ARG)p->p_uid, u->u_comm, (int)pss, (int)sf);
 		Plf = NULL;
 	/*
 	 * Save current working directory information.
@@ -235,79 +315,74 @@ gather_proc_info()
 				link_lfile();
 		}
 
-#if	_HPUXV>=800
+#if	HPUXV>=800
 	/*
 	 * Print information on the text file.
 	 */
 		if (p->p_vas)
 			process_text(p->p_vas);
-#endif	/* _HPUXV>=800 */
+#endif	/* HPUXV>=800 */
 
 	/*
 	 * Loop through user's files.
 	 */
 
-#if	_HPUXV>=800
+#if	HPUXV>=800
 		for (i = 0, j = SFDCHUNK; i < p->p_maxof; i++)
-#else	/* _HPUXV<800 */
+#else	/* HPUXV<800 */
 		for (i = j = k = 0;; i++)
-#endif	/* _HPUXV>=800 */
+#endif	/* HPUXV>=800 */
 
 		{
 
-#if	_HPUXV>=800
-			if (j >= SFDCHUNK) {
-			    if (kread((KA_T)p->p_ofilep, (char *)&ofp,
-				       sizeof(ofp)))
-			    {
-				(void) fprintf(stderr,
-				    "%s: PID %d, can't read ofilep from %#x\n",
-				    Pn, p->p_pid, ofp);
-				exit(1);
-			    }
-			    if (ofp == NULL)
-				break;
-			    if (kread((KA_T)ofp, (char *)&oft, sizeof(oft))) {
-				(void) fprintf(stderr,
-				    "%s: PID %d, can't read ofile_t from %#x\n",
-				    Pn, p->p_pid, ofp);
-				exit(1);
-			    }
-			    j = 0;
-			    p->p_ofilep++;
-			}
-			j++;
-			if (oft.ofile[j - 1])
-#else	/* _HPUXV<800 */
-			if (j >= SFDCHUNK) {
+#if	HPUXV>=800
+		    if (j >= SFDCHUNK) {
+			if (p->p_ofilep == NULL
+			||  kread((KA_T)p->p_ofilep, (char *)&ofp, sizeof(ofp))
+			||  ofp == NULL
+			||  kread((KA_T)ofp, (char *)&oft, sizeof(oft)))
+			    break;
+			j = 0;
+			p->p_ofilep++;
+		    }
+		    j++;
 
-			/*
-			 * Get next file pointer "chunk".
-			 */
-			    while (++k < NFDCHUNKS && u->u_ofilep[k] == NULL)
-				;
-			    if (k >= NFDCHUNKS)
-				break;
-			    if (kread((KA_T)u->u_ofilep[k],
-				      (char *)&u->u_ofile,
-				      sizeof(struct ofile_t)))
-				break;
-			    j = 0;
+# if	HPUXV>=1020
+		    if ((fp = oft.ofa[j - 1].ofile))
+# else	/* HPUXV<1020 */
+		    if ((fp = oft.ofile[j - 1]))
+# endif	/* HPUXV>=1020 */
+#else	/* HPUXV<800 */
+		    if (j >= SFDCHUNK) {
+
+		    /*
+		     * Get next file pointer "chunk".
+		     */
+			while (++k < NFDCHUNKS && u->u_ofilep[k] == NULL)
+			    ;
+			if (k >= NFDCHUNKS)
+			    break;
+			if (kread((KA_T)u->u_ofilep[k], (char *)&u->u_ofile,
+			    sizeof(struct ofile_t)))
+			{
+			    break;
 			}
-			j++;
-			if (u->u_ofile.ofile[j - 1])
-#endif	/* _HPUXV>=800 */
+			j = 0;
+		    }
+		    j++;
+		    if ((fp = u->u_ofile.ofile[j - 1]))
+#endif	/* HPUXV>=800 */
 
 		/*
 		 * Process the file pointer.
 		 */
 
-			{
-				alloc_lfile(NULL, i);
-				process_file(FILEPTR);
-				if (Lf->sf)
-					link_lfile();
-			}
+		    {
+			alloc_lfile(NULL, i);
+			process_file(fp);
+			if (Lf->sf)
+			    link_lfile();
+		    }
 		}
 	/*
 	 * Examine results.
@@ -325,8 +400,13 @@ gather_proc_info()
 static void
 get_kernel_access()
 {
+	unsigned long v;
 
-#if	_HPUXV<800
+#if	defined(HAS_AFS)
+	struct nlist *nl = (struct nlist *)NULL;
+#endif	/* defined(HAS_AFS) */
+
+#if	HPUXV<800
 /*
  * Open access to /dev/mem and SWAP.
  */
@@ -342,7 +422,7 @@ get_kernel_access()
 			err = 1;
 		}
 	}
-#endif	/* _HPUXV<800 */
+#endif	/* HPUXV<800 */
 
 #if	defined(WILLDROPGID)
 /*
@@ -356,16 +436,16 @@ get_kernel_access()
  * See if the non-KMEM memory file is readable.
  */
 	if (Memory && !is_readable(Memory, 1))
-		exit(1);
+		Exit(1);
 #endif	/* defined(WILLDROPGID) */
 
 /*
  * Open kernel memory access.
  */
-	if ((Kmem = open(Memory ? Memory : KMEM, O_RDONLY, 0)) < 0) {
+	if ((Kd = open(Memory ? Memory : KMEM, O_RDONLY, 0)) < 0) {
 		(void) fprintf(stderr, "%s: can't open %s: %s\n", Pn,
 			Memory ? Memory : KMEM, strerror(errno));
-		exit(1);
+		Exit(1);
 	}
 
 #if	defined(WILLDROPGID)
@@ -379,51 +459,87 @@ get_kernel_access()
  * See if the name list file is readable.
  */
 	if (Nmlst && !is_readable(Nmlst, 1))
-		exit(1);
+		Exit(1);
 #endif	/* defined(WILLDROPGID) */
 
+	(void) build_Nl(Drive_Nl);
+
+#if	defined(HAS_AFS)
+	if (!Nmlst) {
+
+	/*
+	 * If AFS is defined and we're getting kernel symbol values from
+	 * from N_UNIX, make a copy of Nl[] for possible use with the AFS
+	 * module name list file.
+	 */
+		if (!(nl = (struct nlist *)malloc(Nll))) {
+			(void) fprintf(stderr,
+				"%s: no space (%d) for Nl[] copy\n", Pn, Nll);
+			Exit(1);
+		}
+		(void) memcpy((void *)nl, (void *)Nl, (size_t)Nll);
+	}
+#endif	/* defined(HAS_AFS) */
+
 /*
  * Access kernel symbols.
  */
 	if (nlist(Nmlst ? Nmlst : N_UNIX, Nl) < 0) {
                 (void) fprintf(stderr, "%s: can't read namelist from %s\n",
 			Pn, Nmlst ? Nmlst : N_UNIX);
-                exit(1);
+                Exit(1);
 	}
-	if (Nl[X_PROC].n_value == NULL
-	||  kread((KA_T)Nl[X_PROC].n_value, (char *)&Kp, sizeof(Kp))
-	||  Nl[X_NPROC].n_value == NULL
-	||  kread((KA_T)Nl[X_NPROC].n_value, (char *)&Np, sizeof(Np))
-	||  Kp == NULL || Np == 0) {
-		(void) fprintf(stderr, "%s: can't read proc table info\n",
-			Pn);
-		exit(1);
+	if (get_Nl_value("proc", Drive_Nl, &v) < 0 || !v
+	||  kread((KA_T)v, (char *)&Kp, sizeof(Kp))
+	||  get_Nl_value("nproc", Drive_Nl, &v) < 0 || !v
+	||  kread((KA_T)v, (char *)&Np, sizeof(Np))
+	||  !Kp || Np < 1) {
+		(void) fprintf(stderr, "%s: can't read proc table info\n", Pn);
+		Exit(1);
 	}
-	Vnfops = (struct fileops *) Nl[X_VN_FOPS].n_value;
+	if (get_Nl_value("vfops", Drive_Nl, (unsigned long *)&Vnfops) < 0)
+		Vnfops = (struct fileops *)NULL;
 
-#if	_HPUXV<800 && defined(hp9000s300)
-	if ((Usrptmap = (struct pte *)Nl[X_USRPTMAP].n_value) == NULL) {
+#if	HPUXV<800 && defined(hp9000s300)
+	if (get_Nl_value("upmap", Drive_Nl, (unsigned long *)&Usrptmap) < 0) {
 		(void) fprintf(stderr, "%s: can't get kernel's Usrptmap\n", Pn);
-		exit(1);
+		Exit(1);
 	}
-	if ((usrpt = (struct pte *)Nl[X_USRPT].n_value) == NULL) {
+	if (get_Nl_value("upt", Drive_Nl, (unsigned long *)&usrpt) < 0) {
 		(void) fprintf(stderr, "%s: can't get kernel's usrpt\n", Pn);
-		exit(1);
+		Exit(1);
 	}
-#endif	/* _HPUXV<800 && defined(hp9000s300) */
+#endif	/* HPUXV<800 && defined(hp9000s300) */
 
-#if	_HPUXV<800 && defined(hp9000s800)
+#if	HPUXV<800 && defined(hp9000s800)
 	proc = (struct proc *)Kp;
-	if ((ubase = (struct user *)Nl[X_UBASE].n_value) == NULL) {
+	if (get_Nl_value("ubase", Drive_Nl, (unsigned long *)&ubase) < 0) {
 		(void) fprintf(stderr, "%s: can't get kernel's ubase\n", Pn);
-		exit(1);
+		Exit(1);
 	}
-	if (Nl[X_NPIDS].n_value == NULL
-	||  kread((KA_T)Nl[X_NPIDS].n_value, (char *)&npids, sizeof(npids))) {
+	if (get_Nl_value("npids", Drive_Nl, &v) < 0 || !v
+	||  kread((KA_T)v, (char *)&npids, sizeof(npids))) {
 		(void) fprintf(stderr, "%s: can't get kernel's npids\n", Pn);
-		exit(1);
+		Exit(1);
 	}
-#endif	/* _HPUXV<800 && defined(hp9000s800) */
+#endif	/* HPUXV<800 && defined(hp9000s800) */
+
+#if	defined(HAS_AFS)
+	if (nl) {
+
+	/*
+	 * If AFS is defined and we're getting kernel symbol values from
+	 * N_UNIX, and if any X_AFS_* symbols isn't there, see if it is in the
+	 * the AFS module name list file.  Make sure that other symbols that
+	 * appear in both name list files have the same values.
+	 */
+		if ((get_Nl_value("arFid", Drive_Nl, &v) >= 0 && !v)
+		||  (get_Nl_value("avops", Drive_Nl, &v) >= 0 && !v)
+		||  (get_Nl_value("avol", Drive_Nl, &v) >= 0 && !v))
+			(void) ckAFSsym(nl);
+		(void) free((FREE_P *)nl);
+	}
+#endif	/* defined(HAS_AFS) */
 
 }
 
@@ -436,7 +552,6 @@ void
 initialize()
 {
 	get_kernel_access();
-	(void) iuidcache(Np);
 }
 
 
@@ -452,14 +567,14 @@ kread(addr, buf, len)
 {
 	int br;
 
-	if (lseek(Kmem, addr, L_SET) == (off_t)-1L)
+	if (lseek(Kd, addr, L_SET) == (off_t)-1L)
 		return(-1);
-	br = read(Kmem, buf, len);
+	br = read(Kd, buf, len);
 	return((br == len) ? 0 : 1);
 }
 
 
-#if	_HPUXV<800
+#if	HPUXV<800
 /*
  * mread() -- read from /dev/mem
  */
@@ -477,10 +592,10 @@ mread(addr, buf, len)
 	br = read(Mem, buf, len);
 	return((br == len) ? 0 : 1);
 }
-#endif	/* _HPUXV<800 */
+#endif	/* HPUXV<800 */
 
 
-#if	_HPUXV>=800
+#if	HPUXV>=800
 /*
  * process_text() - process text access information
  */
@@ -498,7 +613,7 @@ process_text(vasp)
 /*
  * Read virtual address space pointer.
  */
-	if (kread((KA_T)vasp, &v, sizeof(v)))
+	if (kread((KA_T)vasp, (char *)&v, sizeof(v)))
 		return;
 /*
  * Follow the virtual address space pregion structure chain.
@@ -548,7 +663,7 @@ process_text(vasp)
 				(void) fprintf(stderr,
 				    "%s: no space for text vnode pointers\n",
 				    Pn);
-				exit(1);
+				Exit(1);
 			}
 			Nv = 10;
 		} else if (i >= Nv) {
@@ -559,7 +674,7 @@ process_text(vasp)
 			    (void) fprintf(stderr,
 				"%s: no more space for text vnode pointers\n",
 				Pn);
-			    exit(1);
+			    Exit(1);
 			}
 		}
 		Vp[i++] = va;
@@ -586,18 +701,4 @@ process_text(vasp)
 			link_lfile();
 	}
 }
-#endif	/* _HPUXV>=800 */
-
-
-/*
- * The ncache_addr(), ncache_load(), and ncache_lookup() functions are
- * obtained from ../common/rnch.frag.
- */
-
-#if	defined(HASNCACHE)
-# if	_HPUXV<1000
-#define	ADDR_NCACHE	1
-# endif	/* _HPUXV<1000 */
-#endif	/* defined*HASNCACHE) */
-
-
+#endif	/* HPUXV>=800 */
