@@ -30,6 +30,69 @@
  */
 
 
+/*
+ * $Id: machine.h,v 1.8 97/09/23 09:48:12 abe Exp $
+ */
+
+
+#if	!defined(MACHINE_H)
+#define	MACHINE_H	1
+
+
+# if	HPUXV>=1020
+/*
+ * Since we need kernel structures from the HP-UX 10.20 <sys/vnode.h>, and
+ * since defining _KERNEL before #include'ing it causes a conflict between
+ * its FILE enum and the FILE struct definition in <stdio.h>, redefine FILE,
+ * #include <stdio.h>, revoke the FILE redefinition, define _KERNEL,
+ * #include <sys/vnode.h>, revoke the _KERNEL definition, and restore the
+ * FILE redefinition.
+ */
+
+#  if	defined(__GNUC__)
+/*
+ * Do gcc preparation.
+ */
+
+#define	__STDC_EXT__
+#include <sys/_inttypes.h>
+#  endif	/* defined(__GNUC__) */
+
+#define	FILE	STDIO_FILE
+#include <stdio.h>
+#undef	FILE
+#define	_KERNEL	1
+#include <sys/vnode.h>
+#define	FILE	STDIO_FILE
+#undef	_KERNEL
+# endif	HPUXV>=1020
+
+
+/*
+ * GET_MAX_FD is defined for those dialects that provide a function other than
+ * getdtablesize() to obtain the maximum file descriptor number plus one.
+ */
+
+#define	GET_MAX_FD	get_max_fd
+
+
+/*
+ * HASAOPT is defined for those dialects that have AFS support; it specifies
+ * that the default path to an alternate AFS kernel name list file may be
+ * supplied with the -A <path> option.
+ */
+
+#define	HASAOPT		1
+
+
+/*
+ * HASBLKDEV is defined for those dialects that want block device information
+ * recorded in BDevtp[].
+ */
+
+#define	HASBLKDEV	1
+
+
 /*
  * HASDCACHE is defined for those dialects that support a device cache
  * file.
@@ -50,11 +113,14 @@
  *
  * HASSYSDC defines a public device cache file path.  When it's defined, it's
  * used as the path from which to read the device cache.
+ *
+ * Consult the 00DCACHE and 00FAQ files of the lsof distribution for more
+ * information on device cache file path construction.
  */
 
 #define	HASDCACHE	1
 #define	HASENVDC	"LSOFDEVCACHE"
-#define	HASPERSDC	".lsof_%L"
+#define	HASPERSDC	"%h/%p.lsof_%L"
 #define	HASPERSDCPATH	"LSOFPERSDCPATH"
 /* #define	HASSYSDC	"/var/adm/lsof.dc" */
 
@@ -73,6 +139,14 @@
 /* #define	HASFIFONODE	1 */
 
 
+/*
+ * HASFSINO is defined for those dialects that have the file system
+ * inode element, fs_ino, in the lfile structure definition in lsof.h.
+ */
+
+#define	HASFSINO	1
+
+
 /*
  * HASGNODE is defined for those dialects that have gnodes.
  */
@@ -88,7 +162,8 @@
 
 
 /*
- * HASINODE is defined for those dialects that have inodes.
+ * HASINODE is defined for those dialects that have inodes and wish to
+ * use readinode() from node.c.
  */
 
 /* #define	HASINODE	1 */
@@ -116,7 +191,7 @@
  * them.  If any of the additional elements need to be preset in the
  * alloc_lfile() function of proc.c, the SETLFILEADD macro may be defined
  * to do that.  The HASXOPT definition may be used to select the conditions
- * under which provate lfile elements are used.
+ * under which private lfile elements are used.
  *
  * Field identifiers 1 through 9 are reserved for dialect-specific field
  * output.  Define the string HASFIELDAP<x> as the help (-F?) name of the
@@ -162,6 +237,17 @@
 #define	HASNLIST	1
 
 
+/*
+ * HASPIPEFN is defined for those dialects that have a special function to
+ * process DTYPE_PIPE file structure entries.  Its value is the name of the
+ * function.
+ *
+ * NOTE: don't forget to define a prototype for this function in dproto.h.
+ */
+
+/* #define	HASPIPEFN	process_pipe */
+
+
 /*
  * HASPIPENODE is defined for those dialects that have pipe nodes.
  */
@@ -169,6 +255,28 @@
 /* #define	HASPIPENODE	1 */
 
 
+/*
+ * HASPPID is defined for those dialects that support identification of
+ * the parent process IDentifier (PPID) of a process.
+ */
+
+#define	HASPPID		1
+
+
+/*
+ * HASPRIVFILETYPE and PRIVFILETYPE are defined for dialects that have a
+ * file structure type that isn't defined by a DTYPE_* symbol.  They are
+ * used in lib/prfp.c to select the type's processing.
+ *
+ * PRIVFILETYPE is the definition of the f_type value in the file struct.
+ *
+ * HASPRIVFILETYPE is the name of the processing function.
+ */
+
+/* #define	HASPRIVFILETYPE	process_shmf	*/
+/* #define	PRIVFILETYPE	??	*/
+
+
 /*
  * HASPROCFS is defined for those dialects that have a proc file system --
  * usually /proc and usually in SYSV4 derivatives.
@@ -192,16 +300,6 @@
 /* #define		HASPINFO	"/pinfo" */
 
 
-/*
- * HASPWSTAYOPEN is defined for those dialects that have a method to hold
- * the password data base open between searches.  Setting this definition
- * implies the availability of a dialect-specific pw_stay_open() function
- * to implement the method.
- */
-
-/* #define	HASPWSTAYOPEN	1 */
-
-
 /*
  * HASRNODE is defined for those dialects that have rnodes.
  */
@@ -223,9 +321,9 @@
  * HASSNODE is defined for those dialects that have snodes.
  */
 
-#if	_HPUXV>=1000
+# if	HPUXV>=900
 #define	HASSNODE	1
-#endif	/* _HPUXV>=1000 */
+# endif	/* HPUXV>=900 */
 
 
 /*
@@ -235,6 +333,22 @@
 /* #define	HASSTREAMS	1 */
 
 
+/*
+ * HASTCPTPIQ is defined for dialects where it is possible to report the
+ * TCP/TPI Recv-Q and Send-Q values produced by netstat.
+ */
+
+#define	HASTCPTPIQ	1
+
+
+/*
+ * HASTCPTPIW is defined for dialects where it is possible to report the
+ * TCP/TPI send and receive window sizes produced by netstat.
+ */
+
+/* #define	HASTCPTPIW	1 */
+
+
 /*
  * HASTMPNODE is defined for those dialects that have tmpnodes.
  */
@@ -269,6 +383,32 @@
 #define	UID_ARG	uid_t
 
 
+/*
+ * Each USE_LIB_<function_name> is defined for dialects that use the
+ * <function_name> in the lsof library.
+ *
+ * Note: other definitions and operations may be required to condition the
+ * library function source code.  They may be found in the dialect dlsof.h
+ * header files.
+ */
+
+#define	USE_LIB_CK_FILE_ARG			1	/* ckfa.c */
+/* #define	USE_LIB_COMPLETEVFS		1	   cvfs.c */
+#define	USE_LIB_FIND_CH_INO			1	/* fino.c */
+#define	USE_LIB_IS_FILE_NAMED			1	/* isfn.c */
+#define	USE_LIB_LKUPDEV				1	/* lkud.c */
+#define	USE_LIB_PRINTCHDEVNAME			1	/* pdvn.c */
+/* #define	USE_LIB_PROCESS_FILE		1	   prfp.c */
+/* #define	USE_LIB_PRINT_FILE		1	   prtf.c */
+#define	USE_LIB_PRINT_TCPTPI			1	/* ptti.c */
+#define	USE_LIB_READDEV				1	/* rdev.c */
+#define	USE_LIB_READMNT				1	/* rmnt.c */
+/* #define	USE_LIB_READVFS			1	   rvfs.c */
+/* #define	USE_LIB_RNAM			1	   rnam.c */
+#define	USE_LIB_RNCH				1	/* rnch.c */
+/* #define	USE_LIB_RNMH			1	   rnmh.c */
+
+
 /*
  * WARNDEVACCESS is defined for those dialects that should issue a warning
  * when lsof can't access /dev (or /device) or one of its sub-directories.
@@ -278,6 +418,14 @@
 /* #define	WARNDEVACCESS	1 */
 
 
+/*
+ * WARNINGSTATE is defined for those dialects that want to suppress all lsof
+ * warning messages.
+ */
+
+/* #define	WARNINGSTATE	1	warnings are enabled by default */
+
+
 /*
  * WILLDROPGID is defined for those dialects whose lsof executable runs
  * setgid(not_real_GID) and whose setgid power can be relinquished after
@@ -292,3 +440,5 @@
  */
 
 #define	zeromem(a, l)		bzero(a, l)
+
+#endif	/* !defined(MACHINE_H) */
