@@ -30,11 +30,30 @@
  */
 
 
+/*
+ * $Id: dlsof.h,v 1.4 97/08/15 08:24:42 abe Exp $
+ */
+
+
 #if	!defined(PTX_LSOF_H)
 #define	PTX_LSOF_H	1
 
 #include <dirent.h>
+
+# if	PTXV>=420
+#define	_INKERNEL	1
+# endif	/* PTXV>=420 */
+
+# if	PTXV>=430
+#define	_VIEW_OPAQUE_TYPE	1
+# endif	/* PTXV>=430 */
+
 #include <fcntl.h>
+
+# if	PTXV>=420
+#undef	_INKERNEL
+# endif	/* PTXV>=420 */
+
 #include <mntent.h>
 #include <setjmp.h>
 #include <signal.h>
@@ -45,12 +64,12 @@
 #include <sys/time.h>
 #include <sys/vm.h>
 
-# if	_PTXV>=400
+# if	PTXV>=400
 #include <sys/cred.h>
 #include <sys/vm_as.h>
-# else	/* _PTXV<400 */
+# else	/* PTXV<400 */
 #include <sys/swap.h>
-# endif	/* _PTXV>=400 */
+# endif	/* PTXV>=400 */
 
 #include <sys/vnode.h>
 #include <sys/procstats.h>
@@ -58,7 +77,6 @@
 #include <sys/session.h>
 #include <sys/socket.h>
 #undef	AF_OSINET
-#include <sys/socketvar.h>
 #include <sys/snode.h>
 #include <sys/sysmacros.h>
 #include <sys/swap.h>
@@ -72,72 +90,71 @@
 #include <nfs/nfs.h>
 #include <nfs/nfs_clnt.h>
 
-#  if	_PTXV<400
+#  if	PTXV<400
 #include <nfs/rnode.h>
-#  else	/* _PTXV>=400 */
+#  else	/* PTXV>=400 */
 #include "/usr/conf/uts/nfs/rnode.h"
-#  endif	/* _PTXV<400 */
+#  endif	/* PTXV<400 */
 # endif	/* defined(HAS_NFS) */
 
-# if	_PTXV<400
+# if	PTXV<400
 #define _INKERNEL
-# endif	/* _PTXV<400 */
-
-# if	defined(HAS_CDFS)
-
-#  if	_PTXV>=400
-#define _INKERNEL
-#  endif	/* _PTXV>=400 */
-
-#include <sys/cdfsinode.h>
-#undef	IUPD
-#undef	IACC
-#undef	ICHG
-#undef	IFREE
-#undef	IFMT
-#undef	IFREG
-#undef	IFBLK
-#undef	IFCHR
-#undef	IFDIR
-#undef	IREAD
-#undef	IWRITE
-#undef	IEXEC
-#undef	VTOI
-#undef	i_number
-#undef	i_mutex
-#undef	i_devvp
-#undef	i_dev
-
-#  if	_PTXV>=400
-#undef _INKERNEL
-#  endif	/* _PTXV>=400 */
-# endif	/* defined(HAS_CDFS) */
+# endif	/* PTXV<400 */
 
 #include <sys/fifonode.h>
 #include <sys/file.h>
 
-# if	_PTXV>=400
+# if	PTXV>=400
 #define	_INKERNEL	1
+#define	INKERNEL	1
 #include <sys/namenode.h>
 #undef	_INKERNEL
-# endif	/* _PTXV>=400 */
+#undef	INKERNEL
+# endif	/* PTXV>=400 */
 
 #include <sys/stream.h>
+#include <sys/tihdr.h>
 #include <sys/tiuser.h>
+
+# if	PTXV>=421
+#include <sys/unpcb.h>
+# endif	/* PTXV>=421 */
+
+# if	PTXV>=413
+#include <sys/domain.h>
+#include <sys/protosw.h>
+#  if	PTXV>=430
+#define	_INKERNEL	1
+#  else	/* PTXV<430 */
+#define	INKERNEL	1
+#  endif	/* PTXV>=430 */
 #include <sys/sockmod.h>
-#include <sys/ufsinode.h>
-#undef	i_size
+#  if	PTXV>=430
+#undef	_INKERNEL
+#  else	/* PTXV<430 */
+#undef	INKERNEL
+#  endif	/* PTXV>=430 */
+# else	/* PTXV<413 */
+#include <sys/sockmod.h>
+# endif	/* PTXV>=413 */
+
+#include <sys/socketvar.h>
 #include <sys/user.h>
 
-# if	_PTXV<400
+# if	PTXV<400
 #undef _INKERNEL
-# endif	/* _PTXV<400 */
+# endif	/* PTXV<400 */
+
+# if	PTXV>=413
+#include <netinet/ip.h>
+# endif	/* PTXV>=413 */
 
 #include <netinet/in_pcb.h>
 #include <netinet/ip_var.h>
 #include <netinet/tcp.h>
-#include <netinet/tcp_timer.h>
 #include <netinet/tcpip.h>
+#include <netinet/tcp_fsm.h>
+#include <netinet/tcp_timer.h>
 #include <netinet/tcp_var.h>
 
 
@@ -175,13 +192,18 @@ extern struct clone *Clone;
 
 extern int CloneMajor;			/* clone major device number */
 extern int HaveCloneMajor;		/* CloneMajor status */
-extern int Kmem;
+extern int Kd;
 
 struct l_vfs {
 	struct vfs *addr;		/* kernel address */
 	dev_t dev;			/* device */
 	char *dir;			/* mounted directory */
 	char *fsname;			/* file system name */
+
+# if	defined(HASFSINO)
+	ino_t fs_ino;			/* file system inode number */
+# endif	/* defined(HASFSINO) */
+
 	struct l_vfs *next;		/* forward link */
 };
 extern struct l_vfs *Lvfs;
@@ -196,24 +218,9 @@ struct mounts {
 	dev_t rdev;			/* st_rdev */
 };
 
-
-/*
- * Name list (Nl[]) indexes
- */
-
-#define X_ASOPSLARGE	0
-#define X_ASOPSSMALL	1
-#define	X_CDFSOPS	2
-#define	X_CLONEMAJ	3
-#define	X_FIFOOPS	4
-#define	X_NCACHE	5
-#define	X_NCSIZE	6
-#define X_NFSOPS	7
-#define	X_NMOPS		8
-#define	X_NOFILE	9
-#define X_PROC          10
-#define	X_SPECOPS	11
-#define	X_VAR		12
+#define	X_NCACHE	"ncache"
+#define	X_NCSIZE	"ncsize"
+#define	NL_NAME		n_name
 
 extern int Notkmem;
 
@@ -230,6 +237,31 @@ struct sfile {
 	struct sfile *next;		/* forward link */
 };
 
+# if	PTXV<400
 extern int Swap;
+# endif	/* PTXV<400 */
+
+
+/*
+ * Definition for rdev.c
+ */
+
+#define	DIRTYPE	dirent
+
+
+/*
+ * Definition for cvfs.c
+ */
+
+#define	CVFS_DEVSAVE 	1
+
+
+/*
+ * Definitions for rnch.c
+ */
+
+# if     defined(HASNCACHE) && !defined(HASVXFSDNLC)
+#include <sys/dnlc.h>
+# endif  /* defined(HASNCACHE) && !defined(HASVXFSDNLC) */
 
 #endif	/* PTX_LSOF_H */
