@@ -30,6 +30,11 @@
  */
 
 
+/*
+ * $Id: dlsof.h,v 1.2 97/08/15 08:26:16 abe Exp $
+ */
+
+
 #if	!defined(EPIX_LSOF_H)
 #define	RISCOS_LSOF_H	1
 
@@ -49,6 +54,7 @@
 
 #include <bsd/sys/un.h>
 #include <sys/file.h>
+#include <sys/flock.h>
 #undef	KERNEL
 
 #include <sys/immu.h>
@@ -66,6 +72,10 @@
 #include <bsd/net/route.h>
 #include <bsd/net/raw_cb.h>
 #include <bsd/netinet/in_pcb.h>
+#include <bsd/netinet/tcp.h>
+#include <bsd/netinet/tcp_fsm.h>
+#include <bsd/netinet/tcp_timer.h>
+#include <bsd/netinet/tcp_var.h>
 #include <sys/stream.h>
 #include <sys/user.h>
 #include <sys/var.h>
@@ -122,13 +132,20 @@ extern char *sys_errlist[];
  * Global storage definitions (including their structure definitions)
  */
 
-extern int Kmem;
+extern struct file *Fileptr;
+#define	FILEPTR	Fileptr			/* for process_file() in dfile.c */
+extern int Kd;
 
 struct l_vfs {
 	struct vfs *addr;		/* kernel address */
 	dev_t dev;			/* device */
 	char *dir;			/* mounted directory */
 	char *fsname;			/* file system name */
+
+# if	defined(HASFSINO)
+	ino_t fs_ino;			/* file system inode number */
+# endif	/* defined(HASFSINO) */
+
 	struct l_vfs *next;		/* forward link */
 };
 extern struct l_vfs *Lvfs;
@@ -145,12 +162,9 @@ struct mounts {
 	dev_t rdev;			/* st_rdev */
 };
 
-#define	X_NCSIZE	0
-#define	X_NCACHE	1
-#define	X_PROC		2
-#define	X_NPROC		3
-#define	X_NFS_OPS	4
-#define	X_VAR		5
+#define	X_NCACHE	"ncache"
+#define	X_NCSIZE	"ncsize"
+#define	NL_NAME		n_name
 
 struct sfile {
 	char *aname;			/* argument file name */
@@ -187,4 +201,27 @@ struct sfile {
 
 #include <setjmp.h>
 
+
+/*
+ * Definition for rdev.c
+ */
+
+#define	DIRTYPE	dirent
+
+
+/*
+ * Definition for rvfs.c
+ */
+
+#define	RVFS_DEVSAVE	1
+
+
+/*
+ * Definitions for rnch.c
+ */
+
+# if     defined(HASNCACHE)
+#include <sys/dnlc.h>
+# endif  /* defined(HASNCACHE) */
+
 #endif	/* RISCOS_LSOF_H */
