@@ -32,15 +32,14 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dproc.c,v 1.18 95/09/20 20:13:34 abe Exp $";
+static char *rcsid = "$Id: dproc.c,v 1.2 97/04/15 10:48:06 abe Exp $";
 #endif
 
 #include "lsof.h"
-#include "kernelbase.h"
 
-#if	defined(HASNCACHE)
-#include <sys/dnlc.h>
-#endif	/* defined(HASNCACHE) */
+#if	defined(solaris) && solaris<20500
+#include "kernelbase.h"
+#endif	/* defined(solaris) && solaris<20500 */
 
 
 /*
@@ -48,7 +47,7 @@ static char *rcsid = "$Id: dproc.c,v 1.18 95/09/20 20:13:34 abe Exp $";
  */
 
 #if	defined(solaris) && solaris>=20500
-u_long Kb;				/* KERNELBASE for Solaris 2.5 */
+u_long Kb = 0l;				/* KERNELBASE for Solaris 2.5 */
 #endif	/* defined(solaris) && solaris>=20500 */
 
 static int Knp = PROCDFLT;		/* numer of proc structures known
@@ -56,6 +55,7 @@ static int Knp = PROCDFLT;		/* numer of proc structures known
 static int Np;				/* number of P[], Pgrp[], and Pid[]
 					 * entries  */
 static struct proc *P = NULL;		/* local proc structure table */
+static unsigned long Sgvops;		/* [_]segvn_ops address */
 
 #if	defined(solaris)
 static int *Pid = NULL;			/* PIDs for P[] entries */
@@ -67,11 +67,6 @@ static int Switchgid = 0;		/* must switch GIDs for kvm_open() */
 
 _PROTOTYPE(static void close_kvm,(void));
 _PROTOTYPE(static void get_kernel_access,(void));
-
-#if	defined(HASNCACHE)
-_PROTOTYPE(static void ncache_load,(void));
-#endif	/* defined(HASNCACHE) */
-
 _PROTOTYPE(static void open_kvm,(void));
 _PROTOTYPE(static void process_text,(struct as *pa));
 _PROTOTYPE(static void read_proc,(void));
@@ -93,7 +88,7 @@ close_kvm()
 		return;
 	if (kvm_close(Kd) != 0) {
 		(void) fprintf(stderr, "%s: kvm_close failed\n", Pn);
-		exit(1);
+		Exit(1);
 	}
 	Kd = NULL;
 }
@@ -106,6 +101,7 @@ close_kvm()
 void
 gather_proc_info()
 {
+	int ft = 1;
 	int i, j;
 	struct proc *p;
 	int pgrp, pid, px;
@@ -130,6 +126,14 @@ gather_proc_info()
 	static struct file **xuf = NULL;
 #endif	/* solaris */
 
+/*
+ * Do first-time only operations.
+ */
+	if (ft) {
+	    if (get_Nl_value("sgvops", Drive_Nl, &Sgvops) < 0)
+		Sgvops = (unsigned long)0;
+	    ft = 0;
+	}
 /*
  * Read kernel name cache and process table.
  */
@@ -198,7 +202,7 @@ gather_proc_info()
 	 */
 		if (is_cmd_excl(u->u_comm, &pss, &sf))
 			continue;
-		alloc_lproc(pid, pgrp, (UID_ARG)uid, u->u_comm,
+		alloc_lproc(pid, pgrp, (int)p->p_ppid, (UID_ARG)uid, u->u_comm,
 			(int)pss, (int)sf);
 		Plf = NULL;
 
@@ -241,6 +245,11 @@ gather_proc_info()
 	 */
 		if (u->u_cdir) {
 			alloc_lfile(CWD, -1);
+
+#if	defined(FILEPTR)
+			FILEPTR = (struct file *)NULL;
+#endif	/* defined(FILEPTR) */
+
 			process_node((caddr_t)u->u_cdir);
 			if (Lf->sf)
 				link_lfile();
@@ -250,6 +259,11 @@ gather_proc_info()
 	 */
 		if (u->u_rdir) {
 			alloc_lfile(RTD, -1);
+
+#if	defined(FILEPTR)
+			FILEPTR = (struct file *)NULL;
+#endif	/* defined(FILEPTR) */
+
 			process_node((caddr_t)u->u_rdir);
 			if (Lf->sf)
 				link_lfile();
@@ -257,8 +271,14 @@ gather_proc_info()
 	/*
 	 * Save information on text files.
 	 */
-		if (p->p_as && Nl[X_SEGVN].n_value)
+		if (p->p_as && Sgvops) {
+
+#if	defined(FILEPTR)
+			FILEPTR = (struct file *)NULL;
+#endif	/* defined(FILEPTR) */
+
 			process_text(p->p_as);
+		}
 	/*
 	 * Save information on file descriptors.
 	 */
@@ -319,7 +339,7 @@ gather_proc_info()
 					(void) fprintf(stderr,
 						"%s: PID %d, malloc file[]\n",
 						Pn, Lp->pid);
-					exit(1);
+					Exit(1);
 				}
 			} else if (nf > xnf) {
 				xnf = nf;
@@ -329,7 +349,7 @@ gather_proc_info()
 					(void) fprintf(stderr,
 						"%s: PID %d, realloc file[]\n",
 						Pn, Lp->pid);
-					exit(1);
+					Exit(1);
 				}
 			}
 			if (kread((KA_T)u->u_ofile, (char *)xuf, (int)ui))
@@ -372,18 +392,66 @@ gather_proc_info()
 static void
 get_kernel_access()
 {
+	unsigned long v;
+
+#if	defined(HAS_AFS)
+	struct nlist *nl = (struct nlist *)NULL;
+#endif	/* defined(HAS_AFS) */
 
 /*
  * Get kernel symbols.
  */
 	if (Nmlst && !is_readable(Nmlst, 1))
-		exit(1);
+		Exit(1);
+	(void) build_Nl(Drive_Nl);
+
+#if	defined(HAS_AFS)
+	if (!Nmlst) {
+
+	/*
+	 * If AFS is defined and we're getting kernel symbol values from
+	 * from N_UNIX, make a copy of Nl[] for possible use with the AFS
+	 * modload file.
+	 */
+		if (!(nl = (struct nlist *)malloc(Nll))) {
+			(void) fprintf(stderr,
+				"%s: no space (%d) for Nl[] copy\n",
+				Pn, Nll);
+			Exit(1);
+		}
+
+# if	defined(solaris)
+		(void) memcpy((void *)nl, (void *)Nl, (size_t)Nll);
+# else	/* !defined(solaris) */
+		(void) memcpy((char *)nl, (char *)Nl, (int)Nll);
+# endif	/* defined(solaris) */
+
+	}
+#endif	/* defined(HAS_AFS) */
+
 	if (nlist(Nmlst ? Nmlst : N_UNIX, Nl) < 0) {
 		(void) fprintf(stderr, "%s: can't read namelist from %s\n",
 			Pn, Nmlst ? Nmlst : N_UNIX);
-		exit(1);
+		Exit(1);
 	}
 
+#if	defined(HAS_AFS)
+	if (nl) {
+
+	/*
+	 * If AFS is defined and we're getting kernel symbol values from
+	 * N_UNIX, and if any X_AFS_* symbols isn't there, see if it is in the
+	 * the AFS modload file.  Make sure that other symbols that appear in
+	 * both name list files have the same values.
+	 */
+		if ((get_Nl_value("arFID", Drive_Nl, &v) >= 0 && !v)
+		||  (get_Nl_value("avops", Drive_Nl, &v) >= 0 && !v)
+		||  (get_Nl_value("avol",  Drive_Nl, &v) >= 0 && !v))
+			(void) ckAFSsym(nl);
+		(void) free((MALLOC_P *)nl);
+	}
+#endif	/* defined(HAS_AFS) */
+
 #if	defined(WILLDROPGID)
 
 # if	defined(solaris)
@@ -410,7 +478,7 @@ get_kernel_access()
  * See if the non-KMEM memory file is readable.
  */
 	if (Memory && !is_readable(Memory, 1))
-		exit(1);
+		Exit(1);
 #endif	/* defined(WILLDROPGID) */
 
 /*
@@ -420,8 +488,8 @@ get_kernel_access()
 /*
  * Get a proc structure count estimate.
  */
-	if (Nl[X_NPROC].n_value
-	&&  kread((KA_T)Nl[X_NPROC].n_value, (char *)&Knp, sizeof(Knp))
+	if (get_Nl_value("nproc", Drive_Nl, &v) < 0 || !v
+	||  kread((KA_T)v, (char *)&Knp, sizeof(Knp))
 	||  Knp < 1)
 		Knp = PROCDFLT;
 
@@ -430,19 +498,21 @@ get_kernel_access()
 /*
  * Get the kernel's KERNELBASE value for Solaris 2.5 and above.
  */
-	if ( ! Nl[X_KERNELBASE].n_value
-	||  kread((KA_T)Nl[X_KERNELBASE].n_value, (char *)&Kb, sizeof(Kb))) {
-		(void) fprintf(stderr, "%s: can't read %s from %#lx\n", Pn,
-			Nl[X_KERNELBASE].n_name, Nl[X_KERNELBASE].n_value);
-		exit(1);
+	v = (unsigned long)0;
+	if (get_Nl_value("kbase", Drive_Nl, &v) < 0 || !v
+	||  kread((KA_T)v, (char *)&Kb, sizeof(Kb))) {
+		(void) fprintf(stderr,
+			"%s: can't read kernel base address from %#lx\n",
+			Pn, v);
+		Exit(1);
 	}
 # endif	/* solaris >=20500 */
 /*
  * Get the Solaris clone major device number, if possible.
  */
-	if (Nl[X_CLONEMAJ].n_value
-	&&  kread((KA_T)Nl[X_CLONEMAJ].n_value, (char *)&CloneMaj,
-		  sizeof(CloneMaj)) == 0)
+	v = (unsigned long)0;
+	if (get_Nl_value("clmaj", Drive_Nl, &v) >= 0 && v
+	&&  kread((KA_T)v, (char *)&CloneMaj, sizeof(CloneMaj)) == 0)
 	{
 		HaveCloneMaj = 1;
 	}
@@ -470,7 +540,6 @@ void
 initialize()
 {
 	get_kernel_access();
-	iuidcache(UIDCACHEL);
 
 #if	defined(solaris)
 /*
@@ -481,7 +550,7 @@ initialize()
 	readfsinfo();
 
 # if	defined(HASDCACHE)
-	readdev();
+	readdev(0);
 # else	/* !defined(HASDCACHE) */
 	read_clone();
 # endif	/*defined(HASDCACHE) */
@@ -491,12 +560,12 @@ initialize()
  */
 	if ((Cwd = (char *)malloc(MAXPATHLEN)) == NULL) {
 		(void) fprintf(stderr, "%s: no space for cwd buffer\n", Pn);
-		exit(1);
+		Exit(1);
 	}
 	if ((Rtd = (char *)malloc(MAXPATHLEN)) == NULL) {
 		(void) fprintf(stderr,
 			"%s: no space for root directory buffer\n", Pn);
-		exit(1);
+		Exit(1);
 	}
 	*Cwd = *Rtd = '\0';
 #endif	/* solaris */
@@ -526,7 +595,7 @@ kread(addr, buf, len)
 
 #if	defined(solaris) && solaris>=20500
 	if ((u_long)addr < Kb)
-#else	/* 1defined(solaris) || solaris<20500 */
+#else	/* !defined(solaris) || solaris<20500 */
 	if ((u_long)addr < (u_long)KERNELBASE)
 #endif	/* defined(solaris) && solaris>=20500 */
 
@@ -556,7 +625,7 @@ open_kvm()
 			(void) fprintf(stderr,
 				"%s: can't set effective GID to %d: %s\n",
 				Pn, Savedgid, strerror(errno));
-			exit(1);
+			Exit(1);
 		}
 		Setgid = 1;
 	}
@@ -569,7 +638,7 @@ open_kvm()
 			Nmlst ? Nmlst : "default",
 			Memory  ? Memory  : "default",
 			strerror(errno));
-		exit(1);
+		Exit(1);
 	}
 
 #if	defined(WILLDROPGID)
@@ -619,12 +688,12 @@ process_text(pa)
  * Get address space description.
  */
 	if (kread((KA_T)pa, (char *)&as, sizeof(as))) {
-		alloc_lfile(" txt", -1);
-		(void) sprintf(Namech, "can't read text segment list (%#x)",
-			pa);
-		enter_nm(Namech);
+	    alloc_lfile(" txt", -1);
+	    (void) sprintf(Namech, "can't read text segment list (%#x)", pa);
+	    enter_nm(Namech);
+	    if (Lf->sf)
 		link_lfile();
-		return;
+	    return;
 	}
 /*
  * Loop through the segments.  The loop should stop when the segment
@@ -636,8 +705,7 @@ process_text(pa)
 		if (s.S_NEXT == NULL
 		||  kread((KA_T)s.S_NEXT, (char *)&s, sizeof(s)))
 			break;
-		if ((u_long)s.s_ops == Nl[X_SEGVN].n_value
-		&&  s.s_data != NULL) {
+		if ((unsigned long)s.s_ops == Sgvops && s.s_data) {
 			if (kread((KA_T)s.s_data, (char *)&sd, sizeof(sd)))
 				break;
 			if (sd.vn.vp) {
@@ -654,6 +722,11 @@ process_text(pa)
 				}
 				if (l >= k) {
 					alloc_lfile(" txt", -1);
+
+#if	defined(FILEPTR)
+					FILEPTR = (struct file *)NULL;
+#endif	/* defined(FILEPTR) */
+
 					process_node((caddr_t)sd.vn.vp);
 					if (Lf->sf) {
 						link_lfile();
@@ -694,21 +767,21 @@ readfsinfo()
 	if ((Fsinfomax = sysfs(GETNFSTYP)) == -1) {
 		(void) fprintf(stderr, "%s: sysfs(GETNFSTYP) error: %s\n",
 			Pn, strerror(errno));
-		exit(1);
+		Exit(1);
 	} 
 	if (Fsinfomax == 0)
 		return;
 	if ((Fsinfo = (char **)malloc((MALLOC_S)(Fsinfomax * sizeof(char *))))
 	== NULL) {
 		(void) fprintf(stderr, "%s: no space for sysfs info\n", Pn);
-		exit(1);
+		Exit(1);
 	}
 	for (i = 1; i <= Fsinfomax; i++) {
 		if (sysfs(GETFSTYP, i, buf) == -1) {
 			(void) fprintf(stderr,
 				"%s: sysfs(GETFSTYP) error: %s\n",
 				Pn, strerror(errno));
-			exit(1);
+			Exit(1);
 		}
 		if (buf[0] == '\0') {
 			Fsinfo[i-1] = "";
@@ -720,9 +793,15 @@ readfsinfo()
 			(void) fprintf(stderr,
 				"%s: no space for file system entry %s\n",
 				Pn, buf);
-			exit(1);
+			Exit(1);
 		}
 		(void) strcpy(Fsinfo[i-1], buf);
+
+# if	defined(HAS_AFS)
+		if (strcasecmp(buf, "afs") == 0)
+			AFSfstype = i;
+# endif	/* defined(HAS_AFS) */
+
 	}
 }
 #endif	/* solaris */
@@ -759,7 +838,7 @@ read_proc()
 		len = (sz * sizeof(struct proc));
 		if ((P = (struct proc *)malloc(len)) == (struct proc *)NULL) {
 		    (void) fprintf(stderr, "%s: no proc table space\n", Pn);
-		    exit(1);
+		    Exit(1);
 		}
 
 #if	defined(solaris)
@@ -770,12 +849,12 @@ read_proc()
 		if (Fpgrp) {
 		    if ((Pgrp = (int *)malloc(len)) == (int *)NULL) {
 			(void) fprintf(stderr, "%s: no PGRP table space\n", Pn);
-			exit(1);
+			Exit(1);
 		    }
 		}
 		if ((Pid = (int *)malloc(len)) == (int *)NULL) {
 		    (void) fprintf(stderr, "%s: no PID table space\n", Pn);
-		    exit(1);
+		    Exit(1);
 		}
 #endif	/* defined(solaris) */
 
@@ -788,7 +867,7 @@ read_proc()
 	    if (kvm_setproc(Kd) != 0) {
 		(void) fprintf(stderr, "%s: kvm_setproc: %s\n", Pn,
 		    strerror(errno));
-		exit(1);
+		Exit(1);
 	    }
 	/*
 	 * Accumulate proc structures.
@@ -823,7 +902,7 @@ read_proc()
 		    == NULL) {
 			(void) fprintf(stderr,
 			    "%s: no more (%d) proc space\n", Pn, sz);
-			exit(1);
+			Exit(1);
 		    }
 
 #if	defined(solaris)
@@ -836,14 +915,14 @@ read_proc()
 			== NULL) {
 			    (void) fprintf(stderr,
 				"%s: no more (%d) PGRP space\n", Pn, sz);
-			    exit(1);
+			    Exit(1);
 			}
 		    }
 		    if ((Pid = (int *)realloc((MALLOC_P *)Pid, len))
 		    == NULL) {
 			(void) fprintf(stderr,
 			    "%s: no more (%d) PID space\n", Pn, sz);
-			exit(1);
+			Exit(1);
 		    }
 #endif	/* defined(solaris) */
 
@@ -880,7 +959,7 @@ read_proc()
  */
 	if (try >= PROCTRYLM) {
 	    (void) fprintf(stderr, "%s: can't read proc table\n", Pn);
-	    exit(1);
+	    Exit(1);
 	}
 	if (Np < sz && !RptTm) {
 
@@ -892,7 +971,7 @@ read_proc()
 	    if ((P = (struct proc *)realloc((MALLOC_P *)P, len)) == NULL) {
 		(void) fprintf(stderr, "%s: can't reduce proc table to %d\n",
 		    Pn, Np);
-		exit(1);
+		Exit(1);
 	    }
 
 #if	defined(solaris)
@@ -905,23 +984,15 @@ read_proc()
 		if ((Pgrp = (int *)realloc((MALLOC_P *)Pgrp, len)) == NULL) {
 		    (void) fprintf(stderr,
 			"%s: can't reduce PGRP table to %d\n", Pn, Np);
-		    exit(1);
+		    Exit(1);
 		}
 	    }
 	    if ((Pid = (int *)realloc((MALLOC_P *)Pid, len)) == NULL) {
 		(void) fprintf(stderr,
 		    "%s: can't reduce PID table to %d\n", Pn, Np);
-		exit(1);
+		Exit(1);
 	    }
 #endif	/* defined(solaris) */
 
 	}
 }
-
-
-/*
- * The ncache_addr(), ncache_load(), and ncache_lookup() functions are
- * obtained from ../common/rnch.frag.
- */
-
-
