@@ -30,6 +30,55 @@
  */
 
 
+#if	!defined(MACHINE_H)
+#define	MACHINE_H	1
+
+
+# if	defined(solaris) && solaris>=20600
+/*
+ * <sys/poll.h> must be #include'd for Solaris 2.6 while _KMEMUSER is
+ * defined.  Since <netdb.h> also #include's <sys/poll.h> and <netdb.h>
+ * is #include'd from lsof.h, we must perform some early #include magic
+ * here to set things up properly.
+ */
+
+#define	_KMEMUSER	1
+#include	<sys/poll.h>
+# endif	/* defined(solaris) && solaris>=20600 */
+
+
+/*
+ * $Id: machine.h,v 1.7 97/09/23 09:52:02 abe Exp $
+ */
+
+
+/*
+ * GET_MAX_FD is defined for those dialects that provide a function other than
+ * getdtablesize() to obtain the maximum file descriptor number plus one.
+ */
+
+#if	defined(solaris) && solaris<20500
+#define	GET_MAX_FD	get_max_fd
+#endif	/* defined(solaris) && solaris<20500 */
+
+
+/*
+ * HASAOPT is defined for those dialects that have AFS support; it specifies
+ * that the default path to an alternate AFS kernel name list file may be
+ * supplied with the -A <path> option.
+ */
+
+#define	HASAOPT		1
+
+
+/*
+ * HASBLKDEV is defined for those dialects that want block device information
+ * recorded in BDevtp[].
+ */
+
+#define	HASBLKDEV	1
+
+
 /*
  * HASDCACHE is defined for those dialects that support a device cache
  * file.
@@ -50,11 +99,14 @@
  *
  * HASSYSDC defines a public device cache file path.  When it's defined, it's
  * used as the path from which to read the device cache.
+ *
+ * Consult the 00DCACHE and 00FAQ files of the lsof distribution for more
+ * information on device cache file path construction.
  */
 
 #define	HASDCACHE	1
 #define	HASENVDC	"LSOFDEVCACHE"
-#define	HASPERSDC	".lsof_%L"
+#define	HASPERSDC	"%h/%p.lsof_%L"
 #define	HASPERSDCPATH	"LSOFPERSDCPATH"
 /* #define	HASSYSDC	"/var/adm/lsof.dc" */
 
@@ -73,6 +125,14 @@
 #define	HASFIFONODE	1
 
 
+/*
+ * HASFSINO is defined for those dialects that have the file system
+ * inode element, fs_ino, in the lfile structure definition in lsof.h.
+ */
+
+#define	HASFSINO	1
+
+
 /*
  * HASGNODE is defined for those dialects that have gnodes.
  */
@@ -88,10 +148,13 @@
 
 
 /*
- * HASINODE is defined for those dialects that have inodes.
+ * HASINODE is defined for those dialects that have inodes and wish to
+ * use readinode() from node.c.
  */
 
+#if	!defined(solaris)
 #define	HASINODE	1
+#endif	/* !defined(solaris) */
 
 
 /*
@@ -116,7 +179,7 @@
  * them.  If any of the additional elements need to be preset in the
  * alloc_lfile() function of proc.c, the SETLFILEADD macro may be defined
  * to do that.  The HASXOPT definition may be used to select the conditions
- * under which provate lfile elements are used.
+ * under which private lfile elements are used.
  *
  * Field identifiers 1 through 9 are reserved for dialect-specific field
  * output.  Define the string HASFIELDAP<x> as the help (-F?) name of the
@@ -162,6 +225,17 @@
 #define	HASNLIST	1
 
 
+/*
+ * HASPIPEFN is defined for those dialects that have a special function to
+ * process DTYPE_PIPE file structure entries.  Its value is the name of the
+ * function.
+ *
+ * NOTE: don't forget to define a prototype for this function in dproto.h.
+ */
+
+/* #define	HASPIPEFN	process_pipe */
+
+
 /*
  * HASPIPENODE is defined for those dialects that have pipe nodes.
  */
@@ -169,6 +243,28 @@
 /* #define	HASPIPENODE	1 */
 
 
+/*
+ * HASPPID is defined for those dialects that support identification of
+ * the parent process IDentifier (PPID) of a process.
+ */
+
+#define	HASPPID		1
+
+
+/*
+ * HASPRIVFILETYPE and PRIVFILETYPE are defined for dialects that have a
+ * file structure type that isn't defined by a DTYPE_* symbol.  They are
+ * used in lib/prfp.c to select the type's processing.
+ *
+ * PRIVFILETYPE is the definition of the f_type value in the file struct.
+ *
+ * HASPRIVFILETYPE is the name of the processing function.
+ */
+
+/* #define	HASPRIVFILETYPE	process_shmf	*/
+/* #define	PRIVFILETYPE	??	*/
+
+
 /*
  * HASPROCFS is defined for those dialects that have a proc file system --
  * usually /proc and usually in SYSV4 derivatives.
@@ -187,21 +283,11 @@
  * information (typically /pinfo).
  */
 
-#if	defined(solaris)
+# if	defined(solaris)
 #define		HASPROCFS	"proc"
 #define		HASFSTYPE	1
 /* #define		HASPINFO	"/pinfo" */
-#endif
-
-
-/*
- * HASPWSTAYOPEN is defined for those dialects that have a method to hold
- * the password data base open between searches.  Setting this definition
- * implies the availability of a dialect-specific pw_stay_open() function
- * to implement the method.
- */
-
-/* #define	HASPWSTAYOPEN	1 */
+# endif	/* defined(solaris) */
 
 
 /*
@@ -235,6 +321,26 @@
 #define	HASSTREAMS	1
 
 
+/*
+ * HASTCPTPIQ is defined for dialects where it is possible to report the
+ * TCP/TPI Recv-Q and Send-Q values produced by netstat.
+ */
+
+#if	!defined(solaris) || solaris==20300 || solaris>=20500
+#define	HASTCPTPIQ	1
+#endif	/* !defined(solaris) || solaris==20300 || solaris>=20500 */
+
+
+/*
+ * HASTCPTPIW is defined for dialects where it is possible to report the
+ * TCP/TPI send and receive window sizes produced by netstat.
+ */
+
+#if	defined(solaris) && (solaris==20300 || solaris>=20500)
+#define	HASTCPTPIW	1
+#endif	/* defined(solaris) && (solaris==20300 || solaris>=20500) */
+
+
 /*
  * HASTMPNODE is defined for those dialects that have tmpnodes.
  */
@@ -269,6 +375,36 @@
 #define	UID_ARG	long
 
 
+/*
+ * Each USE_LIB_<function_name> is defined for dialects that use the
+ * <function_name> in the lsof library.
+ *
+ * Note: other definitions and operations may be required to condition the
+ * library function source code.  They may be found in the dialect dlsof.h
+ * header files.
+ */
+
+#define	USE_LIB_CK_FILE_ARG			1	/* ckfa.c */
+#define	USE_LIB_COMPLETEVFS			1	/* cvfs.c */
+#define	USE_LIB_FIND_CH_INO			1	/* fino.c */
+/* #define	USE_LIB_IS_FILE_NAMED		1	   isfn.c */
+#define	USE_LIB_LKUPDEV				1	/* lkud.c */
+/* #define	USE_LIB_PRINTCHDEVNAME		1	   pdvn.c */
+/* #define	USE_LIB_PROCESS_FILE		1	   prfp.c */
+/* #define	USE_LIB_PRINT_FILE		1	   prtf.c */
+
+#if	!defined(solaris)
+#define	USE_LIB_PRINT_TCPTPI			1	/* ptti.c */
+#endif	/* !defined(solaris) */
+
+/* #define	USE_LIB_READDEV			1	   rdev.c */
+/* #define	USE_LIB_READMNT			1	   rmnt.c */
+/* #define	USE_LIB_READVFS			1	   rvfs.c */
+/* #define	USE_LIB_RNAM			1	   rnam.c */
+#define	USE_LIB_RNCH				1	/* rnch.c */
+/* #define	USE_LIB_RNMH			1	   rnmh.c */
+
+
 /*
  * WARNDEVACCESS is defined for those dialects that should issue a warning
  * when lsof can't access /dev (or /device) or one of its sub-directories.
@@ -278,6 +414,14 @@
 #define	WARNDEVACCESS	1
 
 
+/*
+ * WARNINGSTATE is defined for those dialects that want to suppress all lsof
+ * warning messages.
+ */
+
+/* #define	WARNINGSTATE	1	warnings are enabled by default */
+
+
 /*
  * WILLDROPGID is defined for those dialects whose lsof executable runs
  * setgid(not_real_GID) and whose setgid power can be relinquished after
@@ -292,3 +436,5 @@
  */
 
 #define	zeromem(a, l)		memset(a, 0, l)
+
+#endif	/* !defined(MACHINE_H) */
