@@ -30,6 +30,11 @@
  */
 
 
+/*
+ * $Id: dlsof.h,v 1.3 97/08/15 08:28:21 abe Exp $
+ */
+
+
 #if	!defined(ULTRIX_LSOF_H)
 #define	ULTRIX_LSOF_H	1
 
@@ -62,6 +67,10 @@ extern char *getenv();
 #include <net/route.h>
 #include <net/raw_cb.h>
 #include <netinet/in_pcb.h>
+#include <netinet/tcp.h>
+#include <netinet/tcp_fsm.h>
+#include <netinet/tcp_timer.h>
+#include <netinet/tcp_var.h>
 #include <nfs/nfs.h>
 #include <nfs/nfs_clnt.h>
 #include <nfs/vfs.h>
@@ -71,6 +80,7 @@ extern char *getenv();
 #include <sys/gnode.h>
 #undef KERNEL
 #include <sys/user.h>
+#include <sys/flock.h>
 
 # if	ULTRIXV>=40200
 /*
@@ -100,22 +110,6 @@ struct fifonode {
 };
 # endif	/* ULTRIXV>=40200 */
 
-# if     defined(RHF)
-/*
- * Local header files for the Purdue University Computing Center's
- * implementation of CDC Remote Host Facility (RHF) sockets.
- */
-#include <net/if.h>
-#include <netlcn/rhf.h>
-#include <netlcn/rhf_timer.h>
-#include <netlcn/rhfcb.h>
-# endif	/* RHF */
-
-# if	defined(HASPWSTAYOPEN)
-extern int _pw_stayopen;
-#define	pw_stay_open()	_pw_stayopen=1
-# endif
-
 extern char *sys_errlist[];
 #define strerror(errno) sys_errlist[errno]
 
@@ -150,7 +144,9 @@ extern char *sys_errlist[];
  * Global storage definitions (including their structure definitions)
  */
 
-extern int Kmem;
+extern struct file *Fileptr;
+#define	FILEPTR	Fileptr			/* for process_file() in lib/prfp.c */
+extern int Kd;
 
 extern int Mem;
 
@@ -164,18 +160,9 @@ struct mounts {
 	dev_t rdev;			/* st_rdev */
 };
 
-#define X_PROC          0
-#define X_NPROC         1
-#define	X_NCACHE	2
-#define	X_NCSIZE	3
-#define X_NFS_OPS	4
-
-# if	ULTRIXV>=40200
-#define	X_CDFS_OPS	5
-#define X_FIFO_OPS	6
-#define X_SPEC_OPS	7
-#define X_UFS_OPS	8
-# endif
+#define	X_NCACHE	"ncache"
+#define	X_NCSIZE	"ncsize"
+#define NL_NAME		n_name
 
 struct sfile {
 	char *aname;			/* argument file name */
@@ -214,4 +201,26 @@ extern int Swap;
 
 #include <setjmp.h>
 
+
+/*
+ * Definitions for rdev.c
+ */
+
+#define	DIRTYPE	direct
+#define HASDNAMLEN	1	/* struct DIRTYPE has d_namlen element */
+
+
+/*
+ * Definitions for rnam.c
+ */
+
+# if     defined(HASNCACHE)
+#define	NCACHE		nch		/* kernel's structure name */
+#define	NCACHE_NM	nc_name		/* name in NCACHE */
+#define	NCACHE_NMLEN	nc_nlen		/* name length in NCACHE */
+#define	NCACHE_NODEADDR	nc_ip		/* node address in NCACHE */
+#define	NCACHE_NODEID	nc_id		/* node ID in NCACHE */
+#define	NCACHE_NO_ROOT	1		/* can't test for root node */
+# endif  /* defined(HASNCACHE) */
+
 #endif	/* ULTRIX_LSOF_H */
