@@ -30,6 +30,11 @@
  */
 
 
+/*
+ * $Id: lsof.h,v 1.7 97/10/10 08:26:54 abe Exp $
+ */
+
+
 #if	!defined(LSOF_H)
 #define	LSOF_H	1
 
@@ -44,6 +49,19 @@
 
 #include <sys/stat.h>
 
+
+/*
+ * Definitions of structures that may be needed by dlsof.h
+ */
+
+struct l_dev {
+	dev_t rdev;			/* device */
+	ino_t inode;			/* inode number */
+	char *name;			/* name */
+	int v;				/* has been verified
+					 * (when DCUnsafe == 1) */
+};
+
 #include "dlsof.h"
 
 extern int errno;
@@ -51,78 +69,131 @@ extern char *optarg;
 extern int optind;
 
 #define	ACCESSERRFMT	"%s: WARNING: access %s: %s\n"
-#define	CMDL		10		/* command character length */
+#define	CMDL		9		/* command character length */
 
 # if	defined(HASDCACHE)
-#define CRC_POLY	0120001 	/* CRC-16 polynomial */
+#define	CRC_POLY	0120001 	/* CRC-16 polynomial */
 #define	CRC_TBLL	256		/* crc table length for software */
 #define	CRC_BITS	8		/* number of bits contributing */
 # endif	/* defined(HASDCACHE) */
 
 #define	CWD		" cwd"		/* current working directory fd name */
 #define	FDLEN		8		/* fd printing array length */
-#define IDINCR		10		/* PID/PGRP table malloc() increment */
-#define IPROTOL		6		/* Internet protocol length */
-#define LOGINML		9		/* login name length */
-#define LPROCINCR	128		/* Lproc[] allocation increment */
+#define	IDINCR		10		/* PID/PGRP table malloc() increment */
+#define	IPROTOL		6		/* Internet protocol length */
+#define	LOGINML		9		/* login name length */
+#define	LPROCINCR	128		/* Lproc[] allocation increment */
+#define	LSOF_URL	"ftp://vic.cc.purdue.edu/pub/tools/unix/lsof"
 #define	MAXDCPATH	4		/* paths in DCpath[] */
-#define N_REGLR		0		/* node on regular file system */
-#define N_BLK		1		/* node on block device */
-#define	N_CACHE		2		/* node on cached file system */
-#define	N_CDFS		3		/* node on CD-ROM */
-#define	N_CHR		4		/* node on character device */
-#define	N_COM		5		/* node on streams common device */
-#define	N_DOOR		6		/* DOOR node */
-#define N_FIFO		7		/* node on FIFO */
-#define N_HSFS		8		/* node on High Sierra file system */
-#define N_LOFS		9		/* node on loopback file system */
-#define N_MPC		10		/* node on multiplexed device */
-#define N_NFS		11		/* node on NFS file system */
-#define N_NM		12		/* node on named file system */
-#define N_PCFS		13		/* node on PC file system */
-#define N_PIPE		14		/* node on pipe device */
-#define	N_PROC		15		/* node on /proc */
-#define	N_SPEC		16		/* node served by spec_vnodeops */
-#define N_STREAM	17		/* node is stream */
-#define N_TMP		18		/* node on tmpfs */
-#define	N_UFS		19		/* node is Unix file system type */
-#define	N_VXFS		20		/* node is Veritas file system type */
+#define	MAXNWAD		100		/* maximum network addresses */
+#define	N_REGLR		0		/* regular file system node */
+#define	N_AFS		1		/* AFS node */
+#define	N_AUTO		2		/* automount node */
+#define	N_BLK		3		/* block device node */
+#define	N_CACHE		4		/* cached file system node */
+#define	N_CDFS		5		/* CD-ROM node */
+#define	N_CHR		6		/* character device node */
+#define	N_COM		7		/* streams common device node */
+#define	N_DOOR		8		/* DOOR node */
+#define	N_FIFO		9		/* FIFO node */
+#define	N_HSFS		10		/* High Sierra node */
+#define	N_KERN		11		/* BSD /kern node */
+#define	N_LOFS		12		/* loopback node */
+#define	N_MPC		13		/* multiplexed device node */
+#define	N_MVFS		14		/* multi-volume file system node (?) */
+#define	N_NFS		15		/* NFS node */
+#define	N_NM		16		/* named file system node */
+#define	N_OBJF		17		/* objfs file system node */
+#define	N_PCFS		18		/* PC file system node */
+#define	N_PIPE		19		/* pipe device node */
+#define	N_PROC		20		/* /proc node */
+#define	N_SOCK		21		/* sock_vnodeops node */
+#define	N_SPEC		22		/* spec_vnodeops node */
+#define	N_STREAM	23		/* stream node */
+#define	N_TMP		24		/* tmpfs node */
+#define	N_UFS		25		/* Unix file system node */
+#define	N_VXFS		26		/* Veritas file system node */
+#define	N_XFS		27		/* XFS node */
 #define	RPTTM		15		/* default repeat seconds */
 #define	RTD		" rtd"		/* root directory fd name */
-#define TMLIMIT		15		/* readlink() & stat() timeout sec */
-#define TMLIMMIN	2		/* minimum timeout */
+#define	TCPTPI_QUEUES	1		/* report TCP/TPI queue lengths */
+#define	TCPTPI_STATE	2		/* report TCP/TPI state */
+#define TCPTPI_WINDOWS	4		/* report TCP/TPI window sizes */
+#define	TCPTPI_ALL	(TCPTPI_QUEUES | TCPTPI_STATE | TCPTPI_WINDOWS)
+					/* report all TCP/TPI info */
+#define	TMLIMIT		15		/* readlink() & stat() timeout sec */
+#define	TMLIMMIN	2		/* minimum timeout */
 #define	TYPEL		8		/* type character length */
-#define UIDINCR		10		/* UID table malloc() increment */
-#define USERPRTL	8		/* login/UID print length */
+#define	UIDCACHEL	1024		/* UID cache length */
+#define	UIDINCR		10		/* UID table malloc() increment */
+#define	USERPRTL	8		/* login/UID print length */
 
 
 /*
  * Selection flags
  */
 
-#define PS_PRI		1		/* primary process selection -- e.g.,
+#define	PS_PRI		1		/* primary process selection -- e.g.,
 					 * by PID or UID */
-#define PS_SEC		2		/* secondary process selection -- e.g.,
+#define	PS_SEC		2		/* secondary process selection -- e.g.,
 					 * by directory or file */
 #define	SELCMD		0x001		/* select process by command name */
 #define	SELFD		0x002		/* select file by descriptor name */
-#define	SELNA		0x004		/* select socket by address (-i) */
-#define	SELNET		0x008		/* select Internet socket files (-n) */
+#define	SELNA		0x004		/* select socket by address (-i@...) */
+#define	SELNET		0x008		/* select Internet socket files (-i) */
 #define	SELNFS		0x010		/* select NFS files (-N) */
 #define	SELNM		0x020		/* select by name */
 #define	SELPGRP		0x040		/* select process group IDs (-g) */
 #define	SELPID		0x080		/* select PIDs (-p) */
 #define	SELUID		0x100		/* select UIDs (-u) */
 #define	SELUNX		0x200		/* select Unix socket (-U) */
-#define SELALL		(SELCMD|SELFD|SELNA|SELNET|SELNM|SELNFS|SELPID|SELUID|SELUNX)
+#define	SELALL		(SELCMD|SELFD|SELNA|SELNET|SELNM|SELNFS|SELPID|SELUID|SELUNX)
+
+/*
+ * Structure definitions
+ */
+
+#if	defined(HAS_AFS)
+struct afsnode {			/* AFS pseudo-node structure */
+	dev_t dev;
+	unsigned char ino_st;		/* 1 if inode has a value */
+	unsigned long inode;
+	unsigned long size;
+};
+#endif	/* defined(HAS_AFS) */
+
+#if	defined(HASNLIST)
+struct drive_Nl {			/* data to drive build_Nl() */
+	char *nn;			/* nickname for lookups */
+	char *knm;			/* kernel variable for name list */
+};
+extern struct drive_Nl Drive_Nl[];	/* defined in dstore.c */
+#endif	/* defined(HASNLIST) */
 
 
 /*
  * Global storage definitions (including their structure definitions)
  */
 
+struct int_lst {
+	int i;				/* integer argument */
+	int f;				/* find state */
+};
+
+struct seluid {
+	uid_t uid;			/* User ID */
+	char *lnm;			/* specified login name (NULL = none) */
+	unsigned char excl;		/* excluded state */
+	unsigned char f;		/* selected User ID find state
+					 * (meaningful only if excl == 0) */
+};
+
+extern int CkPasswd;
+
 struct str_lst {
 	char *str;			/* string */
+	int len;			/* string length */
+	int f;				/* selected string find state */
 	struct str_lst *next;		/* next list entry */
 };
 extern struct str_lst *Cmdl;
@@ -134,17 +205,19 @@ extern FILE *DCfs;
 extern char *DCpathArg;
 extern char *DCpath[];
 extern int DCpathX;
+extern int DCrebuilt;
 extern int DCstate;
+extern int DCunsafe;
 # endif	/* defined(HASDCACHE) */
 
-struct l_dev {
-	dev_t rdev;			/* device */
-	ino_t inode;			/* inode number */
-	char *name;			/* name */
-
-};
+extern int DChelp;
 extern struct l_dev *Devtp;
 
+# if	defined(HASBLKDEV)
+extern struct l_dev *BDevtp, **BSdev;
+extern int BNdev;
+# endif	/* defined(HASBLKDEV) */
+
 extern int Fand;
 extern int Fblock;
 extern int Ffield;
@@ -161,10 +234,17 @@ extern int Foffset;
 extern int Fovhd;
 extern int Fport;
 extern int Fpgrp;
+
+# if	defined(HASPPID)
+extern int Fppid;
+# endif	/* defined(HASPPID) */
+
 extern int Fsize;
+extern int Ftcptpi;
 extern int Fterse;
 extern int Funix;
 extern int Futol;
+extern int Fverbose;
 extern int Fwarn;
 
 # if	defined(HASXOPT_VALUE)
@@ -175,7 +255,7 @@ extern struct str_lst *Fdl;
 
 struct fieldsel {
 	char id;			/* field ID character */
-	unsigned char n;		/* field ID index */
+	unsigned char st;		/* field status */
 	char *nm;			/* field name */
 };
 extern struct fieldsel FieldSel[];
@@ -192,10 +272,14 @@ struct lfile {
 					 *	0: neither inode nor iproto
 					 *	1: inode contains number
 					 *	2: iproto contains string */
-	unsigned char is_chr_dev;	/* character device status */
 	unsigned char is_com;		/* common stream status */
 	unsigned char is_nfs;		/* NFS file status */
 	unsigned char is_stream;	/* stream device status */
+
+# if	defined(HASVXFS) && defined(HASVXFSDNLC)
+	unsigned char is_vxfs;		/* VxFS file status */
+# endif	/* defined(HASVXFS) && defined(HASVXFSDNLC) */
+
 	unsigned char off_def;		/* offset definition status */
 	unsigned char sz_def;		/* size definition status */
 	char fd[FDLEN];
@@ -203,13 +287,49 @@ struct lfile {
 	char type[TYPEL];
 	short sf;			/* select flags -- SEL* symbols */
 	int ch;				/* VMPC channel: -1 = none */
-	int off;
-	int sz;
+	int ntype;			/* node type -- N_* value */
+	unsigned long off;
+	unsigned long sz;
 	dev_t dev;
 	unsigned long inode;
 	char *dev_ch;
 	char *fsdir;			/* file system directory */
 	char *fsdev;			/* file system device */
+
+# if	defined(HASFSINO)
+	unsigned long fs_ino;		/* file system inode number */
+# endif	/* defined HASFSINO) */
+
+	struct linaddr {		/* local Internet address information */
+		int as;			/* address status: 0 == none */
+		int p;			/* port */
+		struct in_addr ia;	/* Internet address */
+	} li[2];			/* li[0]: local
+					 * li[1]: foreign */
+	struct ltstate {		/* local TCP/TPI state */
+		int type;		/* state type: -1 == none
+					 *		0 == TCP
+					 *		1 == TPI */
+		union {
+			int i;		/* integer state */
+			unsigned int u;	/* unsigned integer state */
+		} state;
+
+#if	defined(HASTCPTPIQ)
+		unsigned long rq;	/* receive queue length */
+		unsigned long sq;	/* send queue length */
+		unsigned char rqs;	/* rq status: 0 = none */
+		unsigned char sqs;	/* sq status: 0 = none */
+#endif	/* defined(HASTCPTPIQ) */
+
+#if	defined(HASTCPTPIW)
+		unsigned char rws;	/* rw status: 0 = none */
+		unsigned char wws;	/* ww status: 0 = none */
+		unsigned long rw;	/* read window size */
+		unsigned long ww;	/* write window size */
+#endif	/* defined(HASTCPTPIW) */
+
+	} lts;
 	char *nm;
 	char *nma;			/* NAME column addition */
 
@@ -230,15 +350,16 @@ extern struct lfile *Lf, *Plf;
 
 
 struct lproc {
-	char cmd[CMDL];
+	char *cmd;			/* command name */
 	short sf;			/* select flags -- SEL* symbols */
 	short pss;			/* state: 0 = not selected
 				 	 *	  1 = wholly selected
 				 	 *	  2 = partially selected */
-	int pid;
+	int pid;			/* process ID */
 	int pgrp;			/* process group ID */
-	uid_t uid;
-	struct lfile *file;
+	int ppid;			/* parent process ID */
+	uid_t uid;			/* user ID */
+	struct lfile *file;		/* open files of process */
 };
 extern struct lproc *Lp, *Lproc;
 
@@ -259,12 +380,12 @@ extern char *Namech;
 extern int Ndev;
 
 # if	defined(HASNLIST)
-#  if	defined(NLIST_TYPE)
-extern struct NLIST_TYPE Nl[];
-#  else	/* !defined(NLIST_TYPE) */
-extern struct nlist Nl[];
-#  endif	/* defined(NLIST_TYPE) */
-# endif
+#  if	!defined(NLIST_TYPE)
+#define	NLIST_TYPE	nlist
+#  endif	/* !defined(NLIST_TYPE) */
+extern struct NLIST_TYPE *Nl;
+extern int Nll;
+# endif	/* defined(HASNLIST) */
 
 extern int Nlproc;
 extern char *Nmlst;
@@ -273,11 +394,16 @@ extern int Npid;
 extern int Npuns;
 extern int Ntype;
 extern int Nuid;
+extern int Nuidexcl;
+extern int Nuidincl;
 
 struct nwad {
+	char *arg;				/* argument */
 	char *proto;				/* protocol */
 	unsigned char a[4];			/* address */
-	int port;
+	int sport;				/* starting port */
+	int eport;				/* ending port */
+	int f;					/* find state */
 	struct nwad *next;			/* forward link */
 };
 extern struct nwad *Nwad;
@@ -310,11 +436,12 @@ extern struct l_dev **Sdev;
 extern int Selall;
 extern int Selflags;
 extern int Setgid;
+extern int Selinet;
 extern int Setuidroot;
 extern struct sfile *Sfile;
-extern int *Spgrp;
-extern int *Spid;
-extern uid_t *Suid;
+extern struct int_lst *Spgrp;
+extern struct int_lst *Spid;
+extern struct seluid *Suid;
 extern char Terminator;
 extern int TmLimit;
 
