@@ -34,7 +34,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: main.c,v 1.19 95/09/19 10:45:55 abe Exp $";
+static char *rcsid = "$Id: main.c,v 1.6 97/10/10 08:27:30 abe Exp $";
 #endif
 
 
@@ -67,10 +67,13 @@ main(argc, argv)
 	int err = 0;
 	int fh = 0;
 	struct lfile *lf;
-	char options[64];
+	struct nwad *np, *npn;
+	char options[128];
 	struct sfile *sfp;
 	struct lproc **slp = NULL;
 	int sp = 0;
+	struct str_lst *str;
+	int version = 0;
 
 	if ((Pn = strrchr(argv[0], '/')) != NULL)
 		Pn++;
@@ -86,44 +89,61 @@ main(argc, argv)
 		Setuidroot = 1;
 	if ((Namech = (char *)malloc(MAXPATHLEN)) == NULL) {
 		(void) fprintf(stderr, "%s: no space for name buffer\n", Pn);
-		exit(1);
+		Exit(1);
 	}
 /*
  * Create option mask.
  */
-	(void) sprintf(options, "?ab%sd:%sF:g:hHi:%sl%snNoOp:Pr:sS:tu:Uw%s",
+	(void) sprintf(options,
+		"?ab%sc:%sd:%sF:g:hi:%sl%snNoOp:Pr:%ssS:tT:u:UvVw%s",
+
+#if	defined(HAS_AFS) && defined(HASAOPT)
+		"A:",
+#else	/* !defined(HAS_AFS) || !defined(HASAOPT) */
+		"",
+#endif	/* defined(HAS_AFS) && defined(HASAOPT) */
 
 #if	defined(HASNCACHE)
-			"C",
+		"C",
 #else	/* !defined(HASNCACHE) */
-			"",
+		"",
 #endif	/* defined(HASNCACHE) */
 
 #if	defined(HASDCACHE)
-			"D:",
+		"D:",
 #else	/* !defined(HASDCACHE) */
-			"",
+		"",
 #endif	/* defined(HASDCACHE) */
 
 #if	defined(HASKOPT)
-			"k:",
+		"k:",
 #else	/* !defined(HASKOPT) */
-			"",
+		"",
 #endif	/* defined(HASKOPT) */
 
 #if	defined(HASMOPT)
-			"c:",
+		"m:",
 #else	/* !defined(HASMOPT) */
-			"",
+		"",
 #endif	/* defined(HASMOPT) */
 
+#if	defined(HASPPID)
+		"R",
+#else	/* !defined(HASPPID) */
+		"",
+#endif	/* defined(HASPPID) */
+
 #if	defined(HASXOPT)
-			"X"
+# if	defined(HASXOPT_ROOT)
+		(Myuid == 0) ? "X" : ""
+# else	/* !defined(HASXOPT_ROOT) */
+		"X"
+# endif	/* defined(HASXOPT_ROOT) */
 #else	/* !defined(HASXOPT) */
-			""
+		""
 #endif	/* defined(HASXOPT) */
 
-			);
+		);
 /*
  * Loop through options.
  */
@@ -137,6 +157,18 @@ main(argc, argv)
 		case 'a':
 			Fand = 1;
 			break;
+
+#if	defined(HAS_AFS) && defined(HASAOPT)
+		case 'A':
+			if (GOv == NULL || *GOv == '-') {
+			    (void) fprintf(stderr,
+				"%s: -A not followed by path\n", Pn);
+			    err = 1;
+			} else
+			    AFSApath = GOv;
+			break;
+#endif	/* defined(HAS_AFS) && defined(HASAOPT) */
+
 		case 'b':
 			Fblock = 1;
 			break;
@@ -152,7 +184,7 @@ main(argc, argv)
 #endif	/* defined(HASNCACHE) */
 
 		case 'd':
-			if (enter_str_lst("d", GOv, &Fdl))
+			if (enter_fd(GOv))
 			    err = 1;
 			break;
 
@@ -174,7 +206,13 @@ main(argc, argv)
 				    Terminator = '\0';
 			    }
 			    for (i = 0; FieldSel[i].nm; i++) {
-				FieldSel[i].n = 1;
+
+#if	!defined(HASPPID)
+				if (FieldSel[i].id == LSOF_FID_PPID)
+				    continue;
+#endif	/* !defined(HASPPID) */
+
+				FieldSel[i].st = 1;
 			    }
 			    Ffield = 1;
 			    break;
@@ -185,8 +223,14 @@ main(argc, argv)
 			}
 			for (; *GOv; GOv++) {
 			    for (i = 0; FieldSel[i].nm; i++) {
+
+#if	!defined(HASPPID)
+				if (FieldSel[i].id == LSOF_FID_PPID)
+				    continue;
+#endif	/* !defined(HASPPID) */
+
 				if (FieldSel[i].id == *GOv) {
-				    FieldSel[i].n = 1;
+				    FieldSel[i].st = 1;
 				    if (i == LSOF_FIX_TERM)
 					Terminator = '\0';
 				    break;
@@ -214,19 +258,29 @@ main(argc, argv)
 		case '?':
 			Fhelp = 1;
 			break;
-		case 'H':
-			Fhost = 0;
-			break;
 		case 'i':
+			if (GOv == NULL || *GOv == '-') {
+			    Fnet = 1;
+			    if (GOv != NULL) {
+				GOx1 = GObk[0];
+				GOx2 = GObk[1];
+			    }
+			    break;
+			}
 			if (enter_network_address(GOv))
 			    err = 1;
 			break;
 
 #if	defined(HASKOPT)
 		case 'k':
-			Nmlst = GOv;
+			if (GOv == NULL || *GOv == '-') {
+			    (void) fprintf(stderr,
+				"%s: -k not followed by path\n", Pn);
+			    err = 1;
+			} else
+			    Nmlst = GOv;
 			break;
-#endif
+#endif	/* defined(HASKOPT) */
 
 		case 'l':
 			Futol = 0;
@@ -234,12 +288,17 @@ main(argc, argv)
 
 #if	defined(HASMOPT)
 		case 'm':
-			Memory = GOv;
+			if (GOv == NULL || *GOv == '-') {
+			    (void) fprintf(stderr,
+				"%s: -m not followed by path\n", Pn);
+			    err = 1;
+			} else
+			    Memory = GOv;
 			break;
 #endif	/* defined(HASMOPT) */
 
 		case 'n':
-			Fnet = 1;
+			Fhost = 0;
 			break;
 		case 'N':
 			Fnfs = 1;
@@ -283,6 +342,13 @@ main(argc, argv)
 			    err = 1;
 			}
 			break;
+
+#if	defined(HASPPID)
+		case 'R':
+			Fppid = 1;
+			break;
+#endif	/* defined(HASPPID) */
+
 		case 's':
 			Fsize = 1;
 			break;
@@ -305,6 +371,42 @@ main(argc, argv)
 		case 't':
 			Fterse = Fwarn = 1;
 			break;
+		case 'T':
+			Ftcptpi = 0;
+			if (!GOv || *GOv == '-') {
+			    if (GOv) {
+				GOx1 = GObk[0];
+				GOx2 = GObk[1];
+			    }
+			    break;
+			}
+			for (; *GOv; GOv++) {
+			    switch (*GOv) {
+
+#if	defined(HASTCPTPIQ)
+			    case 'q':
+				Ftcptpi |= TCPTPI_QUEUES;
+				break;
+#endif	/* defined(HASTCPTPIQ) */
+
+			    case 's':
+				Ftcptpi |= TCPTPI_STATE;
+				break;
+
+#if	defined(HASTCPTPIW)
+			    case 'w':
+				Ftcptpi |= TCPTPI_WINDOWS;
+				break;
+#endif	/* defined(HASTCPTPIW) */
+
+			    default:
+				(void) fprintf(stderr,
+				"%s: unsupported TCP/TPI info selection: %c\n",
+				    Pn, *GOv);
+				err = 1;
+			    }
+			}
+			break;
 		case 'u':
 			if (enter_uid(GOv))
 			    err = 1;
@@ -312,8 +414,20 @@ main(argc, argv)
 		case 'U':
 			Funix = 1;
 			break;
+		case 'v':
+			version = 1;
+			break;
+		case 'V':
+			Fverbose = 1;
+			break;
 		case 'w':
+
+#if	defined(WARNINGSTATE)
+			Fwarn = 0;
+#else	/* !defined(WARNINGSTATE) */
 			Fwarn = 1;
+#endif	/* defined(WARNINGSTATE) */
+
 			break;
 
 #if	defined(HASXOPT)
@@ -339,10 +453,24 @@ main(argc, argv)
 				"%s: -f and -t are mutually exclusive\n", Pn);
 			err++;
 		}
-		FieldSel[LSOF_FIX_PID].n = 1;
+		FieldSel[LSOF_FIX_PID].st = 1;
+	}
+	if (DChelp || err || Fhelp || fh || version)
+		usage(err ? 1 : 0, fh, version);
+/*
+ * Reduce the size of Suid[], if necessary.
+ */
+	if (Suid && Nuid && Nuid < Mxuid) {
+		if ((Suid = (struct seluid *)realloc((MALLOC_P *)Suid,
+				(MALLOC_S)(sizeof(struct seluid) * Nuid)))
+		== (struct seluid *)NULL)
+		{
+			(void) fprintf(stderr, "%s: can't realloc UID table\n",
+				Pn);
+			Exit(1);
+		}
+		Mxuid = Nuid;
 	}
-	if (err || Fhelp || fh)
-		usage(err ? 1 : 0, fh);
 
 /*
  * Compute the selection flags.
@@ -361,7 +489,7 @@ main(argc, argv)
 		Selflags |= SELPGRP;
 	if (Npid)
 		Selflags |= SELPID;
-	if (Nuid)
+	if (Nuid && Nuidincl)
 		Selflags |= SELUID;
 	if (Nwad)
 		Selflags |= SELNA;
@@ -371,26 +499,26 @@ main(argc, argv)
 		if (Fand) {
 			(void) fprintf(stderr,
 				"%s: no select options to AND via -a\n", Pn);
-			usage(1, 0);
+			usage(1, 0, 0);
 		}
 		Selflags = SELALL;
-	} else
+	} else {
+		if (GOx1 >= argc && (Selflags & (SELNA|SELNET)) != 0
+		&&  (Selflags & ~(SELNA|SELNET)) == 0)
+			Selinet = 1;
 		Selall = 0;
-/*
- * Read the mount table unless only network or Unix domain socket files
- * are being displayed, or are part of an ANDed option specification.
- */
-	if ((!Fand && (Selflags & ~(SELNA|SELNET|SELUNX)))
-	||  ( Fand && (Selflags &  (SELNA|SELNET|SELUNX)) == 0)) {
-		if (!readmnt())
-			exit(1);
 	}
+/*
+ * Read the mount table, unless listing only network files (-i option).
+ */
+	if (Selinet == 0 && readmnt() == 0)
+		Exit(1);
 /*
  * Process the file arguments.
  */
 	if (GOx1 < argc) {
 		if (ck_file_arg(GOx1, argc, argv))
-			usage(1, 0);
+			usage(1, 0, 0);
 	}
 /*
  * Do dialect-specific initialization.
@@ -410,12 +538,14 @@ main(argc, argv)
  * If there is a device cache, prepare the device table.
  */
 	if (DCstate)
-		readdev();
+		readdev(0);
 #endif	/* defined(HASDCACHE) */
 
 /*
  * Gather and report process information every RptTm seconds.
  */
+	if (RptTm)
+		CkPasswd = 1;
 	do {
 
 	/*
@@ -441,7 +571,7 @@ main(argc, argv)
 		    (void) fprintf(stderr,
 			"%s: WARNING -- no space for %d sort pointers\n",
 			Pn, Nlproc);
-		    exit(1);
+		    Exit(1);
 		}
 		for (i = 0; i < Nlproc; i++) {
 		    slp[i] = &Lproc[i];
@@ -453,18 +583,8 @@ main(argc, argv)
 		Lp = (Nlproc > 1) ? slp[i] : &Lproc[i];
 		if (Lp->pss)
 		    print_proc();
-		if (RptTm) {
-		    for (Lf = Lp->file; Lf; Lf = lf) {
-			if (Lf->dev_ch)
-			    (void) free((FREE_P *)Lf->dev_ch);
-			if (Lf->nm)
-			    (void) free((FREE_P *)Lf->nm);
-			if (Lf->nma)
-			    (void) free((FREE_P *)Lf->nma);
-			lf = Lf->next;
-			(void) free((FREE_P *)Lf);
-		    }
-		}
+		if (RptTm)
+		    (void) free_lproc(Lp);
 	    }
 	/*
 	 * If a repeat time is set, sleep for the specified time.
@@ -476,19 +596,119 @@ main(argc, argv)
 		} else
 		    puts("=======");
 		(void) fflush(stdout);
+		(void) childx();
 		(void) sleep(RptTm);
 		Hdr = Nlproc = 0;
+		CkPasswd = 1;
 	    }
 	} while (RptTm);
 /*
  * See if all requested information was displayed.  Return zero if it
- * was; one, if not.
+ * was; one, if not.  If -V was specified, report what was not displayed.
  */
-	for (sfp = Sfile; sfp; sfp = sfp->next) {
-		if (sfp->f == 0)
-			return(1);
+	(void) childx();
+	rv = 0;
+	for (str = Cmdl; str; str = str->next) {
+
+	/*
+	 * Check command specifications.
+	 */
+	    if (str->f)
+		continue;
+	    rv = 1;
+	    if (Fverbose)
+		(void) printf("%s: command not located: %s\n", Pn, str->str);
 	}
-	return(0);
+	for (sfp = Sfile; sfp; sfp = sfp->next) {
+
+	/*
+	 * Check file specifications.
+	 */
+	    if (sfp->f)
+		continue;
+	    rv = 1;
+	    if (Fverbose)
+		(void) printf("%s: file not located: %s\n", Pn, sfp->aname);
+	}
+	for (np = Nwad; np; np = npn) {
+
+	/*
+	 * Check Internet address specifications.
+	 *
+	 * If any Internet address derived from the same argument was located,
+	 * consider all derivations located.
+	 */
+	    for (cp = np->arg, i = 1;; np = npn) {
+		if (np->f)
+		    i = 0;
+		if (!(npn = np->next) || cp != npn->arg)
+		    break;
+	    }
+	    rv |= i;
+	    if (i && Fverbose)
+		(void) printf("%s: Internet address not located: %s\n",
+		    Pn, cp ? cp : "(unknown)");
+	}
+	if (Fnet && Fnet < 2) {
+
+	/*
+	 * Report no Internet files located.
+	 */
+	    rv = 1;
+	    if (Fverbose)
+		(void) printf("%s: no Internet files located\n", Pn);
+	}
+	if (Fnfs && Fnfs < 2) {
+
+	/*
+	 * Report no NFS files located.
+	 */
+	    rv = 1;
+	    if (Fverbose)
+		(void) printf("%s: no NFS files located\n", Pn);
+	}
+	for (i = 0; i < Npid; i++) {
+
+	/*
+	 * Check process ID specifications.
+	 */
+	    if (Spid[i].f)
+		continue;
+	    rv = 1;
+	    if (Fverbose)
+		(void) printf("%s: process ID not located: %d\n",
+		    Pn, Spid[i].i);
+	}
+	for (i = 0; i < Npgrp; i++) {
+
+	/*
+	 * Check process group ID specifications.
+	 */
+	    if (Spgrp[i].f)
+		continue;
+	    rv = 1;
+	    if (Fverbose)
+		(void) printf("%s: process group ID not located: %d\n",
+		    Pn, Spgrp[i].i);
+	}
+	for (i = 0; i < Nuid; i++) {
+
+	/*
+	 * Check inclusionary user ID specifications.
+	 */
+	    if (Suid[i].excl || Suid[i].f)
+		continue;
+	    rv = 1;
+	    if (Fverbose) {
+		if (Suid[i].lnm)
+		    (void) printf("%s: login name (UID %lu) not located: %s\n",
+			Pn, (unsigned long)Suid[i].uid, Suid[i].lnm);
+		else
+		    (void) printf("%s: user ID not located: %lu\n", Pn,
+			(unsigned long)Suid[i].uid);
+	    }
+	}
+	Exit(rv);
 }
 
 
