@@ -32,24 +32,157 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: misc.c,v 1.19 95/09/19 10:38:58 abe Exp $";
+static char *rcsid = "$Id: misc.c,v 1.6 97/10/10 08:22:17 abe Exp $";
 #endif
 
 
 #include "lsof.h"
 
+
+/*
+ * Local function prototypes
+ */
+
+_PROTOTYPE(static void closePipes,(void));
 _PROTOTYPE(static int dolstat,(char *path, char *buf, int len));
 _PROTOTYPE(static int doreadlink,(char *path, char *buf, int len));
 _PROTOTYPE(static int doinchild,(int (*fn)(), char *fp, char *rbuf, int rbln));
 
+# if	!defined(GET_MAX_FD)
+/*
+ * This is not strictly a prototype, but GET_MAX_FD is the name of the
+ * function that, in lieu of getdtablesize(), returns the maximum file
+ * descriptor plus one (or file descriptor count).  GET_MAX_FD may be
+ * defined in the dialect's machine.h.  If it is not, the following
+ * selects getdtablesize().
+ */
+
+#define	GET_MAX_FD	getdtablesize
+# endif	/* !defined(GET_MAX_FD) */
+
 #if	defined(HASINTSIGNAL)
 _PROTOTYPE(static int handleint,(int sig));
-#else
+#else	/* !defined(HASINTSIGNAL) */
 _PROTOTYPE(static void handleint,(int sig));
-#endif
+#endif	/* defined(HASINTSIGNAL) */
 
 
+/*
+ * Local variables
+ */
+
+static pid_t Cpid = 0;			/* child PID */
 static jmp_buf Jmp_buf;			/* jump buffer */
+static int Pipes[] =			/* pipes for child process */
+	{ -1, -1, -1, -1 };
+
+
+#if	defined(HASNLIST)
+/*
+ * build-Nl() - build kernel name list table
+ */
+
+static struct drive_Nl *Build_Nl = (struct drive_Nl *)NULL;
+					/* the default Drive_Nl address */
+
+void
+build_Nl(d)
+	struct drive_Nl *d;		/* data to drive the construction */
+{
+	struct drive_Nl *dp;
+	int i, n;
+
+	for (dp = d, n = 0; dp->nn; dp++, n++)
+	    ;
+	if (n < 1) {
+	    (void) fprintf(stderr,
+		"%s: can't calculate kernel name list length\n", Pn);
+	    Exit(1);
+	}
+	if (!(Nl = (struct NLIST_TYPE *)calloc((n + 1),
+					       sizeof(struct NLIST_TYPE))))
+	{
+	    (void) fprintf(stderr,
+		"%s: can't allocate %d bytes to kernel name list structure\n",
+		Pn, (n + 1) * sizeof(struct NLIST_TYPE));
+	    Exit(1);
+	}
+	for (dp = d, i = 0; i < n; dp++, i++) {
+	    Nl[i].NL_NAME = dp->knm;
+	}
+	Nll = (int)((n + 1) * sizeof(struct NLIST_TYPE));
+	Build_Nl = d;
+}
+#endif	/* defined(HASNLIST) */
+
+
+/*
+ * childx() - make child process exit
+ */
+
+void
+childx()
+{
+	if (Cpid > 1) {
+	    if (setjmp(Jmp_buf)) {
+		(void) alarm(0);
+		(void) signal(SIGALRM, SIG_DFL);
+		if (!Fwarn)
+		    (void) fprintf(stderr,
+			"%s: WARNING -- child process %d may be hung.\n",
+			Pn, Cpid);
+	    } else {
+		(void) kill(Cpid, SIGINT);
+		(void) kill(Cpid, SIGKILL);
+		(void) signal(SIGALRM, handleint);
+		(void) alarm(TmLimit);
+		(void) wait(NULL);
+		(void) alarm(0);
+		(void) signal(SIGALRM, SIG_DFL);
+	    }
+	    Cpid = 0;
+	    (void) closePipes();
+	}
+}
+
+
+/*
+ * closePipes() - close open pipe file descriptors
+ */
+
+static void
+closePipes()
+{
+	int i;
+
+	for (i = 0; i < 4; i++) {
+	    if (Pipes[i] >= 0) {
+		(void) close(Pipes[i]);
+		Pipes[i] = -1;
+	    }
+	}
+}
+
+
+
+/*
+ * Exit() - do a clean exit()
+ */
+
+void
+Exit(xv)
+	int xv;				/* exit() value */
+{
+	(void) childx();
+
+#if	defined(HASDCACHE)
+	if (DCrebuilt && !Fwarn)
+	    (void) fprintf(stderr, "%s: WARNING: %s was updated.\n",
+		Pn, DCpath[DCpathX]);
+#endif	/* defined(HASDCACHE) */
+
+	exit(xv);
+}
 
 
 #if	defined(WILLDROPGID)
@@ -64,7 +197,7 @@ dropgid()
 		if (setgid(Mygid) < 0) {
 			(void) fprintf(stderr, "%s: can't setgid(%d): %s\n",
 				Pn, Mygid, strerror(errno));
-			exit(1);
+			Exit(1);
 		}
 		Setgid = 0;
 	}
@@ -72,6 +205,37 @@ dropgid()
 #endif	/* defined(WILLDROPGID) */
 
 
+#if	defined(HASNLIST)
+/*
+ * get_Nl_value() - get Nl value for nickname
+ */
+
+int
+get_Nl_value(nn, d, v)
+	char *nn;			/* nickname of requested entry */
+	struct drive_Nl *d;		/* drive_Nl table that built Nl
+					 * (if NULL, use Build_Nl) */
+	unsigned long *v;		/* returned value (if NULL,
+					 * return nothing) */
+{
+	int i;
+
+	if (!Nl || !Nll)
+	    return(-1);
+	if (!d)
+	    d = Build_Nl;
+	for (i = 0; d->nn; d++, i++) {
+	    if (strcmp(d->nn, nn) == 0) {
+		if (v)
+		    *v = (unsigned long)Nl[i].n_value;
+		return(i);
+	    }
+	}
+	return(-1);
+}
+#endif	/* defined(HASNLIST) */
+
+
 /*
  * is_readable() -- is file readable
  */
@@ -101,15 +265,15 @@ compdev(a1, a2)
 	struct l_dev **p1 = (struct l_dev **)a1;
 	struct l_dev **p2 = (struct l_dev **)a2;
 
-	if ((long)((*p1)->rdev) < (long)((*p2)->rdev))
+	if ((dev_t)((*p1)->rdev) < (dev_t)((*p2)->rdev))
 		return(-1);
-	if ((long)((*p1)->rdev) > (long)((*p2)->rdev))
+	if ((dev_t)((*p1)->rdev) > (dev_t)((*p2)->rdev))
 		return(1);
-	if ((long)((*p1)->inode) < (long)((*p2)->inode))
+	if ((ino_t)((*p1)->inode) < (ino_t)((*p2)->inode))
 		return(-1);
-	if ((long)((*p1)->inode) > (long)((*p2)->inode))
+	if ((ino_t)((*p1)->inode) > (ino_t)((*p2)->inode))
 		return(1);
-	return(0);
+	return(strcmp((*p1)->name, (*p2)->name));
 }
 
 
@@ -125,88 +289,149 @@ doinchild(fn, fp, rbuf, rbln)
 	int rbln;			/* response buffer length */
 {
 	int en, rv;
-	static pid_t cpid;
-	int p[2];
-	static int wf;
 /*
- * Set up to handle an alarm signal.
- *
- * Handle the signal.
+ * Check reply buffer size.
  */
-	cpid = wf = 0;
-	if (setjmp(Jmp_buf)) {
-	    (void) alarm(0);
-	    (void) signal(SIGALRM, SIG_DFL);
-	    if (!Fovhd) {
-		if (wf) {
-		    if (cpid && !Fwarn)
+	if (!Fovhd && rbln > MAXPATHLEN) {
+	    (void) fprintf(stderr,
+		"%s: doinchild error; response buffer too large: %d\n",
+		Pn, rbln);
+	    Exit(1);
+	}
+/*
+ * Set up to handle an alarm signal; handle an alarm signal; build
+ * pipes for exchanging information with a child process; start the
+ * child process; and perform functions in the child process.
+ */
+	if (!Fovhd) {
+	    if (setjmp(Jmp_buf)) {
+
+	    /*
+	     * An alarm signal has been received.
+	     */
+		(void) alarm(0);
+		(void) signal(SIGALRM, SIG_DFL);
+		if (setjmp(Jmp_buf)) {
+		    if (!Fwarn && Cpid)
 			(void) fprintf(stderr,
 			    "%s: WARNING -- child process %d may be hung.\n",
-			    Pn, cpid);
-		} else if (cpid) {
-		    (void) kill(cpid, SIGINT);
-		    (void) kill(cpid, SIGKILL);
-		    wf++;
+			    Pn, Cpid);
+		} else if (Cpid) {
+		    (void) kill(Cpid, SIGINT);
+		    (void) kill(Cpid, SIGKILL);
 		    (void) signal(SIGALRM, handleint);
 		    (void) alarm(TmLimit);
 		    (void) wait(NULL);
 		    (void) alarm(0);
 		    (void) signal(SIGALRM, SIG_DFL);
 		}
-	    }
-	    errno = ETIMEDOUT;
-	    return(-1);
-	}
-	if (!Fovhd) {
-
-	/*
-	 * Create a pipe from which to read the child function's response.
-	 */
-	    if (pipe(p) < 0) {
-		(void) fprintf(stderr, "%s: can't open pipe: %s\n",
-		    Pn, strerror(errno));
-		exit(1);
-	    }
-	/*
-	 * Fork a child to execute the function.
-	 */
-	    if ((cpid = fork()) == 0) {
-		(void) close(p[0]);
-		rv = fn(fp, rbuf, rbln);
-		en = errno;
-		write(p[1], (char *)&rv, sizeof(rv));
-		write(p[1], (char *)&en, sizeof(en));
-		write(p[1], rbuf, rbln);
-		(void) close(p[1]);
-		_exit(0);
-	    }
-	    if (cpid < 0) {
-		(void) fprintf(stderr, "%s: can't fork: %s\n",
-		    Pn, strerror(errno));
-		exit(1);
-	    }
-	/*
-	 * Wait for the response in the parent.
-	 */
-	    (void) close(p[1]);
-	    (void) signal(SIGALRM, handleint);
-	    (void) alarm(TmLimit);
-	    if (read(p[0], (char *)&rv, sizeof(rv)) != sizeof(rv)
-	    ||  read(p[0], (char *)&en, sizeof(en)) != sizeof(en)
-	    ||  read(p[0], rbuf, rbln) != rbln) {
+		Cpid = 0;
+		(void) closePipes();
+		errno = ETIMEDOUT;
+		return(1);
+	    } else if (!Cpid) {
 
 	    /*
-	     * Cannot complete the read from the child.
+	     * Create pipes to exchange function information with a child
+	     * process.
 	     */
-		(void) kill(cpid, SIGINT);
-		(void) kill(cpid, SIGKILL);
-		(void) close(p[0]);
-		wf++;
-		(void) alarm(TmLimit);
-		(void) wait(NULL);
+		if (pipe(Pipes) < 0 || pipe(&Pipes[2]) < 0) {
+		    (void) fprintf(stderr, "%s: can't open pipes: %s\n",
+			Pn, strerror(errno));
+		    Exit(1);
+		}
+	    /*
+	     * Fork a child to execute functions.
+	     */
+		if ((Cpid = fork()) == 0) {
+
+		/*
+		 * Begin the child process.
+		 */
+
+		    int fd, nd, r_al, r_rbln;
+		    char r_arg[MAXPATHLEN], r_rbuf[MAXPATHLEN];
+		    int (*r_fn)();
+		/*
+		 * Close all open file descriptors except Pipes[0] and
+		 * Pipes[3].
+		 */
+		    for (fd = 0, nd = GET_MAX_FD(); fd < nd; fd++) {
+			if (fd == Pipes[0] || fd == Pipes[3])
+			    continue;
+			(void) close(fd);
+			if (fd == Pipes[1])
+			    Pipes[1] = -1;
+			else if (fd == Pipes[2])
+			    Pipes[2] = -1;
+		    }
+		    if (Pipes[1] >= 0) {
+			(void) close(Pipes[1]);
+			Pipes[1] = -1;
+		    }
+		    if (Pipes[2] >= 0) {
+			(void) close(Pipes[2]);
+			Pipes[2] = -1;
+		    }
+		/*
+		 * Read function requests, process them, and return replies.
+		 */
+		    for (;;) {
+			if (read(Pipes[0], (char *)&r_fn, sizeof(r_fn))
+			    != sizeof(r_fn)
+			||  read(Pipes[0], (char *)&r_al, sizeof(int))
+			    != sizeof(int)
+			||  r_al < 1
+			||  r_al > sizeof(r_arg)
+			||  read(Pipes[0], r_arg, r_al) != r_al
+			||  read(Pipes[0], (char *)&r_rbln, sizeof(r_rbln))
+			    != sizeof(r_rbln)
+			||  r_rbln < 1 || r_rbln > sizeof(r_rbuf))
+			    break;
+			rv = r_fn(r_arg, r_rbuf, r_rbln);
+			en = errno;
+			if (write(Pipes[3], (char *)&rv, sizeof(rv))
+			    != sizeof(rv)
+			||  write(Pipes[3], (char *)&en, sizeof(en))
+			    != sizeof(en)
+			||  write(Pipes[3], r_rbuf, r_rbln) != r_rbln)
+			    break;
+		    }
+		    _exit(0);
+		}
+	    /*
+	     * Continue in the parent process to finish the setup.
+	     */
+		if (Cpid < 0) {
+		    (void) fprintf(stderr, "%s: can't fork: %s\n",
+			Pn, strerror(errno));
+		    Exit(1);
+		}
+		(void) close(Pipes[0]);
+		(void) close(Pipes[3]);
+		Pipes[0] = Pipes[3] = -1;
+	    }
+	}
+	if (!Fovhd) {
+	    int len;
+
+	/*
+	 * Send a function to the child and wait for the response.
+	 */
+	    len  = strlen(fp) + 1;
+	    (void) signal(SIGALRM, handleint);
+	    (void) alarm(TmLimit);
+	    if (write(Pipes[1], (char *)&fn, sizeof(fn)) != sizeof(fn)
+	    ||  write(Pipes[1], (char *)&len, sizeof(len)) != sizeof(len)
+	    ||  write(Pipes[1], fp, len) != len
+	    ||  write(Pipes[1], (char *)&rbln, sizeof(rbln)) != sizeof(rbln)
+	    ||  read(Pipes[2], (char *)&rv, sizeof(rv)) != sizeof(rv)
+	    ||  read(Pipes[2], (char *)&en, sizeof(en)) != sizeof(en)
+	    ||  read(Pipes[2], rbuf, rbln) != rbln) {
 		(void) alarm(0);
 		(void) signal(SIGALRM, SIG_DFL);
-		errno = EINTR;
+		(void) childx();
+		errno = ECHILD;
 		return(-1);
 	    }
 	} else {
@@ -223,13 +448,6 @@ doinchild(fn, fp, rbuf, rbln)
  * Function completed, response collected -- complete the operation.
  */
 	(void) alarm(0);
-	if (!Fovhd) {
-	    (void) close(p[0]);
-	    wf++;
-	    (void) alarm(TmLimit);
-	    (void) wait(NULL);
-	    (void) alarm(0);
-	}
 	(void) signal(SIGALRM, SIG_DFL);
 	errno = en;
 	return(rv);
@@ -284,7 +502,7 @@ enter_dev_ch(m)
 	if ((mp = (char *)malloc(len)) == NULL) {
 		(void) fprintf(stderr, "%s: no more dev_ch space at PID %d\n",
 			Pn, Lp->pid);
-		exit(1);
+		Exit(1);
 	}
 	(void) strcpy(mp, m);
 	if (Lf->dev_ch)
@@ -310,7 +528,7 @@ enter_nm(m)
 	if ((mp = (char *)malloc(len)) == NULL) {
 		(void) fprintf(stderr, "%s: no more nm space at PID %d\n",
 			Pn, Lp->pid);
-		exit(1);
+		Exit(1);
 	}
 	(void) strcpy(mp, m);
 	if (Lf->nm)
@@ -364,8 +582,11 @@ is_nw_addr(ia, p)
 			||  u[1] != n->a[1] || u[0] != n->a[0])
 				continue;
 		}
-		if (n->port == -1 || p == n->port)
+		if (n->sport == -1
+		||  (p >= n->sport && p <= n->eport)) {
+			n->f = 1;
 			return(1);
+		}
 	}
 	return(0);
 }
@@ -506,7 +727,7 @@ no_readlink_space:
 
 		(void) fprintf(stderr,
 			"%s: no Readlink string space for %s\n", Pn, abuf);
-		exit(1);
+		Exit(1);
 	}
 	(void) strcpy(s1, abuf);
 	if (++sx > ss) {
@@ -654,27 +875,48 @@ x2dev(s, d)
 	char *s;			/* ASCII string */
 	dev_t *d;			/* device receptacle */
 {
-	char c;
-	short n, nl;
+	char c, *cp, *cp1;
+	int n;
 	dev_t r;
-	unsigned char v;
 
+/*
+ * Skip an optional leading 0x.  Count the number of hex digits up to the end
+ * of the string, or to a space, or to a comma.  Return an error if an unknown
+ * character is encountered.  If the count is larger than (2 * sizeof(dev_t))
+ * -- e.g., because of sign extension -- ignore excess leading hex 0xf digits,
+ * but return an error if an excess leading digit isn't 0xf.
+ */
 	if  (strncasecmp(s, "0x", 2) == 0)
 		s += 2;
-	for (nl = (sizeof(dev_t) * 2), n = r = v = 0; *s; s++) {
-		if (isdigit(*s))
-			v = (unsigned char)(*s - '0');
-		else if (isalpha(*s) && (c = toupper(*s)) >= 'A' && c <= 'F')
-			v = (unsigned char)(c - 'A' + 10);
-		else
+	for (cp = s, n = 0; *cp; cp++, n++) {
+		if (isdigit(*cp)
+		||  (isalpha(*cp) && (c = toupper(*cp)) >= 'A' && c <= 'F'))
+			continue;
+		if (*cp == ' ' || *cp == ',')
 			break;
-		if (++n > nl)
-			return(NULL);
-		r = (r << 4) | (v & 0xf);
+		return((char *)NULL);
 	}
-	if (n && ( ! *s || isspace(*s) || *s == ',')) {
-		*d = r;
-		return(s);
+	if (!n)
+		return((char *)NULL);
+	if (n > (2 * sizeof(dev_t))) {
+		cp1 = s;
+		s += (n - (2 * sizeof(dev_t)));
+		while (cp1 < s) {
+			if (*cp1 != 'f' && *cp1 != 'F')
+				return((char *)NULL);
+			cp1++;
+		}
 	}
-	return(NULL);
+/*
+ * Assemble the validated hex digits of the device number, starting at a point
+ * in the string relevant to sizeof(dev_t).
+ */
+	for (r = 0; s < cp; s++) {
+		r = (r << 4)
+		  | ((isdigit(*s) ? (unsigned char)(*s - '0')
+			          : (unsigned char)(toupper(*s) - 'A' + 10))
+		  		  & 0xf);
+	}
+	*d = r;
+	return(s);
 }
