@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: print.c,v 1.19 95/08/22 13:06:05 abe Exp $";
+static char *rcsid = "$Id: print.c,v 1.5 97/09/23 09:13:13 abe Exp $";
 #endif
 
 
@@ -40,15 +40,20 @@ static char *rcsid = "$Id: print.c,v 1.19 95/08/22 13:06:05 abe Exp $";
 
 #if	defined(EMPTY)
 #undef	EMPTY
-#endif
+#endif	/* defined(EMPTY) */
 
 #include <utmp.h>
 
 struct utmp dutmp;			/* dummy entry to get login name size */
 
-#define HCINC	64			/* host cache size increase chunk */
+#define HCINC		64		/* host cache size increase chunk */
 #define LNML sizeof(dutmp.ut_name)
-#define PCINC	512			/* port cache size increase chunk */
+#define PORTHASHBUCKETS	128		/* port hash bucket count
+					 * !!MUST BE A POWER OF 2!! */
+#define	PORTTABTHRESH	10		/* threshold at which we will switch
+					 * from using getservbyport() to
+					 * getservent() -- see lkup_port()
+					 * and fill_porttab() */
 
 struct hostcache {
 	unsigned char addr[4];
@@ -58,22 +63,19 @@ struct hostcache {
 struct porttab {
 	int port;
 	char *name;
+	struct porttab *next;
 };
 
-struct uidcache {
-	uid_t u;
-	char nm[LNML+1];
-};
 
-static int Npt[2] = { 0, 0};		/* TCP and UDP port entry counts */
-static int Nuc = 0;			/* number of entries in UID cache */
-static struct porttab *Pt[2] =		/* TCP and UDP port tables */
-	{ NULL, NULL };
-static int Pts = 0;			/* port tables initialization status */
-static struct uidcache *Uc = NULL;	/* UID cache */
-static int Ucnp = 0;			/* number of processes that determined
-					 * UID cache size */
-static unsigned Ucmask;			/* UID cache address mask */
+static struct porttab **Pth[2] = { NULL, NULL };
+						/* port hash buckets: Pth[0]
+						   for TCP, Pth[1] for UDP */
+#define HASHPORT(p)	(((((int)(p)) * 31415) >> 3) & (PORTHASHBUCKETS - 1))
+
+
+_PROTOTYPE(static void fill_porttab,(void));
+_PROTOTYPE(static char *lkup_port,(int p, int pr));
+_PROTOTYPE(static int printinaddr,(void));
 
 
 /*
@@ -91,6 +93,74 @@ endnm()
 }
 
 
+/*
+ * fill_porttab() -- fill the port table with a getservent() scan
+ */
+
+static void
+fill_porttab()
+{
+	int h, p, pr;
+	MALLOC_S nl;
+	char *nm;
+	struct porttab *pt;
+	struct servent *se;
+
+	(void) endservent();
+/*
+ * Scan the services data base for TCP and UDP entries that have a non-null
+ * name associated with them.
+ */
+	(void) setservent(1);
+	while ((se = getservent())) {
+	    if (!se->s_name || !se->s_proto)
+		continue;
+	    if (strcasecmp(se->s_proto, "TCP") == 0)
+		pr = 0;
+	    else if (strcasecmp(se->s_proto, "UDP") == 0)
+		pr = 1;
+	    else
+		continue;
+	    if (!(nl = (MALLOC_S)strlen(se->s_name)))
+		continue;
+	    p = ntohs(se->s_port);
+	/*
+	 * See if a port->service entry is already cached for this port and
+	 * prototcol.  If it is, leave it alone.
+	 */
+	    h = HASHPORT(p);
+	    for (pt = Pth[pr][h]; pt; pt = pt->next) {
+		if (pt->port == p)
+		    break;
+	    }
+	    if (pt)
+		continue;
+	/*
+	 * Add a new entry to the cache for this port and protocol.
+	 */
+	    if (!(pt = (struct porttab *)malloc(sizeof(struct porttab)))) {
+		(void) fprintf(stderr,
+		    "%s: can't allocate porttab entry for port %d:\n",
+		    Pn, p, se->s_name);
+		Exit(1);
+	    }
+	    nl++;
+	    if (!(nm = (char *)malloc(nl))) {
+		(void) fprintf(stderr,
+		    "%s: can't allocate %d bytes for port %d name: %s\n",
+		    Pn, nl, p, se->s_name);
+		Exit(1);
+	    }
+	    (void) strcpy(nm, se->s_name);
+	    pt->name = nm;
+	    pt->port = p;
+	    pt->next = Pth[pr][h];
+	    Pth[pr][h] = pt;
+	}
+	(void) endservent();
+}
+
+
 /*
  * gethostnm() - get host name
  */
@@ -119,7 +189,7 @@ gethostnm(ia)
 		return(hc[i].name);
 	}
 /*
- * If -H has been specified, construct a dot-form address.  Otherwise,
+ * If -n has been specified, construct a dot-form address.  Otherwise,
  * look up host name by address.  If that fails, or if there is no name
  * in the returned hostent structure, construct a dot-form version of the
  * address.
@@ -137,7 +207,7 @@ gethostnm(ia)
 	if ((np = (char *)malloc((MALLOC_S)(strlen(hn) + 1))) == NULL) {
 		(void) fprintf(stderr, "%s: no space for host name: %s\n",
 			Pn, hn);
-		exit(1);
+		Exit(1);
 	}
 	(void) strcpy(np, hn);
 /*
@@ -154,7 +224,7 @@ gethostnm(ia)
 		if (hc == NULL) {
 			(void) fprintf(stderr, "%s: no space for host cache\n",
 				Pn);
-			exit(1);
+			Exit(1);
 		}
 	}
 	hc[hcx].addr[0] = u[0];
@@ -167,209 +237,212 @@ gethostnm(ia)
 
 
 /*
- * iporttab() - initialize port table
+ * lkup_port() - look up port for protocol
  */
 
-void
-iporttab()
+static char *
+lkup_port(p, pr)
+	int p;				/* port number */
+	int pr;				/* protocol index: 0 = tcp, 1 = udp */
 {
-	int i[2];
-	char *np;
-	short p, ty;
-	struct servent *se;
-
-	if (Pts || !Fport)
-		return;
-	i[0] = i[1] = 0;
-/*
- * Look up port name.
- */
-	(void) setservent(1);
-	while ((se = getservent()) != NULL) {
-	    if (se->s_name == NULL)
-		continue;
-	    p = ntohs(se->s_port);
-	/*
-	 * Determine protocol type -- TCP or UDP.  Ignore others.
-	 */
-	    if (strcasecmp(se->s_proto, "TCP") == 0)
-		ty = 0;
-	    else if (strcasecmp(se->s_proto, "UDP") == 0)
-		ty = 1;
-	    else
-		continue;
-	/*
-	 * Make a copy of the service name.
-	 */
-	    if ((np = (char *)malloc(strlen(se->s_name) + 1)) == NULL) {
-		(void) fprintf(stderr,
-		    "%s: no space for %s port %d cache name: %s\n", Pn,
-		    ty ? "UDP" : "TCP", p, se->s_name);
-		(void) endservent();
-		exit(1);
-	    }
-	    (void) strcpy(np, se->s_name);
-	/*
-	 * Allocate the port table in PCINC chunks.
-	 */
-	    if (i[ty] >= Npt[ty]) {
-		Npt[ty] += PCINC;
-		if (Pt[ty] == NULL)
-		    Pt[ty] = (struct porttab *)malloc(
-				(MALLOC_S)(sizeof(struct porttab) * Npt[ty]));
-		else
-		    Pt[ty] = (struct porttab *)realloc((MALLOC_P *)Pt[ty],
-				(MALLOC_S)(sizeof(struct porttab) * Npt[ty]));
-		if (Pt[ty] == NULL) {
-		    (void) fprintf(stderr,
-			"%s: no space for %s port table\n", Pn,
-			ty ? "UDP" : "TCP");
-		    (void) endservent();
-		    exit(1);
-		}
-	    }
-	/*
-	 * Add the service name to the port table.
-	 */
-	    Pt[ty][i[ty]].port = p;
-	    Pt[ty][i[ty]].name = np;
-	    i[ty]++;
-	}
-/*
- * Reduce the port table sizes to the sizes of their contents.
- */
-	(void) endservent();
-	for (ty = 0; ty < 2; ty++) {
-	    if (Pt[ty] == NULL)
-		continue;
-	    if ((Pt[ty] = (struct porttab *)realloc((MALLOC_P *)Pt[ty],
-			  (MALLOC_S)(sizeof(struct porttab) * i[ty])))
-	    == NULL) {
-		(void) fprintf(stderr,
-		    "%s: can't reduce %s port table size\n",
-		    ty ? "UDP" : "TCP", Pn);
-		exit(1);
-	    }
-	    Npt[ty] = i[ty];
-	}
-	Pts = 1;
-}
-
-
-/*
- * iuidcache() - initialize UID cache
- */
-
-void
-iuidcache(np)
-	int np;				/* number of processes */
-{
-	int i;
+	static int gsbp = 0;
+	int h;
 	MALLOC_S len;
-	unsigned shft;
+	char *nm, pb[128], *pn;
+	struct porttab *pt;
+	static int ptf = 0;
+	struct servent *s;
+/*
+ * If the hash buckets haven't been allocated, do so now.
+ */
+	if (!Pth[0]) {
+	    for (h = 0; h < 2; h++) {
+		if (!(Pth[h] = (struct porttab **)calloc(PORTHASHBUCKETS,
+				sizeof(struct porttab *))))
+		{
+		    (void) fprintf(stderr,
+		      "%s: can't allocate %d bytes for %s port hash buckets\n",
+		      Pn, (2 * (PORTHASHBUCKETS * sizeof(struct porttab *))),
+		      (h == 0) ? "TCP" : "UDP");
+		    Exit(1);
+		}
+	    }
+	}
+/*
+ * Don't do any lookups if the -P option has been specified.  In that case,
+ * just cache the %d conversion of the port number.
+ */
+	if (Fport) {
 
-	if (!Futol)
-		return;
+	/*
+	 * Hash the port and see if its name has been cached.
+	 */
+	    h = HASHPORT(p);
+	    for (;;) {
+		for (pt = Pth[pr][h]; pt; pt = pt->next) {
+		    if (pt->port == p)
+			return(pt->name);
+		}
+	    /*
+	     * The port has no cache entry.  If the port table hasn't already
+	     * been filled by a getservent() scan of the services data base,
+	     * and if the count of getservbyport() calls has reached its
+	     * tolerance limit (PORTTABTHRESH), call fill_porttab() to do the
+	     * getservent() scan and look for the port in the cache once more.
+	     */
+		if (ptf || gsbp < PORTTABTHRESH)
+		    break;
+		(void) fill_porttab();
+		ptf++;
+	    }
+	}
 /*
- * Set the first estimate of the cache size to 512, then adjust the size so
- * that it's a power of 2 and larger than the number of processes.
+ * Allocate a new porttab entry.
  */
-	for (i = Nuc, Nuc = 512, shft = 9; Nuc < np; shft++) {
-		Nuc += Nuc;
+	if (!(pt = (struct porttab *)malloc(sizeof(struct porttab)))) {
+	    (void) fprintf(stderr,
+		"%s: can't allocate porttab entry for port %d\n", Pn, p);
+	    Exit(1);
 	}
-	Ucmask = (1 << shft) - 1;
-	if (i >= Nuc)
-		return;
-	len = (MALLOC_S)(Nuc * sizeof(struct uidcache));
 /*
- * Allocate or reallocate space for the cache and set its new entries empty.
+ * If the port cache has been filled via getservent(), then this lookup has
+ * failed completely; enter a %d cache entry for it.
+ *
+ * If the port cache hasn't been filled via getservent(), look up the port's
+ * service name via getservbyport().  Count the getservbyport() calls.
+ *
+ * Don't do a getservbyport() call if the -P option has been specified; just
+ * enter a %d conversion of the port number.
  */
-	if (Uc)
-		Uc = (struct uidcache *)realloc(Uc, len);
-	else
-		Uc = (struct uidcache *)malloc(len);
-	if (Uc == NULL) {
-		(void) fprintf(stderr, "%s: no space for UID cache\n", Pn);
-		exit(1);
+	pn = (char *)NULL;
+	if (Fport && !ptf && p) {
+	    gsbp++;
+	    if ((s = getservbyport(htons(p), (pr == 0) ? "tcp" : "udp")))
+		pn = s->s_name;
 	}
-	while (i < Nuc) {
-		Uc[i++].nm[0] = '\0';
+	if (!pn || !strlen(pn)) {
+	    if (p) {
+		(void) sprintf(pb, "%d", p);
+		pn = pb;
+	    } else
+		pn = "*";
 	}
+/*
+ * Allocate space for the name; copy it to the porttab entry; and link the
+ * porttab entry to its hash bucket.
+ *
+ * Return a pointer to the name.
+ */
+	len = strlen(pn) + 1;
+	if (!(nm = malloc(len))) {
+	    (void) fprintf(stderr,
+		"%s: can't allocate %d bytes for port name: %s\n",
+		Pn, len, pn);
+	    Exit(1);
+	}
+	(void) strcpy(nm, pn);
+	pt->name = nm;
+	pt->port = p;
+	pt->next = Pth[pr][h];
+	Pth[pr][h] = pt;
+	return(nm);
 }
 
 
 /*
- * printinaddr() - print Internet address
+ * printinaddr() - print Internet addresses
  */
 
-void
-printinaddr(ia, p)
-	struct in_addr *ia;		/* Internet address */
-	int p;				/* port */
+static int
+printinaddr()
 {
-	char pbuf[16];
-	int i, len, nl, ty;
-	char *host = NULL;
-	char *port = NULL;
-	char *np;
-	unsigned char *u;
+	int i, j, len, ty;
+	char *host, *port;
+	int nl = MAXPATHLEN - 1;
+	char *np = Namech;
+	char pbuf[32];
+/*
+ * Process local network address first.  If there's a foreign address,
+ * separate it from the local address with "->".
+ */
+	for (i = 0, *np = '\0'; i < 2; i++) {
+	    if (!Lf->li[i].as)
+		continue;
+	    host = port = (char *)NULL;
+	    if (i) {
 
-	if ((Selflags & SELNA) && Nwad && is_nw_addr(ia, p))
-		Lf->sf |= SELNA;
-	if (ia->s_addr == INADDR_ANY)
-		host = "*";
-	else
-		host = gethostnm(ia);
-	if (p > 0) {
+	    /*
+	     * If this is the foreign address, insert the separator.
+	     */
+		if (nl < 2)
+
+addr_too_long:
+
+		    {
+			(void) fputs("network addresses too long", stdout);
+			return(1);
+		    }
+		(void) strcpy(np, "->");
+		np += 2;
+		nl -= 2;
+	    }
+	/*
+	 * Convert the address to a host name.
+	 */
+	    if (Lf->li[i].ia.s_addr == INADDR_ANY)
+		host ="*";
+	    else
+		host = gethostnm(&Lf->li[i].ia);
+	/*
+	 * Process the port number.
+	 */
+	    if (Lf->li[i].p > 0) {
 		if (Fport) {
-			if (strcasecmp(Lf->iproto, "TCP") == 0)
-				ty = 0;
-			else if (strcasecmp(Lf->iproto, "UDP") == 0)
-				ty = 1;
-			else
-				ty = 2;
-			if (ty < 2) {
-				if (! Pts)
-					iporttab();
-				for (i = 0; i < Npt[ty]; i++) {
-					if (p == Pt[ty][i].port) {
-						port = Pt[ty][i].name;
-						break;
-					}
-				}
-			}
+
+		/*
+		 * If converting port numbers to service names, check the
+		 * protocol.
+		 */
+		    if (strcasecmp(Lf->iproto, "TCP") == 0)
+			port = lkup_port(Lf->li[i].p, 0);
+		    else if (strcasecmp(Lf->iproto, "UDP") == 0)
+			port = lkup_port(Lf->li[i].p, 1);
 		}
-		if (port == NULL) {
-			(void) sprintf(pbuf, "%d", p);
-			port = pbuf;
+		if (!port) {
+		    (void) sprintf(pbuf, "%d", Lf->li[i].p);
+		    port = pbuf;
 		}
-	} else if (p == 0)
-		port = "0";
-	np = endnm();
-	nl = MAXPATHLEN - 1 - (np - Namech);
-	if ((len = strlen(host)) > nl)
-		len = nl;
-	if (len) {
-		(void) strncpy(np, host, len);
-		np += len;
-		*np = '\0';
-		nl -= len;
-	}
-	if (port) {
-		if (nl >= 1) {
-			*np = ':';
-			np++;
-			*np = '\0';
-			nl--;
-		}
-		if ((len = strlen(port)) > nl)
-			len = nl;
+	    } else if (Lf->li[i].p == 0)
+		port = "*";
+	/*
+	 * Enter the host name.
+	 */
+	    if (host) {
+		if ((len = strlen(host)) > nl)
+		    goto addr_too_long;
 		if (len) {
-			(void) strncpy(np, port, len);
-			np += len;
-			*np = '\0';
+		    (void) strcpy(np, host);
+		    np += len;
+		    nl -= len;
 		}
+	    }
+	/*
+	 * Enter the port number, preceded by a colon.
+	 */
+	    if (port) {
+		if (((len = strlen(port)) + 1) >= nl)
+		    goto addr_too_long;
+		*np++ = ':';
+		(void) strcpy(np, port);
+		np += len;
+		nl -= (len + 1);
+	    }
 	}
+	if (Namech) {
+	    (void) fputs(Namech, stdout);
+	    return(1);
+	}
+	return(0);
 }
 
 
@@ -506,7 +579,8 @@ printname(nl)
 
 #if	defined(HASNCACHE)
 	char buf[MAXPATHLEN];
-	char *cp;
+	char *cp, *cp1;
+	int fp;
 #endif	/* defined(HASNCACHE) */
 
 	int ps = 0;
@@ -516,35 +590,49 @@ printname(nl)
 	/*
 	 * Print the name characters, if there are some.
 	 */
-		if (nl && !Lf->nma) {
-			(void) puts(Lf->nm);
-			return;
-		}
-		(void) fputs(Lf->nm, stdout);
-		ps++;
+	    (void) fputs(Lf->nm, stdout);
+	    ps++;
+	    if (!Lf->li[0].as && !Lf->li[1].as)
 		goto print_nma;
 	}
-/*
- * If this is a character device and it has a name, print it.
- */
-	if (Lf->is_chr_dev && printchdevname(&Lf->dev, Lf->nma ? 0 : nl)) {
-		if (!Lf->nma)
-			return;
+	if (Lf->li[0].as || Lf->li[1].as) {
+	    if (ps)
+		putchar(' ');
+	/*
+	 * If the file has Internet addresses, print them.
+	 */
+	    if (printinaddr())
 		ps++;
-		goto print_nma;
+	    goto print_nma;
 	}
+	if (Lf->ntype == N_CHR && Lf->dev_def && printchdevname(&Lf->dev, 0)) {
+
+	/*
+	 * If this is a character device and it has a name, print it.
+	 */
+	    ps++;
+	    goto print_nma;
+	}
+
+#if	defined(HASBLKDEV)
+	if (Lf->ntype == N_BLK && printbdevname(&Lf->dev, 0)) {
+
+	/*
+	 * If this is a block device and it has a name, print it.
+	 */
+	    ps++;
+	    goto print_nma;
+	}
+#endif	/* defined(HASBLKDEV) */
+
 	if (Lf->is_com) {
 
 	/*
 	 * If this is a common node, print that fact.
 	 */
-		if (nl && !Lf->nma) {
-			(void) puts("COMMON: ");
-			return;
-		}
-		(void) fputs("COMMON: ", stdout);
-		ps++;
-		goto print_nma;
+	    (void) fputs("COMMON: ", stdout);
+	    ps++;
+	    goto print_nma;
 	}
 	if (Lf->fsdir || Lf->fsdev) {
 
@@ -552,38 +640,39 @@ printname(nl)
 	 * Print the file system directory name, device name, and
 	 * possible path name components.
 	 */
-		if (Lf->fsdir) {
-			(void) fputs(Lf->fsdir, stdout);
-			ps++;
-		}
+	    if (Lf->fsdir) {
+		(void) fputs(Lf->fsdir, stdout);
+		ps++;
+	    }
 
 #if	defined(HASNCACHE)
-		if (Lf->na
-		&&  (cp = ncache_lookup(buf, sizeof(buf)))) {
-			(void) fputs(" -- ", stdout);
-			if (nl && !Lf->nma) {
-				(void) puts(cp);
-				return;
-			}
-			(void) fputs(cp, stdout);
-			ps++;
-			goto print_nma;
-		}
+	    if (Lf->na
+	    &&  (cp = ncache_lookup(buf, sizeof(buf), &fp))) {
+		if (*cp == '\0')
+		    goto print_nma;
+		if (fp) {
+		    if (*cp != '/') {
+			cp1 = strrchr(Lf->fsdir, '/');
+			if (cp1 == (char *)NULL ||  *(cp1 + 1) != '\0')
+			    putchar('/');
+		    }
+		} else
+		    (void) fputs(" -- ", stdout);
+		(void) fputs(cp, stdout);
+		ps++;
+		goto print_nma;
+	    }
 #endif	/* defined(HASNCACHE) */
 
-		if (Lf->fsdev) {
-			if (Lf->fsdir)
-				(void) fputs(" (", stdout);
-			else
-				(void) putchar('(');
-			(void) fputs(Lf->fsdev, stdout);
-			if (nl && !Lf->nma) {
-				(void) puts(")");
-				return;
-			}
-			(void) putchar(')');
-			ps++;
-		}
+	    if (Lf->fsdev) {
+		if (Lf->fsdir)
+		    (void) fputs(" (", stdout);
+		else
+		    (void) putchar('(');
+		(void) fputs(Lf->fsdev, stdout);
+		(void) putchar(')');
+		ps++;
+	    }
 	}
 /*
  * Print the NAME column addition, if there is one.  If there isn't
@@ -593,14 +682,33 @@ printname(nl)
 print_nma:
 
 	if (Lf->nma) {
-		if (ps)
-			putchar(' ');
-		if (nl)
-			(void) puts(Lf->nma);
-		else
-			(void) fputs(Lf->nma, stdout);
-	} else if (nl)
-		putchar('\n');
+	    if (ps)
+		putchar(' ');
+	    (void) fputs(Lf->nma, stdout);
+	    ps++;
+	}
+/*
+ * If this file has TCP/IP state information, print it.
+ */
+	if (!Ffield && Ftcptpi
+	&&  (Lf->lts.type >= 0
+
+#if	defined(HASTCPTPIQ)
+	||   Lf->lts.rqs || Lf->lts.sqs
+#endif	/* defined(HASTCPTPIQ) */
+
+#if	defined(HASTCPTPIW)
+	||   Lf->lts.rws || Lf->lts.wws
+#endif	/* defined(HASTCPTPIW) */
+
+	)) {
+	    if (ps)
+		putchar(' ');
+	    (void) print_tcptpi(1);
+	    return;
+	}
+	if (nl)
+	    putchar('\n');
 }
 
 
@@ -643,70 +751,116 @@ printuid(u, ty)
 					 * then: *ty = 0 = login name
 					 *	     = 1 = UID number */
 {
-	long i;
-	int j = 0;
-	int k;
+	int i;
 	struct passwd *pw;
+	struct stat sb;
+	static struct stat sbs;
+	static struct uidcache {
+	    uid_t u;
+	    char nm[LNML+1];
+	    struct uidcache *next;
+	} **uc = (struct uidcache **)NULL;
+	struct uidcache *up, *upn;
 	static char user[USERPRTL+1];
 
-	if (Futol && Uc) {
+	if (Futol) {
+	    if (CkPasswd) {
 
-	/*
-	 * If UID to login name conversion is in effect, and if there is
-	 * a UID cache, search the cache.
-	 */
-		for (i = (long)u; j < Nuc; j++) {
-			k = (int)(((i * 31415L) >> 7) & Ucmask);
-			if (Uc[k].nm[0] == '\0') {
-
-			/*
-			 * Create a new cache entry.
-			 */
-
-#if	defined(HASPWSTAYOPEN)
-				pw_stay_open();
-#endif
-
-				if ((pw = getpwuid(u)) == NULL) {
-				    if (!Fwarn) {
-					(void) fprintf(stderr,
-					    "%s: no pwd entry for UID %d\n",
-					    Pn, u);
-				    }
-				    break;
-				}
-				(void) strncpy(Uc[k].nm, pw->pw_name, LNML);
-				Uc[k].nm[LNML] = '\0';
-				Uc[k].u = (uid_t)u;
-			}
-			if (Uc[k].u == (uid_t)u) {
-
-			/*
-			 * Use an existing cache entry.
-			 */
-				if (ty)
-					*ty = 0;
-				return(Uc[k].nm);
-			}
-		/*
-		 * Reprobe after a cache collision.
-		 */
-			i = ((i + 1) >= Nuc) ? 0 : i + 1;
+	    /*
+	     * Get the mtime and ctime of /etc/passwd, as required.
+	     */
+		if (stat("/etc/passwd", &sb) != 0) {
+		    (void) fprintf(stderr, "%s: can't stat(/etc/passwd): %s\n",
+			Pn, strerror(errno));
+		    Exit(1);
 		}
-	}
-/*
- * Issue a warning if at the reprobe limit.
- */
-	if (Nuc && j >= Nuc && !Fwarn) {
-		(void) fprintf(stderr, "%s: at reprobe limit for UID %d\n",
+	    }
+	/*
+	 * Define the UID cache, if necessary.
+	 */
+	    if (!uc) {
+		if ((uc = (struct uidcache **)calloc(UIDCACHEL,
+						sizeof(struct uidcache *)))
+		== (struct uidcache **)NULL) {
+		    (void) fprintf(stderr,
+			"%s: no space for %d byte UID cache hash buckets\n",
+			Pn, UIDCACHEL * (sizeof(struct uidcache *)));
+		    Exit(1);
+		}
+		if (CkPasswd) {
+		    sbs = sb;
+		    CkPasswd = 0;
+		}
+	    }
+	/*
+	 * If it's time to check /etc/passwd and if its the mtime/ctime has
+	 * changed, destroy the existing UID cache.
+	 */
+	    if (CkPasswd) {
+		if (sbs.st_mtime != sb.st_mtime || sbs.st_ctime != sb.st_ctime)
+		{
+		    for (i = 0; i < UIDCACHEL; i++) {
+			if ((up = uc[i]) != (struct uidcache *)NULL) {
+			    do {
+				upn = up->next;
+				(void) free((FREE_P *)up);
+			    } while ((up = upn) != (struct uidcache *)NULL);
+			    uc[i] = (struct uidcache *)NULL;
+			}
+		    }
+		    sbs = sb;
+	        }
+		CkPasswd = 0;
+	    }
+	/*
+	 * Search the UID cache.
+	 */
+	    i = (int)((((unsigned long)u * 31415L) >> 7) & (UIDCACHEL - 1));
+	    for (up = uc[i]; up; up = up->next) {
+		if (up->u == (uid_t)u) {
+		    if (ty)
+			*ty = 0;
+		    return(up->nm);
+		}
+	    }
+	/*
+	 * The UID is not in the cache.
+	 *
+	 * Look up the login name from the UID for a new cache entry.
+	 */
+	    if ((pw = getpwuid((uid_t)u)) == (struct passwd *)NULL) {
+		if (!Fwarn) {
+		    (void) fprintf(stderr, "%s: no pwd entry for UID %d\n",
 			Pn, u);
+		}
+	    } else {
+
+	    /*
+	     * Allocate and fill a new cache entry.  Link it to its hash bucket.
+	     */
+		if ((upn = (struct uidcache *)malloc(sizeof(struct uidcache)))
+		== (struct uidcache *)NULL) {
+		    (void) fprintf(stderr,
+			"%s: no space for UID cache entry for: %lu, %s)\n",
+			(unsigned long)u, pw->pw_name);
+		    Exit(1);
+		}
+		(void) strncpy(upn->nm, pw->pw_name, LNML);
+		upn->nm[LNML] = '\0';
+		upn->u = (uid_t)u;
+		upn->next = uc[i];
+		uc[i] = upn;
+		if (ty)
+		    *ty = 0;
+		return(upn->nm);
+	    }
 	}
 /*
  * Produce a numeric conversion of the UID.
  */
 	(void) sprintf(user, "%*u", USERPRTL, u);
 	if (ty)
-		*ty = 1;
+	    *ty = 1;
 	return(user);
 }
 
