@@ -30,6 +30,11 @@
  */
 
 
+/*
+ * $Id: proto.h,v 1.6 97/10/10 08:21:45 abe Exp $
+ */
+
+
 /*
  * The _PROTOTYPE macro provides strict ANSI C prototypes if __STDC__
  * is defined, and old-style K&R prototypes otherwise.
@@ -45,8 +50,10 @@
 
 
 _PROTOTYPE(extern void alloc_lfile,(char *nm, int num));
-_PROTOTYPE(extern void alloc_lproc,(int pid, int pgrp, UID_ARG uid, char *cmd, int pss, int sf));
+_PROTOTYPE(extern void alloc_lproc,(int pid, int pgrp, int ppid, UID_ARG uid, char *cmd, int pss, int sf));
+_PROTOTYPE(extern void childx,(void));
 _PROTOTYPE(extern int ck_file_arg,(int i, int ac, char *av[]));
+_PROTOTYPE(extern void clr_devtab,(void));
 _PROTOTYPE(extern int compdev,(COMP_P *a1, COMP_P *a2));
 _PROTOTYPE(extern int comppid,(COMP_P *a1, COMP_P *a2));
 
@@ -56,45 +63,55 @@ _PROTOTYPE(extern void dropgid,(void));
 
 _PROTOTYPE(extern char *endnm,(void));
 _PROTOTYPE(extern void enter_dev_ch,(char *m));
+_PROTOTYPE(extern int enter_fd,(char *f));
 _PROTOTYPE(extern int enter_network_address,(char *na));
 _PROTOTYPE(extern int enter_id,(enum IDType ty, char *p));
 _PROTOTYPE(extern void enter_nm,(char *m));
 _PROTOTYPE(extern int enter_str_lst,(char *opt, char *s, struct str_lst **lp));
 _PROTOTYPE(extern int enter_uid,(char *u));
 _PROTOTYPE(extern int examine_lproc,(void));
+_PROTOTYPE(extern void ent_inaddr,(struct in_addr *la, int lp, struct in_addr *fa, int fp));
+_PROTOTYPE(extern void Exit,(int xv));
 _PROTOTYPE(extern void find_ch_ino,(void));
+_PROTOTYPE(extern void free_lproc,(struct lproc *lp));
 _PROTOTYPE(extern void gather_proc_info,(void));
 _PROTOTYPE(extern char *gethostnm,(struct in_addr *ia));
 _PROTOTYPE(extern void initialize,(void));
-_PROTOTYPE(extern void iporttab,(void));
 _PROTOTYPE(extern int is_cmd_excl,(char *cmd, short *pss, short *sf));
 _PROTOTYPE(extern int is_file_sel,(struct lfile *lf));
 _PROTOTYPE(extern int is_nw_addr,(struct in_addr *ia, int p));
 _PROTOTYPE(extern int is_proc_excl,(int pid, int pgrp, UID_ARG uid, short *pss, short *sf));
 _PROTOTYPE(extern int is_readable,(char *path, int msg));
-_PROTOTYPE(extern void iuidcache,(int np));
 _PROTOTYPE(extern int kread,(KA_T addr, char *buf, READLEN_T len));
 _PROTOTYPE(extern void link_lfile,(void));
-_PROTOTYPE(extern struct l_dev *lkupdev,(dev_t *dev, int i));
+_PROTOTYPE(extern struct l_dev *lkupdev,(dev_t *dev, int i, int r));
 _PROTOTYPE(extern int main,(int argc, char *argv[]));
 _PROTOTYPE(extern int printchdevname,(dev_t *rdev, int f));
 _PROTOTYPE(extern void print_file,(void));
-_PROTOTYPE(extern void printinaddr,(struct in_addr *ia, int p));
 _PROTOTYPE(extern void printiproto,(int p));
 _PROTOTYPE(extern void printname,(int nl));
 _PROTOTYPE(extern void print_proc,(void));
 _PROTOTYPE(extern void printrawaddr,(struct sockaddr *sa));
+_PROTOTYPE(extern void print_tcptpi,(int nl));
 _PROTOTYPE(extern char *printuid,(UID_ARG u, int *ty));
 _PROTOTYPE(extern void printunkaf,(int fam));
 _PROTOTYPE(extern void process_file,(struct file *fp));
 _PROTOTYPE(extern void process_node,(caddr_t f));
 _PROTOTYPE(extern char *Readlink,(char *arg));
-_PROTOTYPE(extern void readdev,(void));
+_PROTOTYPE(extern void readdev,(int skip));
 _PROTOTYPE(extern int readmnt,(void));
+_PROTOTYPE(extern void rereaddev,(void));
 _PROTOTYPE(extern int statsafely,(char *path, struct stat *buf));
-_PROTOTYPE(extern void usage,(int xv, int fh));
+_PROTOTYPE(extern void usage,(int xv, int fh, int version));
+_PROTOTYPE(extern int vfy_dev,(struct l_dev *dp));
 _PROTOTYPE(extern char *x2dev,(char *s, dev_t *d));
 
+#if	defined(HASBLKDEV)
+_PROTOTYPE(extern void find_bl_ino,(void));
+_PROTOTYPE(extern struct l_dev *lkupbdev,(dev_t *dev, int i, int r));
+_PROTOTYPE(extern int printbdevname,(dev_t *rdev, int f));
+#endif	/* defined(HASBLKDEV) */
+
 #if	defined(HASCDRNODE)
 _PROTOTYPE(extern int readcdrnode,(caddr_t ca, struct cdrnode *c));
 #endif	/* defined(HASCDRNODE) */
@@ -104,7 +121,7 @@ _PROTOTYPE(extern void alloc_dcache,(void));
 _PROTOTYPE(extern void crc,(char *b, int l, unsigned *s));
 _PROTOTYPE(extern void crdbld,(void));
 _PROTOTYPE(extern int ctrl_dcache,(char *p));
-_PROTOTYPE(extern int dcpath,(int rw));
+_PROTOTYPE(extern int dcpath,(int rw, int npw));
 _PROTOTYPE(extern int open_dcache,(int m, int r, struct stat *sb));
 _PROTOTYPE(extern int read_dcache,(void));
 _PROTOTYPE(extern int wr2DCfd,(char *b, unsigned *c));
@@ -119,6 +136,11 @@ _PROTOTYPE(extern int readfifonode,(caddr_t fa, struct fifonode *f));
 _PROTOTYPE(extern int readgnode,(caddr_t ga, struct gnode *g));
 #endif	/* defined(HASGNODE) */
 
+#if	defined(HASNLIST)
+_PROTOTYPE(extern void build_Nl,(struct drive_Nl *d));
+_PROTOTYPE(extern int get_Nl_value,(char *nn, struct drive_Nl *d, unsigned long *v));
+#endif	/* defined(HASNLIST) */
+
 #if	defined(HASHSNODE)
 _PROTOTYPE(extern int readhsnode,(caddr_t ha, struct hsnode *h));
 #endif	/* defined(HASHSNODE) */
@@ -128,7 +150,8 @@ _PROTOTYPE(extern int readinode,(struct inode *ia, struct inode *i));
 #endif	/* defined(HASINODE) */
 
 #if	defined(HASNCACHE)
-_PROTOTYPE(extern char *ncache_lookup,(char *buf, int blen));
+_PROTOTYPE(extern void ncache_load,(void));
+_PROTOTYPE(extern char *ncache_lookup,(char *buf, int blen, int *fp));
 #endif	/* defined(HASNCACHE) */
 
 #if	defined(HASPIPENODE)
